@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -122,6 +122,9 @@ typedef mysqlshdk::utils::Enum_set<Cluster_global_status,
                                    Cluster_global_status::UNKNOWN>
     Cluster_global_status_mask;
 
+typedef mysqlshdk::utils::Enum_set<Cluster_status, Cluster_status::UNKNOWN>
+    Cluster_status_mask;
+
 // Note that this structure may be initialized using initializer
 // lists, so the order of the fields is very important
 struct Function_availability {
@@ -133,6 +136,7 @@ struct Function_availability {
   // Defines the global state in which the operation is allowed
   // Empty indicates the operation is not allowed for instances in a cluster set
   Cluster_global_status_mask cluster_set_state = {};
+  bool allowed_on_fenced = false;
 };
 
 struct Cluster_check_info {
@@ -244,11 +248,13 @@ constexpr const char kOnlyUpgradeRequired[] = "onlyUpgradeRequired";
 constexpr const char kUpdate[] = "update";
 constexpr const char kUpdateTopologyMode[] = "updateTopologyMode";
 constexpr const char kUpgradeCommProtocol[] = "upgradeCommProtocol";
+constexpr const char kUpdateViewChangeUuid[] = "updateViewChangeUuid";
 constexpr const char kAll[] = "all";
 constexpr const char kTimeout[] = "timeout";
 constexpr const char kInvalidateErrorInstances[] = "invalidateErrorInstances";
 constexpr const char kClusterSetReplicationSslMode[] =
     "clusterSetReplicationSslMode";
+constexpr const char kReplicationAllowedHost[] = "replicationAllowedHost";
 
 constexpr const int k_group_replication_members_limit = 9;
 
@@ -264,7 +270,7 @@ struct Option_availability {
  * Map of the global Cluster configuration options of the AdminAPI
  * <sysvar, name>
  */
-const std::map<std::string, std::string> k_global_options{
+const std::map<std::string, std::string> k_global_cluster_options{
     {kGroupName, kGrGroupName}, {kMemberSslMode, kGrMemberSslMode}};
 
 /**
@@ -497,11 +503,13 @@ std::unique_ptr<mysqlshdk::config::Config> create_server_config(
  *
  * @param cfg pointer to a config object
  * @param handler_name name to be given to the config_file handler
+ * @param server_uuid of the instance the file belongs to
  * @param mycnf_path path to the mycnf_path
  * @param output_mycnf_path path to the output_cnf_path
  */
 void add_config_file_handler(mysqlshdk::config::Config *cfg,
-                             const std::string handler_name,
+                             const std::string &handler_name,
+                             const std::string &server_uuid,
                              const std::string &mycnf_path,
                              const std::string &output_mycnf_path);
 
