@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2018, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -71,9 +71,11 @@ namespace checks {
  *
  * @param  instance target instance for the validation. Must be authenticated
  *         with an account with SELECT access to all schemas.
+ * @param  skip_check_tables_pk true to skip checking tables without PKs
  * @return true if no issues found.
  */
-bool validate_schemas(const mysqlshdk::mysql::IInstance &instance) {
+bool validate_schemas(const mysqlshdk::mysql::IInstance &instance,
+                      bool skip_check_tables_pk) {
   bool ok = true;
   std::string k_gr_compliance_skip_schemas =
       "('mysql', 'sys', 'performance_schema', 'information_schema')";
@@ -104,10 +106,11 @@ bool validate_schemas(const mysqlshdk::mysql::IInstance &instance) {
         row = result->fetch_one();
       }
       tables.append("\n");
-      console->println(tables);
+      console->print_info(tables);
     }
   }
-  {
+
+  if (!skip_check_tables_pk) {
     std::shared_ptr<mysqlshdk::db::IResult> result = instance.query(
         "SELECT t.table_schema, t.table_name "
         "FROM information_schema.tables t "
@@ -140,9 +143,10 @@ bool validate_schemas(const mysqlshdk::mysql::IInstance &instance) {
         row = result->fetch_one();
       }
       tables.append("\n");
-      console->println(tables);
+      console->print_info(tables);
     }
   }
+
   if (!ok) {
     console->print_info(
         "Group Replication requires tables to use InnoDB and "
@@ -249,7 +253,7 @@ void validate_host_address(const mysqlshdk::mysql::IInstance &instance,
   }
 
   if (verbose > 0) {
-    console->println();
+    console->print_info();
     console->print_info(
         "This instance reports its own address as " +
         mysqlshdk::textui::bold(instance.get_canonical_address()));
@@ -360,7 +364,7 @@ std::vector<mysqlshdk::mysql::Invalid_config> validate_configuration(
   *restart_needed = false;
   *mycnf_change_needed = false;
   *sysvar_change_needed = false;
-  bool log_bin_wrong = false;
+  bool log_bin_present = false;
 
   if (invalid_cfs_vec.empty()) {
     (*map_val)["status"] = shcore::Value("ok");
@@ -371,13 +375,16 @@ std::vector<mysqlshdk::mysql::Invalid_config> validate_configuration(
     for (mysqlshdk::mysql::Invalid_config &cfg : invalid_cfs_vec) {
       shcore::Value::Map_type_ref error(new shcore::Value::Map_type());
       std::string action;
-      if (!log_bin_wrong && cfg.var_name == "log_bin" &&
+      if (!log_bin_present &&
+          (cfg.var_name == "log_bin" || cfg.var_name == "skip_log_bin" ||
+           cfg.var_name == "disable_log_bin") &&
           cfg.types.is_set(mysqlshdk::mysql::Config_type::CONFIG)) {
-        log_bin_wrong = true;
+        log_bin_present = true;
       }
 
-      if (cfg.types.is_set(mysqlshdk::mysql::Config_type::CONFIG) &&
-          cfg.types.is_set(mysqlshdk::mysql::Config_type::SERVER)) {
+      if (log_bin_present ||
+          (cfg.types.is_set(mysqlshdk::mysql::Config_type::CONFIG) &&
+           cfg.types.is_set(mysqlshdk::mysql::Config_type::SERVER))) {
         *mycnf_change_needed = true;
         if (cfg.restart) {
           *restart_needed = true;
@@ -423,7 +430,7 @@ std::vector<mysqlshdk::mysql::Invalid_config> validate_configuration(
   if (!invalid_cfs_vec.empty()) {
     auto console = mysqlsh::current_console();
 
-    console->println();
+    console->print_info();
     print_validation_results(check_result.as_map(), true);
     if (*restart_needed) {
       // if we have to change some read only variables, print a message to the
@@ -431,7 +438,7 @@ std::vector<mysqlshdk::mysql::Invalid_config> validate_configuration(
       std::string base_msg =
           "Some variables need to be changed, but cannot be done dynamically "
           "on the server";
-      if (log_bin_wrong && *mycnf_change_needed) {
+      if (log_bin_present && *mycnf_change_needed) {
         base_msg += ": an option file is required";
       } else {
         if (*mycnf_change_needed) {
@@ -599,8 +606,8 @@ size_t check_illegal_async_channels(
                   ch.channel_name) != allowed_channels.end())
       illegal_channels--;
     else
-      console->print_note("Found unexpected replication channel " +
-                          ch.channel_name + "at " + instance.descr());
+      console->print_note("Found unexpected replication channel '" +
+                          ch.channel_name + "' at " + instance.descr());
   }
 
   return illegal_channels;
