@@ -64,6 +64,13 @@ const Cluster_global_status_mask kClusterGlobalStateAnyOk =
         .set(Cluster_global_status::OK_NOT_CONSISTENT)
         .set(Cluster_global_status::OK_MISCONFIGURED);
 
+const Cluster_global_status_mask kClusterGlobalStateAnyOkorNotOk =
+    Cluster_global_status_mask(Cluster_global_status::OK)
+        .set(Cluster_global_status::OK_NOT_REPLICATING)
+        .set(Cluster_global_status::OK_NOT_CONSISTENT)
+        .set(Cluster_global_status::OK_MISCONFIGURED)
+        .set(Cluster_global_status::NOT_OK);
+
 const Cluster_global_status_mask kClusterGlobalStateAny =
     Cluster_global_status_mask::any();
 
@@ -90,6 +97,7 @@ class Precondition_checker {
   FRIEND_TEST(Preconditions, check_quorum_state_preconditions);
   FRIEND_TEST(Preconditions, check_cluster_set_preconditions_errors);
   FRIEND_TEST(Preconditions, check_cluster_set_preconditions);
+  FRIEND_TEST(Preconditions, check_cluster_fenced_preconditions);
 #endif
 
   void check_session() const;
@@ -105,9 +113,11 @@ class Precondition_checker {
       mysqlsh::dba::ReplicationQuorum::State allowed_states) const;
   void check_cluster_set_preconditions(
       mysqlsh::dba::Cluster_global_status_mask allowed_states);
+  void check_cluster_fenced_preconditions(bool allowed_on_fenced);
 
  protected:
   virtual Cluster_global_status get_cluster_global_state();
+  virtual Cluster_status get_cluster_status();
 
   std::shared_ptr<MetadataStorage> m_metadata;
   std::shared_ptr<Instance> m_group_server;
