@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -27,9 +27,11 @@
 #include <string>
 #include <utility>
 #include <vector>
+#include "modules/adminapi/common/dba_errors.h"
 #include "modules/adminapi/common/metadata_storage.h"
 #include "mysqld_error.h"
 #include "mysqlshdk/libs/mysql/group_replication.h"
+#include "mysqlshdk/libs/utils/debug.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/utils_net.h"
 #include "mysqlshdk/libs/utils/utils_sqlstring.h"
@@ -90,8 +92,8 @@ TargetType::Type get_gr_instance_type(
     // mysql_server_uuid too.
     query =
         "SELECT COUNT(*) FROM mysql_innodb_cluster_metadata.instances WHERE "
-        "CAST(mysql_server_uuid AS char ascii) = CAST(@@server_uuid AS char "
-        "ascii)";
+        "CAST(mysql_server_uuid AS CHAR CHARACTER SET ASCII) = "
+        "CAST(@@server_uuid AS CHAR CHARACTER SET ASCII)";
 
     try {
       auto result = instance.query(query);
@@ -141,8 +143,8 @@ TargetType::Type get_gr_instance_type(
       // mysql_server_uuid too.
       query =
           "SELECT COUNT(*) FROM mysql_innodb_cluster_metadata.instances WHERE "
-          "CAST(mysql_server_uuid AS char ascii) = CAST(@@server_uuid AS char "
-          "ascii)";
+          "CAST(mysql_server_uuid AS CHAR CHARACTER SET ASCII) = "
+          "CAST(@@server_uuid AS CHAR CHARACTER SET ASCII)";
 
       result = instance.query(query);
       row = result->fetch_one();
@@ -234,57 +236,6 @@ Cluster_check_info get_replication_group_state(
   return ret_val;
 }
 
-/*
- * Retrieves the list of group replication
- * addresses of the peer instances of
- * instance_host
- */
-std::vector<std::string> get_peer_seeds(
-    const mysqlshdk::mysql::IInstance &instance,
-    const std::string &instance_host) {
-  std::vector<std::string> ret_val;
-  shcore::sqlstring query = shcore::sqlstring(
-      "SELECT JSON_UNQUOTE(addresses->'$.grLocal') "
-      "FROM mysql_innodb_cluster_metadata.instances "
-      "WHERE addresses->'$.mysqlClassic' <> ? "
-      "AND cluster_id IN (SELECT cluster_id "
-      "FROM mysql_innodb_cluster_metadata.instances "
-      "WHERE addresses->'$.mysqlClassic' = ?)",
-      0);
-
-  query << instance_host.c_str();
-  query << instance_host.c_str();
-  query.done();
-
-  try {
-    // Get current GR group seeds value
-    auto result =
-        instance.query("SELECT @@global.group_replication_group_seeds");
-    auto row = result->fetch_one();
-    std::string group_seeds_str = row->get_string(0);
-    if (!group_seeds_str.empty())
-      ret_val = shcore::split_string(group_seeds_str, ",");
-
-    // Get the list of known seeds from the metadata.
-    result = instance.query(query);
-    row = result->fetch_one();
-    while (row) {
-      std::string seed = row->get_string(0);
-
-      if (std::find(ret_val.begin(), ret_val.end(), seed) == ret_val.end()) {
-        // Only add seed from metadata if not already in the GR group seeds.
-        ret_val.push_back(seed);
-      }
-      row = result->fetch_one();
-    }
-  } catch (const shcore::Error &error) {
-    log_warning("Unable to retrieve group seeds for instance '%s': %s",
-                instance_host.c_str(), error.what());
-  }
-
-  return ret_val;
-}
-
 std::vector<std::pair<std::string, int>> get_open_sessions(
     const mysqlshdk::mysql::IInstance &instance) {
   std::vector<std::pair<std::string, int>> ret;
@@ -312,7 +263,7 @@ std::vector<std::pair<std::string, int>> get_open_sessions(
 }
 
 Instance_metadata query_instance_info(
-    const mysqlshdk::mysql::IInstance &instance) {
+    const mysqlshdk::mysql::IInstance &instance, bool validate_gr_endpoint) {
   int xport = -1;
   std::string local_gr_address;
 
@@ -333,7 +284,24 @@ Instance_metadata query_instance_info(
     local_gr_address =
         instance.get_sysvar_string("group_replication_local_address")
             .get_safe("");
+
+    if (validate_gr_endpoint && local_gr_address.empty())
+      throw shcore::Exception::error_with_code(
+          "", "group_replication_local_address is empty",
+          SHERR_DBA_EMPTY_LOCAL_ADDRESS);
+
+    // Set debug trap to simulate exception / not be able to read string
+    DBUG_EXECUTE_IF("dba_instance_query_gr_local_address", {
+      throw shcore::Exception::error_with_code(
+          "", "group_replication_local_address is empty",
+          SHERR_DBA_EMPTY_LOCAL_ADDRESS);
+    });
+
   } catch (const std::exception &e) {
+    log_error(
+        "Unable to read Group Replication local address on instance '%s': %s",
+        instance.descr().c_str(), e.what());
+    throw;
   }
 
   Instance_metadata instance_def;
