@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -45,6 +45,8 @@ namespace dba {
 /**
  * \ingroup AdminAPI
  * $(CLUSTER_BRIEF)
+ *
+ * $(CLUSTER_DETAIL)
  */
 class Cluster : public std::enable_shared_from_this<Cluster>,
                 public shcore::Cpp_object_bridge {
@@ -68,7 +70,7 @@ class Cluster : public std::enable_shared_from_this<Cluster>,
   String status(Dictionary options);
   Undefined switchToSinglePrimaryMode(InstanceDef instance);
   Undefined switchToMultiPrimaryMode();
-  Undefined setPrimaryInstance(InstanceDef instance);
+  Undefined setPrimaryInstance(InstanceDef instance, Dictionary options);
   String options(Dictionary options);
   Undefined setOption(String option, String value);
   Undefined setInstanceOption(InstanceDef instance, String option,
@@ -76,6 +78,9 @@ class Cluster : public std::enable_shared_from_this<Cluster>,
   Boolean removeRouterMetadata(RouterDef routerDef);
   Undefined setupAdminAccount(String user, Dictionary options);
   Undefined setupRouterAccount(String user, Dictionary options);
+  Undefined fenceAllTraffic();
+  Undefined fenceWrites();
+  Undefined unfenceWrites();
 #elif DOXYGEN_PY
   str name;  //!< $(CLUSTER_GETNAME_BRIEF)
   None add_instance(InstanceDef instance, dict options);
@@ -95,13 +100,16 @@ class Cluster : public std::enable_shared_from_this<Cluster>,
   str status(dict options);
   None switch_to_single_primary_mode(InstanceDef instance);
   None switch_to_multi_primary_mode();
-  None set_primary_instance(InstanceDef instance);
+  None set_primary_instance(InstanceDef instance, dict options);
   str options(dict options);
   None set_option(str option, str value);
   None set_instance_option(InstanceDef instance, str option, str value);
   bool remove_router_metadata(RouterDef routerDef);
   None setup_admin_account(str user, dict options);
   None setup_router_account(str user, dict options);
+  None fence_all_traffic();
+  None fence_writes();
+  None unfence_writes();
 #endif
 
   explicit Cluster(const std::shared_ptr<Cluster_impl> &impl);
@@ -122,9 +130,6 @@ class Cluster : public std::enable_shared_from_this<Cluster>,
 
   void assert_valid(const std::string &option_name) const;
 
-  /**
-   * Mark the cluster as invalid (e.g., dissolved).
-   */
   void invalidate() { m_invalidated = true; }
 
   void add_instance(const Connection_options &instance_def,
@@ -168,7 +173,10 @@ class Cluster : public std::enable_shared_from_this<Cluster>,
       const Connection_options &instance_def = Connection_options());
 
   void switch_to_multi_primary_mode(void);
-  void set_primary_instance(const Connection_options &instance_def);
+  void set_primary_instance(
+      const Connection_options &instance_def,
+      const shcore::Option_pack_ref<cluster::Set_primary_instance_options>
+          &options = {});
 
   shcore::Value options(
       const shcore::Option_pack_ref<cluster::Options_options> &options);
@@ -179,6 +187,10 @@ class Cluster : public std::enable_shared_from_this<Cluster>,
                            const std::string &option,
                            const shcore::Value &value);
 
+  void fence_all_traffic();
+  void fence_writes();
+  void unfence_writes();
+
   // ClusterSet
   shcore::Value create_cluster_set(
       const std::string &domain_name,
