@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2014, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -132,6 +132,7 @@ void Session::init() {
   expose("releaseSavepoint", &Session::release_savepoint, "name");
   expose("rollbackTo", &Session::rollback_to, "name");
   add_property("uri", "getUri");
+  add_property("sshUri", "getSshUri");
   add_property("defaultSchema", "getDefaultSchema");
   add_property("currentSchema", "getCurrentSchema");
 
@@ -963,6 +964,10 @@ static ::xcl::Argument_value convert(const shcore::Value &value) {
       return xcl::Argument_value(convert_array(value.as_array()));
     case shcore::Null:
       return xcl::Argument_value();
+    case shcore::Binary:
+      return xcl::Argument_value(value.get_string(),
+                                 xcl::Argument_value::String_type::k_octets);
+      break;
     case shcore::Object:
     case shcore::MapRef:
     case shcore::Function:
@@ -1045,6 +1050,13 @@ std::shared_ptr<shcore::Object_bridge> Session::create(
                                      mysqlshdk::db::kSslModeRequired);
   }
 
+  // before creating a normal session we need to establish ssh if needed:
+  co.set_default_data();
+  auto &ssh = co.get_ssh_options_handle(mysqlshdk::db::k_default_mysql_x_port);
+  if (ssh.has_data()) {
+    mysqlshdk::ssh::current_ssh_manager()->create_tunnel(&ssh);
+  }
+
   const auto session = std::make_shared<Session>();
   session->connect(co);
 
@@ -1152,6 +1164,30 @@ Retrieves the URI for the current session.
  *
  * $(SESSION_GETURI)
  */
+#if DOXYGEN_JS
+String Session::getSshUri() {}
+#elif DOXYGEN_PY
+str Session::get_ssh_uri() {}
+#endif
+
+REGISTER_HELP_PROPERTY(sshUri, Session);
+REGISTER_HELP(SESSION_SSHURI_BRIEF, "${SESSION_GETSSHURI_BRIEF}");
+REGISTER_HELP_FUNCTION(getSshUri, Session);
+REGISTER_HELP_FUNCTION_TEXT(SESSION_GETSSHURI, R"*(
+Retrieves the SSH URI for the current session.
+
+@return A string representing the SSH connection data.
+)*");
+/**
+ * $(SESSION_GETSSHURI_BRIEF)
+ *
+ * $(SESSION_GETSSHURI)
+ */
+#if DOXYGEN_JS
+String Session::getSshUri() {}
+#elif DOXYGEN_PY
+str Session::get_ssh_uri() {}
+#endif
 
 Value Session::get_member(const std::string &prop) const {
   Value ret_val;
@@ -1165,6 +1201,8 @@ Value Session::get_member(const std::string &prop) const {
 
   if (prop == "uri") {
     ret_val = shcore::Value(uri());
+  } else if (prop == "sshUri") {
+    ret_val = shcore::Value(ssh_uri());
   } else if (prop == "defaultSchema") {
     if (_connection_options.has_schema()) {
       ret_val =
@@ -1286,4 +1324,4 @@ shcore::Dictionary_t Session::_fetch_notice() {
     return tmp;
   }
   return nullptr;
-}
\ No newline at end of file
+}
