@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2018, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -31,6 +31,7 @@
 #include "mysqlshdk/include/scripting/type_info/custom.h"
 #include "mysqlshdk/include/shellcore/console.h"
 #include "mysqlshdk/include/shellcore/utils_help.h"
+#include "mysqlshdk/libs/utils/threads.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 
@@ -183,6 +184,14 @@ const std::map<std::string, shcore::Value_type> kTypeMapping = {
     {"dictionary", shcore::Map}, {"function", shcore::Function},
 };
 
+// The help for the extension objects is registered in the corresponding
+// context depending on where the plugins are being loaded.
+shcore::Keyword_location help_context() {
+  return mysqlshdk::utils::in_main_thread()
+             ? shcore::Keyword_location::GLOBAL_CTX
+             : shcore::Keyword_location::LOCAL_CTX;
+}
+
 }  // namespace
 
 Parameter_definition::Parameter_definition()
@@ -347,7 +356,8 @@ void Extensible_object::register_member(
   }
 }
 
-void Extensible_object::set_registered(const std::string &name) {
+void Extensible_object::set_registered(const std::string &name,
+                                       bool do_register_help) {
   // The caller must ensure this function is not called for a registered
   // object
   assert(!m_registered);
@@ -357,20 +367,29 @@ void Extensible_object::set_registered(const std::string &name) {
     m_name = name.empty() ? m_definition->name : name;
     m_registered = true;
 
-    if (parent)
-      m_qualified_name = parent->m_qualified_name + "." + m_name;
-    else
-      m_qualified_name = m_name;
+    if (m_qualified_name.empty()) {
+      if (parent)
+        m_qualified_name = parent->m_qualified_name + "." + m_name;
+      else
+        m_qualified_name = m_name;
+    }
 
     // When parent is shellapi then it is a global object
-    register_help(m_definition, !parent);
+    if (do_register_help) {
+      register_help(m_definition, !parent);
 
-    for (const auto &mdef : m_property_definition) register_property_help(mdef);
+      for (const auto &mdef : m_property_definition)
+        register_property_help(mdef);
 
-    for (const auto &fdef : m_function_definition) register_function_help(fdef);
+      for (const auto &fdef : m_function_definition)
+        register_function_help(fdef);
+    } else {
+      // Enables help for the topic if it exists
+      enable_help();
+    }
 
     for (auto &child : m_children) {
-      child.second->set_registered();
+      child.second->set_registered("", do_register_help);
     }
   }
 }
@@ -763,7 +782,8 @@ void Extensible_object::register_help(const std::string &brief,
     auto help = shcore::Help_registry::get();
 
     // Attempts getting the fully qualified object
-    shcore::Help_topic *topic = help->get_topic(m_qualified_name, true);
+    shcore::Help_topic *topic = help->get_topic(
+        m_qualified_name, true, shcore::Topic_mask::any(), true);
 
     if (topic) {
       mysqlsh::current_console()->print_warning("Help for " + m_qualified_name +
@@ -781,19 +801,17 @@ void Extensible_object::register_help(const std::string &brief,
       auto type = is_global ? shcore::Topic_type::GLOBAL_OBJECT
                             : shcore::Topic_type::OBJECT;
       help->add_help_topic(m_name, type, m_qualified_name, parent, mask,
-                           shcore::Keyword_location::LOCAL_CTX);
+                           help_context(), true);
 
       // Now registers the object brief, parameters and details
       auto prefix = shcore::str_upper(m_qualified_name);
 
-      help->add_help(prefix, "BRIEF", brief,
-                     shcore::Keyword_location::LOCAL_CTX);
+      help->add_help(prefix, "BRIEF", brief, help_context());
       help->add_help(prefix, "DETAIL", &m_detail_sequence, details,
-                     shcore::Keyword_location::LOCAL_CTX);
+                     help_context());
 
       if (is_global)
-        help->add_help(prefix, "GLOBAL_BRIEF", brief,
-                       shcore::Keyword_location::LOCAL_CTX);
+        help->add_help(prefix, "GLOBAL_BRIEF", brief, help_context());
     }
   } else {
     m_definition.reset(new Member_definition("", brief, details));
@@ -809,22 +827,20 @@ void Extensible_object::register_property_help(
 
   // Creates the help topic for the object
   help->add_help_topic(def->name, shcore::Topic_type::PROPERTY, def->name,
-                       m_qualified_name, mask,
-                       shcore::Keyword_location::LOCAL_CTX);
+                       m_qualified_name, mask, help_context(), true);
 
   // Now registers the object brief, parameters and details
   auto object = shcore::str_upper(m_name);
   auto prefix = object + "_" + shcore::str_upper(def->name);
   if (!def->brief.empty()) {
-    help->add_help(prefix, "BRIEF", def->brief,
-                   shcore::Keyword_location::LOCAL_CTX);
+    help->add_help(prefix, "BRIEF", def->brief, help_context());
   }
 
   size_t member_sequence = 0;
 
   if (!def->details.empty()) {
     help->add_help(prefix, "DETAIL", &member_sequence, def->details,
-                   shcore::Keyword_location::LOCAL_CTX);
+                   help_context());
   }
 }
 
@@ -846,17 +862,14 @@ void Extensible_object::register_function_help(
 
     // Creates the help topic for the function
     help->add_help_topic(name, shcore::Topic_type::FUNCTION, names[0],
-                         m_qualified_name, mask,
-                         shcore::Keyword_location::LOCAL_CTX);
+                         m_qualified_name, mask, help_context(), true);
 
     // Now registers the function brief, parameters and details
     auto prefix = shcore::str_upper(m_name + "_" + names[0]);
     if (!brief.empty()) help->add_help(prefix, "BRIEF", brief);
-    help->add_help(prefix, "PARAM", params,
-                   shcore::Keyword_location::LOCAL_CTX);
-    help->add_help(prefix, "DETAIL", details,
-                   shcore::Keyword_location::LOCAL_CTX);
-    help->add_help(prefix, examples, shcore::Keyword_location::LOCAL_CTX);
+    help->add_help(prefix, "PARAM", params, help_context());
+    help->add_help(prefix, "DETAIL", details, help_context());
+    help->add_help(prefix, examples, help_context());
   } else {
     topic->set_enabled(true);
   }
@@ -1080,7 +1093,8 @@ struct Compare_help_topic_and_string {
 }  // namespace
 
 void Extensible_object::enable_help() {
-  auto topic = shcore::Help_registry::get()->get_topic(m_qualified_name, true);
+  auto topic = shcore::Help_registry::get()->get_topic(
+      m_qualified_name, true, shcore::Topic_mask::any(), true);
 
   if (topic && !topic->is_enabled()) {
     topic->set_enabled(true);
@@ -1105,7 +1119,8 @@ void Extensible_object::enable_help() {
 }
 
 void Extensible_object::disable_help() {
-  auto topic = shcore::Help_registry::get()->get_topic(m_qualified_name, true);
+  auto topic = shcore::Help_registry::get()->get_topic(
+      m_qualified_name, true, shcore::Topic_mask::any(), true);
 
   if (topic && topic->is_enabled()) {
     topic->set_enabled(false);
