@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -23,6 +23,7 @@
 #include "mysqlshdk/libs/db/connection_options.h"
 #include <algorithm>
 #include <cassert>
+#include "mysqlshdk/include/shellcore/shell_options.h"
 #include "mysqlshdk/libs/db/uri_encoder.h"
 #include "mysqlshdk/libs/db/uri_parser.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
@@ -49,33 +50,21 @@ std::string to_string(Transport_type type) {
   return "Unknown";
 }
 
-namespace {
-static constexpr const char *fixed_str_list[] = {kHost,   kSocket, kScheme,
-                                                 kSchema, kUser,   kPassword};
-}
-
 Connection_options::Connection_options(Comparison_mode mode)
-    : m_mode(mode),
-      m_options(m_mode, "connection"),
+    : IConnection("connection", mode),
       m_ssl_options(m_mode),
+      m_ssh_options(m_mode),
       m_extra_options(m_mode),
       m_enable_connection_attributes(true),
       m_connection_attributes(m_mode) {
-  for (auto o : fixed_str_list) m_options.set(o, nullptr, Set_mode::CREATE);
+  for (auto o : {kSocket}) m_options.set(o, nullptr, Set_mode::CREATE);
 }
 
 Connection_options::Connection_options(const std::string &uri,
                                        Comparison_mode mode)
-    : m_mode(mode),
-      m_options(m_mode, "connection"),
-      m_ssl_options(m_mode),
-      m_extra_options(m_mode),
-      m_enable_connection_attributes(true),
-      m_connection_attributes(m_mode) {
-  for (auto o : fixed_str_list) m_options.set(o, nullptr, Set_mode::CREATE);
-
+    : Connection_options(mode) {
   try {
-    uri::Uri_parser parser;
+    uri::Uri_parser parser(false);
     *this = parser.parse(uri, m_options.get_mode());
   } catch (const std::invalid_argument &error) {
     std::string msg = "Invalid URI: ";
@@ -95,36 +84,80 @@ void Connection_options::set_login_options_from(
     set_password(options.get_password());
   }
 
-  m_ssl_options.clear_cert();
-  m_ssl_options.clear_key();
-  // SSL client certificate options are login options
-  const Ssl_options &ssl = options.get_ssl_options();
-  if (ssl.has_cert()) m_ssl_options.set_cert(ssl.get_cert());
-  if (ssl.has_key()) m_ssl_options.set_key(ssl.get_key());
-}
+  clear_mfa_passwords();
+  if (options.has_mfa_passwords()) {
+    set_mfa_passwords(options.get_mfa_passwords());
+  }
 
-void Connection_options::set_ssl_connection_options_from(
-    const Ssl_options &options) {
-  Ssl_options orig(m_ssl_options);
-  // Copy all SSL options
-  m_ssl_options = options;
-  // Restore the client certificate options
+  m_ssl_options.clear_ca();
+  m_ssl_options.clear_capath();
   m_ssl_options.clear_cert();
+  m_ssl_options.clear_cipher();
   m_ssl_options.clear_key();
-  // SSL client certificate options are login options
-  if (orig.has_cert()) m_ssl_options.set_cert(orig.get_cert());
-  if (orig.has_key()) m_ssl_options.set_key(orig.get_key());
+  m_ssl_options.clear_crl();
+  m_ssl_options.clear_crlpath();
+  m_ssl_options.clear_tls_version();
+  m_ssl_options.clear_tls_ciphersuites();
+  const Ssl_options &ssl = options.get_ssl_options();
+  if (ssl.has_ca()) m_ssl_options.set_ca(ssl.get_ca());
+  if (ssl.has_capath()) m_ssl_options.set_capath(ssl.get_capath());
+  if (ssl.has_cert()) m_ssl_options.set_cert(ssl.get_cert());
+  if (ssl.has_cipher()) m_ssl_options.set_cipher(ssl.get_cipher());
+  if (ssl.has_key()) m_ssl_options.set_key(ssl.get_key());
+  if (ssl.has_crl()) m_ssl_options.set_crl(ssl.get_crl());
+  if (ssl.has_crlpath()) m_ssl_options.set_crlpath(ssl.get_crlpath());
+  if (ssl.has_tls_version())
+    m_ssl_options.set_tls_version(ssl.get_tls_version());
+  if (ssl.has_tls_ciphersuites())
+    m_ssl_options.set_tls_ciphersuites(ssl.get_tls_ciphersuites());
 }
 
 void Connection_options::set_ssl_options(const Ssl_options &options) {
   m_ssl_options = options;
 }
 
-bool Connection_options::has_data() const {
-  for (auto o : fixed_str_list) {
-    if (has_value(o)) return true;
+const std::string &Connection_options::get_password() const {
+  return m_mfa_passwords[0].has_value() ? *m_mfa_passwords[0]
+                                        : get_value(kPassword);
+}
+
+void Connection_options::set_ssh_options(
+    const mysqlshdk::ssh::Ssh_connection_options &options) {
+  m_ssh_options = options;
+}
+
+ssh::Ssh_connection_options &Connection_options::get_ssh_options_handle(
+    int fallback_port) {
+  if (m_ssh_options.has_data()) {
+    int mysql_port = fallback_port;
+    if (!m_port.is_null()) {
+      mysql_port = *m_port;
+    } else if (has_scheme()) {
+      if (get_scheme() == "mysql") {
+        mysql_port = k_default_mysql_port;
+      } else if (get_scheme() == "mysqlx") {
+        mysql_port = k_default_mysql_x_port;
+      }
+    }
+
+    if (has_host() && mysql_port != 0) {
+      m_ssh_options.clear_remote_host();
+      m_ssh_options.clear_remote_port();
+
+      m_ssh_options.set_remote_host(get_host());
+      m_ssh_options.set_remote_port(mysql_port);
+    } else {
+      throw std::invalid_argument(
+          "Host and port for database are required in advance when using "
+          "SSH tunneling functionality");
+    }
   }
-  return false;
+
+  return m_ssh_options;
+}
+
+bool Connection_options::has_data() const {
+  return IConnection::has_data() || has_value(kSocket);
 }
 
 void Connection_options::_set_fixed(const std::string &key,
@@ -153,11 +186,42 @@ void Connection_options::set_pipe(const std::string &pipe) {
   m_transport_type = Pipe;
 }
 
-void Connection_options::set_compression(bool compression) {
-  if (compression)
-    m_extra_options.set(mysqlshdk::db::kCompression, "true");
+void Connection_options::check_compression_conflicts() {
+  if (has_compression_algorithms() &&
+      get_compression_algorithms() == "uncompressed" && has_compression() &&
+      get_compression() == kCompressionRequired)
+    throw std::invalid_argument(
+        "Conflicting connection options: compression=REQUIRED, "
+        "compression-algorithms=uncompressed.");
+}
+
+void Connection_options::set_compression(const std::string &compression) {
+  if (has_compression())
+    throw std::invalid_argument(std::string("Redefinition of '") +
+                                kCompression + "' option");
+
+  auto c = shcore::str_upper(shcore::str_strip(compression));
+  if (c == "1" || c == "TRUE" || c == kCompressionRequired)
+    m_extra_options.set(mysqlshdk::db::kCompression, kCompressionRequired);
+  else if (c == "0" || c == "FALSE" || c == kCompressionDisabled)
+    m_extra_options.set(mysqlshdk::db::kCompression, kCompressionDisabled);
+  else if (c == kCompressionPreferred)
+    m_extra_options.set(mysqlshdk::db::kCompression, kCompressionPreferred);
   else
-    m_extra_options.set(mysqlshdk::db::kCompression, "false");
+    throw std::invalid_argument(
+        "Invalid value '" + compression + "' for '" + kCompression +
+        "'. Allowed values: '" + kCompressionRequired + "', '" +
+        kCompressionPreferred + "', '" + kCompressionDisabled +
+        "', 'True', 'False', '1', and '0'.");
+
+  check_compression_conflicts();
+}
+
+void Connection_options::set_compression_algorithms(
+    const std::string &compression_algorithms) {
+  auto ca = shcore::str_lower(compression_algorithms);
+  m_extra_options.set(kCompressionAlgorithms, ca);
+  check_compression_conflicts();
 }
 
 void Connection_options::set_socket(const std::string &socket) {
@@ -237,14 +301,11 @@ void Connection_options::set_port(int port) {
 }
 
 std::string Connection_options::get_iname(const std::string &name) const {
-  // DbUser and DbPassword are only supported as input
-  // Internally, they are handled as User and Password
-  std::string iname(name);
-  if (m_options.compare(name, kDbUser) == 0)
-    return kUser;
-  else if (m_options.compare(name, kDbPassword) == 0)
-    return kPassword;
-
+  // Deprecated options are only supported as input
+  // Internally, they are handled with replacements
+  const auto it = deprecated_connection_attributes.find(name);
+  if (it != deprecated_connection_attributes.end() && !it->second.empty())
+    return it->second;
   return name;
 }
 
@@ -264,21 +325,29 @@ bool Connection_options::is_bool_value(const std::string &value) {
 
 void Connection_options::set(const std::string &name,
                              const std::string &value) {
-  std::string iname = get_iname(name);
+  const std::string iname = get_iname(name);
+  if (name != iname)
+    m_warnings.emplace_back("'" + name +
+                            "' connection option is deprecated, use '" + iname +
+                            "' option instead.");
 
   if (m_options.has(iname)) {
     m_options.set(iname, value, Set_mode::UPDATE_NULL);
   } else if (m_ssl_options.has(iname)) {
     m_ssl_options.set(iname, value);
+  } else if (iname == kCompression) {
+    set_compression(value);
+  } else if (iname == kCompressionAlgorithms) {
+    set_compression_algorithms(value);
   } else if (is_extra_option(iname)) {
-    if (name == kGetServerPublicKey || name == kCompression) {
+    if (iname == kGetServerPublicKey) {
       if (!is_bool_value(value)) {
         throw std::invalid_argument(
             shcore::str_format("Invalid value '%s' for '%s'. Allowed "
                                "values: true, false, 1, 0.",
-                               value.c_str(), name.c_str()));
+                               value.c_str(), iname.c_str()));
       }
-    } else if (name == kConnectTimeout) {
+    } else if (iname == kConnectTimeout) {
       for (auto digit : value) {
         if (!std::isdigit(digit)) {
           throw_invalid_connect_timeout(value);
@@ -287,7 +356,7 @@ void Connection_options::set(const std::string &name,
     }
 
     m_extra_options.set(iname, value, Set_mode::CREATE);
-  } else if (name == kConnectionAttributes) {
+  } else if (iname == kConnectionAttributes) {
     // When connection-attributes has assigned a single value, it must
     // be a boolean. We do NOT do anything with it: in DevAPI standard
     // it should avoid ANY connection attribute to be sent, however
@@ -303,6 +372,13 @@ void Connection_options::set(const std::string &name,
         m_enable_connection_attributes = false;
       }
     }
+  } else if (iname == kCompressionLevel) {
+    try {
+      set_compression_level(shcore::lexical_cast<int>(value));
+    } catch (...) {
+      throw std::invalid_argument(std::string("The value of '") +
+                                  kCompressionLevel + "' must be an integer.");
+    }
   } else {
     throw std::invalid_argument("Invalid connection option '" + name + "'.");
   }
@@ -317,13 +393,19 @@ void Connection_options::set(const std::string &name,
                              const std::vector<std::string> &values) {
   std::string iname = get_iname(name);
 
-  if (m_options.has(iname) || m_ssl_options.has(iname) ||
-      is_extra_option(iname)) {
+  if (name == kConnectionAttributes) {
+    set_connection_attributes(values);
+  } else if (shcore::str_caseeq(name, kSslTlsVersions)) {
+    m_ssl_options.set_tls_version(
+        shcore::str_join(values.begin(), values.end(), ","));
+  } else if (shcore::str_caseeq(name, kSslTlsCiphersuites)) {
+    m_ssl_options.set_tls_ciphersuites(
+        shcore::str_join(values.begin(), values.end(), ":"));
+  } else if (m_options.has(iname) || m_ssl_options.has(iname) ||
+             is_extra_option(iname)) {
     // All the connection parameters accept only 1 value
     throw std::invalid_argument("The connection option '" + name +
                                 "' requires exactly one value.");
-  } else if (name == kConnectionAttributes) {
-    set_connection_attributes(values);
   } else {
     throw std::invalid_argument("Invalid connection option '" + name + "'.");
   }
@@ -351,6 +433,10 @@ bool Connection_options::has_value(const std::string &name) const {
   return false;
 }
 
+bool Connection_options::has_ssh_options() const {
+  return m_ssh_options.has_data();
+}
+
 int Connection_options::get_port() const {
   if (m_port.is_null()) {
     throw std::invalid_argument(
@@ -360,6 +446,21 @@ int Connection_options::get_port() const {
   return *m_port;
 }
 
+/**
+ * This function is used to determine the port to be used to create the MySQL
+ * connection, i.e if through SSH tunnels returns the tunnel local port,
+ * otherwise the configured port in the connection data.
+ */
+int Connection_options::get_target_port() const {
+  int port = 0;
+  if (m_ssh_options.has_data()) {
+    port = m_ssh_options.get_local_port();
+  } else if (has_port()) {
+    port = get_port();
+  }
+  return port;
+}
+
 Transport_type Connection_options::get_transport_type() const {
   if (m_transport_type.is_null())
     throw std::invalid_argument("Transport Type is undefined.");
@@ -367,12 +468,15 @@ Transport_type Connection_options::get_transport_type() const {
   return *m_transport_type;
 }
 
-bool Connection_options::get_compression() const {
-  if (!has_compression())
-    throw std::invalid_argument("Compression is undefined.");
+int64_t Connection_options::get_compression_level() const {
+  if (m_compress_level.is_null())
+    throw std::invalid_argument("Compression level is undefined.");
+  return *m_compress_level;
+}
 
-  return shcore::lexical_cast<bool>(
-      shcore::str_lower(m_extra_options.get_value(kCompression)));
+const Mfa_passwords &Connection_options::get_mfa_passwords() const {
+  if (has_password()) m_mfa_passwords[0] = get_password();
+  return m_mfa_passwords;
 }
 
 const std::string &Connection_options::get(const std::string &name) const {
@@ -386,6 +490,19 @@ const std::string &Connection_options::get(const std::string &name) const {
     return get_value(name);  // <-- This will throw the standard message
 }
 
+bool Connection_options::has_mfa_passwords() const {
+  for (const auto &p : m_mfa_passwords) {
+    if (p.has_value()) return true;
+  }
+  return false;
+}
+
+void Connection_options::clear_mfa_passwords() {
+  for (auto &p : m_mfa_passwords) {
+    p.reset();
+  }
+}
+
 void Connection_options::clear_host() {
   if (!m_transport_type.is_null() && *m_transport_type == Transport_type::Tcp &&
       m_port.is_null())
@@ -428,7 +545,7 @@ void Connection_options::remove(const std::string &name) {
 
 std::string Connection_options::as_uri(
     mysqlshdk::db::uri::Tokens_mask format) const {
-  Uri_encoder encoder;
+  Uri_encoder encoder(false);
   return encoder.encode_uri(*this, format);
 }
 
@@ -459,9 +576,15 @@ mysqlsh::SessionType Connection_options::get_session_type() const {
   }
 }
 
-void Connection_options::set_default_connection_data() {
+void Connection_options::set_default_data() {
   // Default values
-  if (!has_user()) set_user(shcore::get_system_user());
+  if (!has_user() && !is_auth_method(mysqlshdk::db::kAuthMethodKerberos)) {
+    // The system user is not used with kerberos to trigger the client lib
+    // functionality of using cashed TGT
+    set_user(shcore::get_system_user());
+  }
+
+  set_plugins_dir();
 
   if (!has_host() &&
       (!has_transport_type() || get_transport_type() == mysqlshdk::db::Tcp))
@@ -476,6 +599,24 @@ void Connection_options::set_default_connection_data() {
     set_scheme("mysql");
   }
 #endif  // _WIN32
+
+  if (m_ssh_options.has_data()) {
+    m_ssh_options.set_default_data();
+  }
+}
+
+void Connection_options::set_plugins_dir() {
+  // Use the plugin dir from the shell options if not included on the connection
+  // options already
+  if (!has(mysqlshdk::db::kMysqlPluginDir)) {
+    // NOTE:Allowing empty options as required for many tests
+    // In the case of the shell binary that will never be the case
+    auto options = mysqlsh::current_shell_options(true);
+    if (options && !options->get().mysql_plugin_dir.empty()) {
+      set(mysqlshdk::db::kMysqlPluginDir,
+          mysqlsh::current_shell_options()->get().mysql_plugin_dir);
+    }
+  }
 }
 
 void Connection_options::throw_invalid_connect_timeout(
@@ -486,6 +627,23 @@ void Connection_options::throw_invalid_connect_timeout(
                          "(including 0).",
                          value.c_str(), kConnectTimeout));
 }
+
+bool Connection_options::is_auth_method(const std::string &method_id) const {
+  bool ret_val = false;
+
+  if (has_value(mysqlshdk::db::kAuthMethod)) {
+    auto auth_method = get(mysqlshdk::db::kAuthMethod);
+    if (method_id == mysqlshdk::db::kAuthMethodKerberos) {
+      ret_val = auth_method == mysqlshdk::db::kAuthMethodKerberos ||
+                auth_method == mysqlshdk::db::kAuthMethodLdapSasl;
+    } else {
+      ret_val = auth_method == method_id;
+    }
+  }
+
+  return ret_val;
+}
+
 void Connection_options::set_connection_attribute(const std::string &attribute,
                                                   const std::string &value) {
   std::string error;
@@ -535,5 +693,15 @@ void Connection_options::set_connection_attributes(
     }
   }
 }
+
+int64_t default_connect_timeout() {
+  // some tests execute SQL before options are available, in that case use the
+  // default value
+  const auto options = mysqlsh::current_shell_options(true);
+  const auto &storage =
+      options ? options->get() : mysqlsh::Shell_options::Storage();
+  return storage.connect_timeout * 1000;
+}
+
 }  // namespace db
 }  // namespace mysqlshdk
