@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -33,6 +33,7 @@
 
 #include "mysql/instance.h"
 #include "mysqlshdk/libs/config/config.h"
+#include "mysqlshdk/libs/mysql/gtid_utils.h"
 #include "mysqlshdk/libs/utils/nullable.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 
@@ -45,9 +46,6 @@ namespace mysqlshdk {
 namespace gr {
 
 static constexpr const char k_gr_plugin_name[] = "group_replication";
-static constexpr const char k_value_not_set[] = "<not set>";
-static constexpr const char k_no_value[] = "<no value>";
-static constexpr const char k_must_be_initialized[] = "<must be initialized>";
 static constexpr const char k_group_recovery_user_prefix[] =
     "mysql_innodb_cluster_";
 static constexpr const char k_group_recovery_old_user_prefix[] =
@@ -56,6 +54,12 @@ static constexpr const char k_gr_applier_channel[] =
     "group_replication_applier";
 static constexpr const char k_gr_recovery_channel[] =
     "group_replication_recovery";
+static constexpr const char k_gr_disable_super_read_only_if_primary[] =
+    "mysql_disable_super_read_only_if_primary";
+static constexpr const char k_gr_member_action_after_primary_election[] =
+    "AFTER_PRIMARY_ELECTION";
+static constexpr const char k_gr_start_failover_channels_if_primary[] =
+    "mysql_start_failover_channels_if_primary";
 
 /**
  * Enumeration of the supported states for Group Replication members.
@@ -69,9 +73,9 @@ enum class Member_state {
   MISSING
 };
 
-enum class Member_role { PRIMARY, SECONDARY };
+enum class Member_role { PRIMARY, SECONDARY, NONE };
 
-enum class Topology_mode { SINGLE_PRIMARY, MULTI_PRIMARY };
+enum class Topology_mode { SINGLE_PRIMARY, MULTI_PRIMARY, NONE };
 
 std::string to_string(const Member_state state);
 Member_state to_member_state(const std::string &state);
@@ -113,53 +117,7 @@ struct Member {
   std::string version;
 };
 
-enum class Config_type {
-  SERVER,
-  CONFIG,
-  RESTART_ONLY,
-};
-
-enum class Gr_seeds_change_type {
-  ADD,
-  REMOVE,
-  OVERRIDE,
-};
-
-typedef mysqlshdk::utils::Enum_set<Config_type, Config_type::RESTART_ONLY>
-    Config_types;
-
-struct Invalid_config {
-  std::string var_name;
-  std::string current_val;
-  std::string required_val;
-  mysqlshdk::utils::nullable<std::string> persisted_val;
-  Config_types types;
-  bool restart;
-  shcore::Value_type val_type;
-
-  // constructors
-  Invalid_config(std::string name, std::string req_val)
-      : Invalid_config(std::move(name), k_must_be_initialized,
-                       std::move(req_val), Config_types(), false,
-                       shcore::Value_type::String) {}
-  Invalid_config(std::string name, std::string curr_val, std::string req_val,
-                 Config_types types, bool rest, shcore::Value_type val_t)
-      : var_name(std::move(name)),
-        current_val(std::move(curr_val)),
-        required_val(std::move(req_val)),
-        types(std::move(types)),
-        restart(rest),
-        val_type(val_t) {}
-  // comparison operator to be used for sorting Invalid_config objects
-  bool operator<(const Invalid_config &rhs) const {
-    return var_name < rhs.var_name;
-  }
-};
-
 // Function to check membership and state.
-bool is_member(const mysqlshdk::mysql::IInstance &instance);
-bool is_member(const mysqlshdk::mysql::IInstance &instance,
-               const std::string &group_name);
 Member_state get_member_state(const mysqlshdk::mysql::IInstance &instance);
 std::vector<Member> get_members(const mysqlshdk::mysql::IInstance &instance,
                                 bool *out_single_primary_mode = nullptr,
@@ -174,13 +132,14 @@ bool has_quorum(const mysqlshdk::mysql::IInstance &instance,
 // Fetch various basic info bits from the group the given instance is member of
 bool get_group_information(const mysqlshdk::mysql::IInstance &instance,
                            Member_state *out_member_state,
-                           std::string *out_member_id,
-                           std::string *out_group_name,
-                           bool *out_single_primary_mode,
+                           std::string *out_member_id = nullptr,
+                           std::string *out_group_name = nullptr,
+                           std::string *out_group_view_change_uuid = nullptr,
+                           bool *out_single_primary_mode = nullptr,
                            bool *out_has_quorum = nullptr,
                            bool *out_is_primary = nullptr);
 
-std::string get_group_primary_uuid(const std::shared_ptr<db::ISession> &session,
+std::string get_group_primary_uuid(const mysqlshdk::mysql::IInstance &instance,
                                    bool *out_single_primary_mode);
 
 /**
@@ -198,6 +157,15 @@ std::string get_group_primary_uuid(const std::shared_ptr<db::ISession> &session,
 mysqlshdk::utils::Version get_group_protocol_version(
     const mysqlshdk::mysql::IInstance &instance);
 
+/**
+ * Gets the maximum GR protocol version supported by a instance of that version.
+ *
+ * Note that this only supports protocol version up to 8.0.16 and needs to be
+ * updated when a new version is added.
+ */
+mysqlshdk::utils::Version get_max_supported_group_protocol_version(
+    const mysqlshdk::utils::Version &server_version);
+
 /**
  * Set the Group Replication protocol version.
  *
@@ -213,39 +181,21 @@ void set_group_protocol_version(const mysqlshdk::mysql::IInstance &instance,
                                 mysqlshdk::utils::Version version);
 
 /**
- * Verify if a Group Replication protocol version downgrade is required
- *
- * This function verifies if a Group Replication protocol version downgrade is
- * required. This is necessary to verify if an instance that is joining a group
- * does not support the GR protocol version in use on the group (because it is
- * an older version) so the group protocol version must be downgraded
- *
- * @param version mysqlshdk::utils::Version object with the current GR protocol
- * version in used in the group
- * @param instance Instance object that points to the server that is joining the
- * cluster.
- */
-bool is_protocol_downgrade_required(
-    mysqlshdk::utils::Version current_group_version,
-    const mysqlshdk::mysql::IInstance &instance);
-
-/**
- * Verify if a Group Replication protocol version upgrade is required
+ * Verify if a Group Replication protocol version upgrade is possible
  *
  * This function verifies if a Group Replication protocol version upgrade is
- * required. This is necessary to verify if a group needs to upgrade its
+ * possible. This is necessary to verify if a group needs to upgrade its
  * protocol version after the removal of the target instance.
  *
  * @param instance Instance object that points to the server that is leaving the
  * cluster or a member of the cluster
- * @param server_uuid nullable string containing the instance server_uuid to be
- * skipped.
+ * @param skip_server_uuid instance server_uuid to be skipped.
  * @param out_protocol_version mysqlshdk::utils::Version object with the version
  * value that the group should be upgraded to.
  */
-bool is_protocol_upgrade_required(
+bool is_protocol_upgrade_possible(
     const mysqlshdk::mysql::IInstance &instance,
-    mysqlshdk::utils::nullable<std::string> server_uuid,
+    const std::string &skip_server_uuid,
     mysqlshdk::utils::Version *out_protocol_version);
 
 /**
@@ -257,11 +207,6 @@ bool is_protocol_upgrade_required(
 std::map<std::string, utils::nullable<std::string>> get_all_configurations(
     const mysqlshdk::mysql::IInstance &instance);
 
-// Function to do a change master (set the GR recovery user)
-void change_recovery_credentials(const mysqlshdk::mysql::IInstance &instance,
-                                 const std::string &rpl_user,
-                                 const std::string &rpl_pwd);
-
 // Functions to manage the GR plugin
 bool install_group_replication_plugin(
     const mysqlshdk::mysql::IInstance &instance,
@@ -271,21 +216,20 @@ bool uninstall_group_replication_plugin(
     mysqlshdk::config::Config *config);
 
 void start_group_replication(const mysqlshdk::mysql::IInstance &instance,
-                             const bool bootstrap,
-                             const uint16_t read_only_timeout = 900);
+                             const bool bootstrap);
 void stop_group_replication(const mysqlshdk::mysql::IInstance &instance);
 
 /**
- * Generate a UUID to use for the group name.
+ * Generate a UUID to use for the group name / view change uuid
  *
  * The UUID is generated from the target instance (MySQL server) using the
  * UUID() SQL function.
  *
  * @param instance target instance used to generate the UUID.
  *
- * @return A string with a new UUID to be used for the group name.
+ * @return A string with a new UUID to be used,
  */
-std::string generate_group_name(const mysqlshdk::mysql::IInstance &instance);
+std::string generate_uuid(const mysqlshdk::mysql::IInstance &instance);
 
 // Function to manage the replication (recovery) user for GR.
 mysql::User_privileges_result check_replication_user(
@@ -303,64 +247,22 @@ mysql::User_privileges_result check_replication_user(
  * @param hosts list of hosts that will be used for the user creation.
  * @param password password to be used for the account.
  *        If null a random password is assigned.
+ * @param clone_supportes boolean flag to indicate whether clone is supported or
+ * not to determine if the set of grants required to manage clone are to be
+ * given
+ * @param auto_failover boolean flag to indicate whether the account will be
+ * used for automatic connection failover (clusterSet) or not, to determine if
+ * the set of grants required for it are to be given
  * @return a Auth_options object with information about the created user
  * @throw std::runtime error if the replication user already exists on the
  * target instance with any of the given hosts or if an error occurs with the
  * user creation.
  */
 mysqlshdk::mysql::Auth_options create_recovery_user(
-    const std::string &username, mysqlshdk::mysql::IInstance *primary,
+    const std::string &username, const mysqlshdk::mysql::IInstance &primary,
     const std::vector<std::string> &hosts,
     const mysqlshdk::utils::nullable<std::string> &password,
-    bool clone_supported = false);
-
-std::string get_recovery_user(const mysqlshdk::mysql::IInstance &instance);
-
-// Function to check compliance to use GR.
-std::map<std::string, std::string> check_data_compliance(
-    const mysqlshdk::mysql::IInstance &instance, const uint16_t max_errors = 0);
-
-/**
- * Checks if several of the required MySQL variables for GR are valid.
- *
- * @param config Config object to obtain the settings to check. It can hold
- *        more than one configuration handler, for example for the server and
- *        option file (e.g., my.cnf).
- * @param out_invalid_vec pointer to vector that will be filled with the details
- *        of the invalid settings.
- */
-void check_server_variables_compatibility(
-    const mysqlshdk::config::Config &config,
-    std::vector<Invalid_config> *out_invalid_vec);
-
-/**
- * Checks if the server_id variable is valid for Group Replication.
- *
- * @param instance Instance object that points to the server to be checked.
- * @param config Config object to obtain the settings to check. It can hold
- *        more than one configuration handler, for example for the server and
- *        option file (e.g., my.cnf).
- * @param out_invalid_vec pointer to vector that will be filled with the details
- *        of the invalid settings.
- */
-void check_server_id_compatibility(
-    const mysqlshdk::mysql::IInstance &instance,
-    const mysqlshdk::config::Config &config,
-    std::vector<Invalid_config> *out_invalid_vec);
-
-/**
- * Checks if the the log_bin variable for is valid for Group Replication.
- *
- * @param instance Instance object that points to the server to be checked.
- * @param config Config object to obtain the settings to check. It can hold
- *        more than one configuration handler, for example for the server and
- *        option file (e.g., my.cnf).
- * @param out_invalid_vec pointer to vector that will be filled with the details
- *        of the invalid settings.
- */
-void check_log_bin_compatibility(const mysqlshdk::mysql::IInstance &instance,
-                                 const mysqlshdk::config::Config &config,
-                                 std::vector<Invalid_config> *out_invalid_vec);
+    bool clone_supported = false, bool auto_failover = false);
 
 /**
  * Checks if the thread for a delayed initialization of the group replication is
@@ -374,21 +276,35 @@ bool is_group_replication_delayed_starting(
     const mysqlshdk::mysql::IInstance &instance);
 
 /**
- * Check if the specified instance address corresponds to an active member from
+ * Check if the specified instance corresponds to an active member from
  * the perspective of the given instance.
  *
  * The instance is considered active if it belongs to the group and it is not
  * OFFLINE or UNREACHABLE.
  *
+ * By default, the instance UUID is used for the verification. If the host and
+ * port are provided, those are used instead and the check is performed from the
+ * point of view of the instance session.
+ *
  * @param instance Instance to use to perform the check.
- * @param host string with the host of the instance to check.
- * @param port int with the port of the instance to check.
+ * @param host optional string with the host of the instance to check.
+ * @param port optional int with the port of the instance to check.
  *
  * @return a boolean value indicating if the instance is an active member of the
  *         Group Replication group (true) or not (false).
  */
 bool is_active_member(const mysqlshdk::mysql::IInstance &instance,
-                      const std::string &host, const int port);
+                      const std::string &host = "", const int port = 0);
+
+/**
+ * Waits for a specific instance to become an ONLINE member of the group
+ * from its own perspective.
+ *
+ * @param instance Instance to use to perform the check.
+ * @param timeout_ms Integer with the maximum wait time (in milliseconds).
+ */
+void wait_member_online(const mysqlshdk::mysql::IInstance &instance,
+                        int32_t timeout_ms);
 
 /**
  * Update auto-increment setting based on the GR mode.
@@ -420,23 +336,22 @@ void update_auto_increment(mysqlshdk::config::Config *config,
  *
  * @param config Config object used to set the GR group_seeds
  *               on all servers.
- * @param gr_address string with the input GR address used to update the current
- *                   group_seeds value.
- * @param change_type Enumeration with the type of change that will be
- *                    performed to the group_seeds (ADD, REMOVE, or OVERRIDE).
+ * @param group_seeds for each member of the group, by server_uuid
  */
 void update_group_seeds(mysqlshdk::config::Config *config,
-                        const std::string &gr_address,
-                        Gr_seeds_change_type change_type);
+                        const std::map<std::string, std::string> &group_seeds);
 
 /**
  * Configure which member of a single-primary replication group is the primary
  *
  * @param instance Instance to run the operations.
  * @param uuid server_uuid of the member that shall become the new primary.
+ * @param running_transactions_timeout number of seconds before ongoing
+ * transactions are killed and the request is allowed to proceed
  */
 void set_as_primary(const mysqlshdk::mysql::IInstance &instance,
-                    const std::string &uuid);
+                    const std::string &uuid,
+                    std::optional<uint32_t> running_transactions_timeout);
 
 /**
  * Changes a group running in single-primary mode to multi-primary mode
@@ -480,7 +395,7 @@ bool is_running_gr_auto_rejoin(const mysqlshdk::mysql::IInstance &instance);
  * @param lowest_cluster_version Version object with the lowest server version
  *                               in the cluster to join.
  */
-void check_instance_version_compatibility(
+void check_instance_check_installed_schema_version(
     const mysqlshdk::mysql::IInstance &instance,
     mysqlshdk::utils::Version lowest_cluster_version);
 
@@ -535,6 +450,75 @@ Group_member_recovery_status detect_recovery_status(
     const mysqlshdk::mysql::IInstance &instance, const std::string &start_time,
     bool *out_recovering = nullptr);
 
+/**
+ * Validate if the endpoint (host + port) is supported by Group replication.
+ * Performs "syntactic" validation of the host part when it is
+ * an IP address (either IPv4 or IPv6). Hostnames are not validated (the method
+ * returns true in these cases) because even if we could resolve them we cannot
+ * assume that name resolution is the same across all the cluster instances and
+ * the machine where the ngshell is running.
+ *
+ * // Note cannot be used for ipWhitelist validation as ipWhitelist values
+ * // don't have a port, just IP/hostname and also accept netmasks.
+ * @param endpoint The endpoint we want to check is supported by the instance to
+ * be used as either localAddress or groupSeed.
+ * @param version The version of the instance
+ * @return true if endpoint is supported with the given version or an hostname
+ * is used on the host part, false otherwise.
+ */
+
+bool is_endpoint_supported_by_gr(const std::string &endpoint,
+                                 const mysqlshdk::utils::Version &version);
+
+/**
+ * Disable a Group Replication member action
+ *
+ * @param instance Instance to run the operations.
+ * @param name The name of the action
+ * @param state On which stage the action is triggered
+ *
+ * @returns nothing
+ */
+void disable_member_action(const mysqlshdk::mysql::IInstance &instance,
+                           const std::string &action_name,
+                           const std::string &stage);
+
+/**
+ * Enable a Group Replication member action
+ *
+ * @param instance Instance to run the operations.
+ * @param name The name of the action
+ * @param state On which stage the action is triggered
+ *
+ * @returns nothing
+ */
+void enable_member_action(const mysqlshdk::mysql::IInstance &instance,
+                          const std::string &action_name,
+                          const std::string &stage);
+
+/**
+ * Get a Group Replication member action status
+ *
+ * @param instance Instance to run the operations.
+ * @param name The name of the action
+ * @param out_enabled true if action exists and is enabled, false if it exists
+ * and is disabled, undefined if the action doesn't exist
+ *
+ * @returns true if the action exists, false otherwise
+ */
+bool get_member_action_status(const mysqlshdk::mysql::IInstance &instance,
+                              const std::string &action_name,
+                              bool *out_enabled);
+
+/**
+ * Reset all Group Replication member actions configuration
+ *
+ * @param instance Instance to run the operations.
+ *
+ * @returns nothing
+ */
+void reset_member_actions(const mysqlshdk::mysql::IInstance &instance);
+
 }  // namespace gr
 }  // namespace mysqlshdk
 
