@@ -29,6 +29,7 @@
 #include <vector>
 
 #include "mysqlshdk/libs/db/uri_encoder.h"
+#include "mysqlshdk/libs/utils/fault_injection.h"
 #include "mysqlshdk/libs/utils/utils_json.h"
 
 namespace mysqlshdk {
@@ -36,6 +37,13 @@ namespace oci {
 
 const size_t MAX_LIST_OBJECTS_LIMIT = 1000;
 
+FI_DEFINE(oci_put_object, ([](const mysqlshdk::utils::FI::Args &args) {
+            throw mysqlshdk::oci::Response_error(
+                static_cast<mysqlshdk::oci::Response::Status_code>(
+                    args.get_int("code")),
+                args.get_string("msg"));
+          }));
+
 namespace {
 std::string encode_path(const std::string &data) {
   mysqlshdk::db::uri::Uri_encoder encoder;
@@ -159,6 +167,14 @@ void Bucket::ensure_connection() {
                                          std::unique_ptr<Oci_rest_service>>
       services;
 
+  // need to detect when this instance was passed to another thread and replace
+  // the REST service, or we may end up sharing it across multiple threads
+  const auto current_thread = std::this_thread::get_id();
+
+  if (current_thread != m_rest_service_thread) {
+    m_rest_service = nullptr;
+  }
+
   if (!m_rest_service) {
     auto &service = services[m_options.get_hash()];
 
@@ -168,6 +184,7 @@ void Bucket::ensure_connection() {
     }
 
     m_rest_service = service.get();
+    m_rest_service_thread = current_thread;
   }
 }
 
@@ -333,6 +350,9 @@ void Bucket::put_object(const std::string &object_name, const char *body,
   request.size = size;
 
   try {
+    FI_TRIGGER_TRAP(oci_put_object, mysqlshdk::utils::FI::Trigger_options(
+                                        {{"name", object_name}}));
+
     m_rest_service->put(&request);
   } catch (const Response_error &error) {
     throw Response_error(error.code(),
