@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2014, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -23,6 +23,7 @@
 
 #include "mysqlshdk/libs/utils/utils_general.h"
 
+#include <string_view>
 #include <utility>
 
 #include "mysqlshdk/libs/textui/textui.h"
@@ -61,6 +62,7 @@ errno_t memset_s(void *__s, rsize_t __smax, int __c, rsize_t __n);
 #include "mysqlshdk/libs/db/connection_options.h"
 #include "mysqlshdk/libs/db/uri_parser.h"
 #include "mysqlshdk/libs/utils/utils_file.h"
+#include "mysqlshdk/libs/utils/utils_path.h"
 #include "mysqlshdk/libs/utils/utils_sqlstring.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 #include "shellcore/utils_help.h"
@@ -179,11 +181,23 @@ mysqlshdk::db::Connection_options get_connection_options(const std::string &uri,
                                                          bool set_defaults) {
   mysqlshdk::db::Connection_options connection_options(uri);
 
-  if (set_defaults) connection_options.set_default_connection_data();
+  if (set_defaults) connection_options.set_default_data();
 
   return connection_options;
 }
 
+mysqlshdk::ssh::Ssh_connection_options get_ssh_connection_options(
+    const std::string &uri, bool set_defaults, const std::string &config_path) {
+  mysqlshdk::ssh::Ssh_connection_options config(uri);
+  if (!config_path.empty()) {
+    config.set_config_file(config_path);
+  }
+
+  if (set_defaults) config.set_default_data();
+
+  return config;
+}
+
 /**
  * Overrides connection data parameters with specific values, also adds
  * parameters with default values if missing
@@ -373,22 +387,18 @@ std::string get_system_user() {
 }
 
 std::string errno_to_string(int err) {
+  if (!err) return std::string();
+
 #ifdef _WIN32
 #define strerror_r(E, B, S) strerror_s(B, S, E)
 #endif
-#if defined(_WIN32) || defined(__sun)
+
+#if defined(_WIN32) || defined(__sun) || defined(__APPLE__) || \
+    ((_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600) &&   \
+     !_GNU_SOURCE)  // NOLINT
   char buf[256];
   if (!strerror_r(err, buf, sizeof(buf))) return std::string(buf);
-  return "";
-#elif __APPLE__ || ((_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600) && \
-                    !_GNU_SOURCE)  // NOLINT
-  std::string ret;
-  ret.resize(256);
-  auto i = strerror_r(err, &ret[0], ret.size());
-  assert(i == 0);
-  (void)i;
-  ret.resize(strlen(&ret[0]));
-  return ret;
+  return std::string();
 #else
   char buf[256];
   return strerror_r(err, buf, sizeof(buf));
@@ -567,71 +577,74 @@ std::string from_camel_case_to_dashes(const std::string &name) {
 }
 
 static std::size_t span_quotable_identifier(const std::string &s, std::size_t p,
-                                            std::string *out_string) {
-  bool seen_not_a_digit = false;
-  if (s.size() <= p) return p;
+                                            std::string *out_string,
+                                            bool allow_ansi_quotes = false) {
+  if (p >= s.length()) return p;
 
-  char quote = s[p];
-  if (quote !=
-      '`') {  // Ignoring \" since ANSI_QUOTES is not meant to be supported
-    // check if valid initial char
-    if (!std::isalnum(quote) && quote != '_' && quote != '$')
-      throw std::runtime_error("Invalid character in identifier");
-    if (!std::isdigit(quote)) seen_not_a_digit = true;
-    quote = 0;
-  } else {
-    seen_not_a_digit = true;
-    p++;
-  }
+  const auto copy_current_character = [&s, &p, out_string]() {
+    if (out_string) out_string->push_back(s[p]);
+  };
+
+  if ('`' == s[p] || (allow_ansi_quotes && '"' == s[p])) {
+    const char quote = s[p++];
+    bool esc = false;
+    bool done = false;
+
+    while (!done && p < s.size()) {
+      if (quote == s[p]) {
+        if (esc) {
+          copy_current_character();
+          esc = false;
+        } else {
+          esc = true;
+        }
+      } else {
+        if (esc) {
+          done = true;
+          break;
+        } else {
+          copy_current_character();
+        }
+      }
 
-  if (quote == 0) {
-    while (p < s.size()) {
-      if (!std::isalnum(s[p]) && s[p] != '_' && s[p] != '$') break;
-      if (out_string) out_string->push_back(s[p]);
-      if (!seen_not_a_digit && !isdigit(s[p])) seen_not_a_digit = true;
       ++p;
     }
-    if (!seen_not_a_digit)
+
+    // was the last character a quote?
+    if (!done && esc) {
+      done = true;
+    }
+
+    if (!done) {
+      throw std::runtime_error("Invalid syntax in identifier");
+    }
+  } else {
+    const auto first = p;
+    bool seen_not_a_digit = false;
+
+    while (p < s.size()) {
+      if (!std::isalnum(s[p]) && s[p] != '_' && s[p] != '$') {
+        if (first == p) {
+          throw std::runtime_error("Invalid character in identifier");
+        } else {
+          break;
+        }
+      }
+
+      copy_current_character();
+
+      if (!seen_not_a_digit && !isdigit(s[p])) seen_not_a_digit = true;
+
+      ++p;
+    }
+
+    if (!seen_not_a_digit) {
       throw std::runtime_error(
           "Invalid identifier: identifiers may begin with a digit but unless "
           "quoted may not consist solely of digits.");
-
-  } else {
-    int esc = 0;
-    bool done = false;
-    while (p < s.size() && !done) {
-      if (esc == quote && s[p] != esc) {
-        done = true;
-        break;
-      }
-      switch (s[p]) {
-        case '`':
-          if (esc == quote) {
-            if (out_string) out_string->push_back(s[p]);
-            esc = 0;
-          } else {
-            esc = s[p];
-          }
-          break;
-        default:
-          if (esc == quote) {
-            if (out_string) out_string->push_back(s[p]);
-            esc = 0;
-          } else if (esc == 0) {
-            if (out_string) out_string->push_back(s[p]);
-          } else {
-            done = true;
-          }
-          break;
-      }
-      ++p;
-    }
-    if (!done && esc == quote) {
-      done = true;
-    } else if (!done) {
-      throw std::runtime_error("Invalid syntax in identifier");
     }
   }
+
   return p;
 }
 
@@ -797,9 +810,11 @@ static std::size_t span_account_hostname_relaxed(const std::string &s,
   if (s[p] == '`') {
     res = span_quotable_identifier(s, p, out_string);
   } else {
+    bool quoted = false;
     // Do not allow quote characters unless they are surrounded by quotes
     if (s[p] == s[s.size() - 1] && (s[p] == '\'' || s[p] == '"')) {
       // hostname surrounded by quotes.
+      quoted = true;
     } else {
       if ((s.find('\'', p) != std::string::npos) ||
           (s.find('"', p) != std::string::npos)) {
@@ -815,9 +830,6 @@ static std::size_t span_account_hostname_relaxed(const std::string &s,
       // If the complete string was not consumed could be a hostname that
       // requires quotes, they should be enabled only if not quoted already
       if (res < s.size() && auto_quote_hosts) {
-        bool quoted = ((s[p] == '\'' && s.at(s.size() - 1) == '\'') ||
-                       (s[p] == '"' && s.at(s.size() - 1) == '"'));
-
         try_quoting = !quoted;
       }
     } catch (const std::runtime_error &) {
@@ -844,7 +856,7 @@ static std::size_t span_account_hostname_relaxed(const std::string &s,
  * literals for username and hostname.
  */
 void split_account(const std::string &account, std::string *out_user,
-                   std::string *out_host, bool auto_quote_hosts) {
+                   std::string *out_host, Account::Auto_quote auto_quote) {
   std::size_t pos = 0;
   if (out_user) *out_user = "";
   if (out_host) *out_host = "";
@@ -859,11 +871,15 @@ void split_account(const std::string &account, std::string *out_user,
     } else {
       pos = account.rfind('@');
       if (pos == 0) throw std::runtime_error("User name must not be empty.");
-      // dont allow @ on the username unless it is quoted
-      if (account.rfind('@', pos - 1) != std::string::npos) {
-        throw std::runtime_error("Invalid user name: " +
-                                 account.substr(0, pos));
+
+      if (Account::Auto_quote::USER_AND_HOST != auto_quote) {
+        // don't allow @ on the username unless it is quoted
+        if (account.rfind('@', pos - 1) != std::string::npos) {
+          throw std::runtime_error("Invalid user name: " +
+                                   account.substr(0, pos));
+        }
       }
+
       if (out_user != nullptr) out_user->assign(account, 0, pos);
     }
   } else {
@@ -876,8 +892,8 @@ void split_account(const std::string &account, std::string *out_user,
       pos = account.length();
       if (out_host != nullptr) *out_host = "skip-grants host";
     } else {
-      pos = span_account_hostname_relaxed(account, pos, out_host,
-                                          auto_quote_hosts);
+      pos = span_account_hostname_relaxed(
+          account, pos, out_host, Account::Auto_quote::NO != auto_quote);
     }
   }
   if (pos < account.size())
@@ -885,9 +901,10 @@ void split_account(const std::string &account, std::string *out_user,
                              "'");
 }
 
-Account split_account(const std::string &account, bool auto_quote_hosts) {
+Account split_account(const std::string &account,
+                      Account::Auto_quote auto_quote) {
   Account result;
-  split_account(account, &result.user, &result.host, auto_quote_hosts);
+  split_account(account, &result.user, &result.host, auto_quote);
   return result;
 }
 
@@ -902,27 +919,39 @@ std::string make_account(const Account &account) {
   return make_account(account.user, account.host);
 }
 
+namespace {
+
+void ensure_dot(const std::string &str, std::size_t pos) {
+  if (str[pos] != '.') {
+    throw std::runtime_error(
+        std::string("Invalid object name, expected '.', but got: '") +
+        str[pos] + "'.");
+  }
+}
+
+void ensure_eos(const std::string &str, std::size_t pos) {
+  if (pos < str.length()) {
+    throw std::runtime_error(
+        std::string("Invalid object name, expected end of name, but got: '") +
+        str[pos] + "'.");
+  }
+}
+
+}  // namespace
+
 void split_schema_and_table(const std::string &str, std::string *out_schema,
-                            std::string *out_table) {
+                            std::string *out_table, bool allow_ansi_quotes) {
   std::string schema;
   std::string table;
 
-  auto pos = span_quotable_identifier(str, 0, &schema);
+  auto pos = span_quotable_identifier(str, 0, &schema, allow_ansi_quotes);
 
   if (pos < str.length()) {
-    if (str[pos] != '.') {
-      throw std::runtime_error(
-          std::string("Invalid object name, expected '.', but got: '") +
-          str[pos] + "'.");
-    }
+    ensure_dot(str, pos);
 
-    pos = span_quotable_identifier(str, ++pos, &table);
+    pos = span_quotable_identifier(str, ++pos, &table, allow_ansi_quotes);
 
-    if (pos < str.length()) {
-      throw std::runtime_error(
-          std::string("Invalid object name, expected end of name, but got: '") +
-          str[pos] + "'.");
-    }
+    ensure_eos(str, pos);
   } else {
     // there's only table name
     std::swap(schema, table);
@@ -942,6 +971,56 @@ void split_schema_and_table(const std::string &str, std::string *out_schema,
   }
 }
 
+void split_schema_table_and_object(const std::string &str,
+                                   std::string *out_schema,
+                                   std::string *out_table,
+                                   std::string *out_object,
+                                   bool allow_ansi_quotes) {
+  std::string schema;
+  std::string table;
+  std::string object;
+
+  auto pos = span_quotable_identifier(str, 0, &schema, allow_ansi_quotes);
+
+  if (pos < str.length()) {
+    ensure_dot(str, pos);
+
+    pos = span_quotable_identifier(str, ++pos, &table, allow_ansi_quotes);
+
+    if (pos < str.length()) {
+      ensure_dot(str, pos);
+
+      pos = span_quotable_identifier(str, ++pos, &object, allow_ansi_quotes);
+
+      ensure_eos(str, pos);
+    } else {
+      // there are table and object names
+      std::swap(table, object);
+      std::swap(schema, table);
+    }
+  } else {
+    // there's only object name
+    std::swap(schema, object);
+  }
+
+  if (object.empty()) {
+    throw std::runtime_error(
+        "Invalid identifier, object name cannot be empty.");
+  }
+
+  if (out_schema) {
+    *out_schema = std::move(schema);
+  }
+
+  if (out_table) {
+    *out_table = std::move(table);
+  }
+
+  if (out_object) {
+    *out_object = std::move(object);
+  }
+}
+
 void split_priv_level(const std::string &str, std::string *out_schema,
                       std::string *out_object, size_t *out_leftover) {
   assert(out_schema && out_object);
@@ -1081,7 +1160,13 @@ OperatingSystem get_os_type() {
   return os;
 }
 
-std::string get_machine_type() { return MACHINE_TYPE; }
+std::string get_machine_type() {
+  {
+    constexpr std::string_view machine_type{MACHINE_TYPE};
+    static_assert(!machine_type.empty());
+  }
+  return MACHINE_TYPE;
+}
 
 std::string to_string(OperatingSystem os_type) {
   switch (os_type) {
@@ -1194,29 +1279,6 @@ const char *get_long_version() {
          " - for MySQL " LIBMYSQL_VERSION " (" MYSQL_COMPILATION_COMMENT ")";
 }
 
-void clear_buffer(char *buffer, size_t size) {
-#ifdef _WIN32
-  SecureZeroMemory(buffer, size);
-#else
-#if defined HAVE_EXPLICIT_BZERO
-  explicit_bzero(buffer, size);
-#elif defined HAVE_MEMSET_S
-  memset_s(buffer, size, '\0', size);
-#else
-  volatile char *p = buffer;
-  while (size--) {
-    *p++ = '\0';
-  }
-#endif
-#endif
-}
-
-void clear_buffer(std::string *buffer) {
-  assert(buffer);
-  clear_buffer(&(*buffer)[0], buffer->capacity());
-  buffer->clear();
-}
-
 #ifdef _WIN32
 
 std::string SHCORE_PUBLIC last_error_to_string(DWORD code) {
