@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -23,7 +23,7 @@
 
 #include "mysqlshdk/libs/utils/utils_net.h"
 
-#ifdef WIN32
+#ifdef _WIN32
 // clang-format off
 #include <WinSock2.h>
 #include <Ws2tcpip.h>
@@ -48,6 +48,9 @@
 #include <memory>
 #include <vector>
 #include <bitset>
+#include <iomanip>
+#include <tuple>
+#include <regex>
 
 #include "mysqlshdk/libs/utils/enumset.h"
 #include "mysqlshdk/libs/utils/logger.h"
@@ -88,8 +91,9 @@ int get_protocol_family(const std::string &address) {
   return family;
 }
 
-void get_host_ipv4_addresses(const std::string &host,
-                             std::vector<std::string> *out_addrs) {
+void get_host_ip_addresses(const std::string &host,
+                           std::vector<std::string> *out_addrs,
+                           unsigned int ip_addr_type) {
   assert(out_addrs);
 
   if (AF_INET == get_protocol_family(host)) {
@@ -111,7 +115,7 @@ void get_host_ipv4_addresses(const std::string &host,
   addrinfo hints;
 
   memset(&hints, 0, sizeof(hints));
-  hints.ai_family = AF_INET;  // IPv4
+  hints.ai_family = ip_addr_type;
 
   addrinfo *info = nullptr;
   int result = getaddrinfo(host.c_str(), nullptr, &hints, &info);
@@ -145,6 +149,54 @@ std::string get_this_hostname() {
   return hostname;
 }
 
+enum class Hostname_type { UNKNOWN, HOSTNAME, ABSOLUTE_HOSTNAME, IPV4, IPV6 };
+Hostname_type get_hostname_type(const std::string &address) {
+  Hostname_type type =
+      address.empty() ? Hostname_type::UNKNOWN : Hostname_type::HOSTNAME;
+
+  if (Net::is_ipv4(address)) {
+    return Hostname_type::IPV4;
+  }
+
+  if (Net::is_ipv6(address)) {
+    return Hostname_type::IPV6;
+  }
+
+  if (type == Hostname_type::HOSTNAME && address.back() == '.') {
+    return Hostname_type::ABSOLUTE_HOSTNAME;
+  }
+
+  return type;
+}
+
+std::string get_fqdn(const std::string &address, int fqdn_type) {
+  if (address.empty()) {
+    return std::string{};
+  }
+
+  addrinfo hints = {};
+  hints.ai_flags = fqdn_type;
+
+  addrinfo *info = nullptr;
+  int r = getaddrinfo(address.c_str(), nullptr, &hints, &info);
+
+  if (r != 0) {
+    return std::string{};
+  }
+
+  if (info != nullptr) {
+    // NOTE: If hints.ai_flags includes the AI_CANONNAME flag, then the
+    // ai_canonname field of the first of the addrinfo structures in the
+    // returned list is set to point to the official name of the host. Therefore
+    // there is no need to iterate over whole list.
+    std::unique_ptr<addrinfo, void (*)(addrinfo *)> deleter{info, freeaddrinfo};
+    if (info->ai_canonname) {
+      return std::string{info->ai_canonname};
+    }
+  }
+  return std::string{};
+}
+
 namespace detail {
 
 enum class Address_type { LOOPBACK, LOCAL };
@@ -266,7 +318,6 @@ void get_this_host_addresses(const Address_types &types,
     out_addrs->push_back(get_this_hostname());
   }
 }
-
 }  // namespace detail
 }  // namespace
 
@@ -276,16 +327,35 @@ std::string Net::resolve_hostname_ipv4(const std::string &name) {
   return resolve_hostname_ipv4_all(name)[0];
 }
 
+std::string Net::resolve_hostname_ipv6(const std::string &name) {
+  return resolve_hostname_ipv6_all(name)[0];
+}
+
 std::vector<std::string> Net::resolve_hostname_ipv4_all(
     const std::string &name) {
   return get()->resolve_hostname_ipv4_all_impl(name);
 }
 
+std::vector<std::string> Net::resolve_hostname_ipv6_all(
+    const std::string &name) {
+  return get()->resolve_hostname_ipv6_all_impl(name);
+}
+
+std::vector<std::string> Net::get_hostname_ips(const std::string &name) {
+  return get()->resolve_hostname_ipv_any_all_impl(name);
+}
+
 bool Net::is_ipv4(const std::string &host) {
   return get_protocol_family(host) == AF_INET;
 }
 
 bool Net::is_ipv6(const std::string &host) {
+  if (host.empty() || '[' == host[0]) {
+    // on Windows `[IPv6]` host string is considered an IPv6 address, we do this
+    // test on all platforms to speed up things a bit
+    return false;
+  }
+
   // Handling of the zone ID by getaddrinfo() varies on different platforms:
   // numeric values are always accepted, some of them require the zone ID to
   // match the name of one of the network interfaces, while others accept
@@ -309,14 +379,15 @@ bool Net::is_port_listening(const std::string &address, int port) {
   return get()->is_port_listening_impl(address, port);
 }
 
-bool Net::strip_cidr(std::string *address, int *cidr) {
+std::tuple<std::string, mysqlshdk::utils::nullable<int>> Net::strip_cidr(
+    const std::string &address) {
   // Strip the CIDR value, if specified
-  size_t pos = address->find("/");
+  const size_t pos = address.find("/");
   if (pos != std::string::npos) {
-    std::string cidr_str = address->substr(pos + 1);
-
+    const std::string cidr_str = address.substr(pos + 1);
+    mysqlshdk::utils::nullable<int> cidr;
     try {
-      *cidr = std::stoi(cidr_str);
+      cidr = std::stoi(cidr_str);
     } catch (const std::invalid_argument &e) {
       throw std::invalid_argument("Invalid subnet CIDR value: '" + cidr_str +
                                   "': " + e.what());
@@ -324,12 +395,11 @@ bool Net::strip_cidr(std::string *address, int *cidr) {
       throw std::out_of_range("Converted CIDR value: '" + cidr_str +
                               "' out of range: " + e.what());
     }
-
-    address->erase(pos);
-
-    return true;
+    return std::tuple<std::string, mysqlshdk::utils::nullable<int>>{
+        address.substr(0, pos), cidr};
   }
-  return false;
+  return std::tuple<std::string, mysqlshdk::utils::nullable<int>>{
+      address, mysqlshdk::utils::nullable<int>{}};
 }
 
 std::vector<std::string> Net::get_local_addresses() {
@@ -360,12 +430,68 @@ void Net::set(Net *implementation) { s_implementation = implementation; }
 std::vector<std::string> Net::resolve_hostname_ipv4_all_impl(
     const std::string &name) const {
   std::vector<std::string> addrs;
-  get_host_ipv4_addresses(name, &addrs);
+  get_host_ip_addresses(name, &addrs, AF_INET);
   return addrs;
 }
 
+std::vector<std::string> Net::resolve_hostname_ipv6_all_impl(
+    const std::string &name) const {
+  std::vector<std::string> addrs;
+  get_host_ip_addresses(name, &addrs, AF_INET6);
+  return addrs;
+}
+
+std::vector<std::string> Net::resolve_hostname_ipv_any_all_impl(
+    const std::string &name) const {
+  std::vector<std::string> addrs;
+  get_host_ip_addresses(name, &addrs, AF_UNSPEC);
+  return addrs;
+}
+
+std::string Net::get_fqdn(const std::string &address) noexcept {
+  return get()->get_fqdn_impl(address);
+}
+
+std::string Net::get_fqdn_impl(const std::string &address) const noexcept {
+  // windows resolves localhost to `hostname`.
+  if (address.compare("localhost") == 0) {
+    return std::string{"localhost"};
+  }
+
+  std::string cname{};
+#ifdef AI_FQDN
+  if (AI_FQDN != AI_CANONNAME) {
+    cname = ::mysqlshdk::utils::get_fqdn(address, AI_FQDN);
+  }
+#endif
+  if (cname.empty()) {
+    cname = ::mysqlshdk::utils::get_fqdn(address, AI_CANONNAME);
+  }
+  return cname;
+}
+
+bool Net::is_externally_addressable(const std::string &address) {
+  const std::string fqdn = get_fqdn(address);
+
+  // Some addresses do not have FQDN (for example MacOS return empty string for
+  // "127.0.0.1"), so we fallback to check original address.
+  std::string check = !fqdn.empty() ? fqdn : address;
+  auto type = get_hostname_type(check);
+
+  if (Hostname_type::HOSTNAME == type) {
+    // Convert to absolute hostname
+    check += '.';
+  }
+
+  if (is_loopback(check)) {
+    return false;
+  }
+
+  return true;
+}
+
 bool Net::is_loopback_impl(const std::string &name) const {
-  if (name == "localhost" || name == "::1" || name.compare(0, 4, "127.") == 0) {
+  if (name == "localhost" || name == "::1") {
     return true;
   }
 
@@ -536,5 +662,67 @@ bool Net::is_port_listening_impl(const std::string &address, int port) const {
   }
   throw std::runtime_error("Could not resolve address");
 }
+
+std::pair<std::string, uint16_t> split_host_and_port(const std::string &s) {
+  std::string host;
+  std::string port;
+
+  std::smatch m;
+  if (!s.empty() && s[0] == '[') {
+    std::regex re("^\\[([a-fA-F0-9:]+|[a-fA-F0-9:]+%[^%\\]:]+)\\]:([0-9]+)$");
+
+    if (!std::regex_match(s, m, re)) {
+      throw std::invalid_argument("Invalid IPv6 address format in '" + s +
+                                  "'. Must be [<host>]:<port>");
+    }
+  } else {
+    std::regex re("^([a-zA-Z0-9._-]+):([0-9]+)$");
+
+    if (!std::regex_match(s, m, re)) {
+      throw std::invalid_argument(
+          "Invalid address format in '" + s +
+          "'. Must be <host>:<port> or [<ip>]:<port> for IPv6 addresses");
+    }
+  }
+  int nport = std::stoi(m[2].str());
+  if (nport < 1 || nport > UINT16_MAX)
+    throw std::invalid_argument("Invalid port number in '" + s + "'");
+
+  return {m[1].str(), static_cast<uint16_t>(nport)};
+}
+
+std::string make_host_and_port(const std::string &host, uint16_t port) {
+  if (mysqlshdk::utils::Net::is_ipv6(host))
+    return "[" + host + "]:" + std::to_string(port);
+  else
+    return host + ":" + std::to_string(port);
+}
+
+uint64_t host_to_network(uint64_t v) {
+#ifdef htonll
+  return htonll(v);
+#else
+  int le = 1;
+  if (*(char *)&le == 1) {
+    return ((uint64_t)htonl(v & 0xFFFFFFFFLL) << 32) | htonl(v >> 32);
+  } else {
+    return v;
+  }
+#endif
+}
+
+uint64_t network_to_host(uint64_t v) {
+#ifdef ntohll
+  return ntohll(v);
+#else
+  int le = 1;
+  if (*(char *)&le == 1) {
+    return ((uint64_t)ntohl(v & 0xFFFFFFFFLL) << 32) | ntohl(v >> 32);
+  } else {
+    return v;
+  }
+#endif
+}
+
 }  // namespace utils
 }  // namespace mysqlshdk
