@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -34,11 +34,25 @@
 
 #include <string>
 
+#include "scripting/python_utils.h"
+
 #include "mysqlshdk/libs/utils/utils_general.h"
 
-using namespace shcore;
+namespace shcore {
 
-static PyObject *dict_dir(PyShDictObject *self, PyObject *) {
+namespace {
+
+/*
+ * Wraps an map object as a Python sequence object
+ */
+struct PyShDictObject {
+  // clang-format off
+  PyObject_HEAD
+  shcore::Value::Map_type_ref *map;
+  // clang-format on
+};
+
+PyObject *dict_dir(PyShDictObject *self, PyObject *) {
   static constexpr const char *const methods[] = {
       "keys", "items", "values", "has_key", "update", "setdefault"};
   PyObject *members =
@@ -55,13 +69,7 @@ static PyObject *dict_dir(PyShDictObject *self, PyObject *) {
   return members;
 }
 
-static PyObject *dict_keys(PyShDictObject *self, PyObject *args) {
-  if (args) {
-    Python_context::set_python_error(PyExc_ValueError,
-                                     "method takes no arguments");
-    return NULL;
-  }
-
+PyObject *dict_keys(PyShDictObject *self, PyObject *) {
   PyObject *list = PyList_New(self->map->get()->size());
 
   Py_ssize_t i = 0;
@@ -72,16 +80,7 @@ static PyObject *dict_keys(PyShDictObject *self, PyObject *args) {
   return list;
 }
 
-static PyObject *dict_items(PyShDictObject *self, PyObject *args) {
-  if (args) {
-    Python_context::set_python_error(PyExc_ValueError,
-                                     "method takes no arguments");
-    return NULL;
-  }
-
-  Python_context *ctx = Python_context::get_and_check();
-  if (!ctx) return NULL;
-
+PyObject *dict_items(PyShDictObject *self, PyObject *) {
   PyObject *list = PyList_New(self->map->get()->size());
 
   Py_ssize_t i = 0;
@@ -89,50 +88,41 @@ static PyObject *dict_items(PyShDictObject *self, PyObject *args) {
        iter != self->map->get()->end(); ++iter) {
     PyObject *tuple = PyTuple_New(2);
     PyTuple_SetItem(tuple, 0, PyString_FromString(iter->first.c_str()));
-    PyTuple_SetItem(tuple, 1, ctx->shcore_value_to_pyobj(iter->second));
+    PyTuple_SetItem(tuple, 1, py::convert(iter->second));
     PyList_SetItem(list, i++, tuple);
   }
   return list;
 }
 
-static PyObject *dict_values(PyShDictObject *self, PyObject *args) {
-  if (args) {
-    Python_context::set_python_error(PyExc_ValueError,
-                                     "method takes no arguments");
-    return NULL;
-  }
-  Python_context *ctx = Python_context::get_and_check();
-  if (!ctx) return NULL;
-
+PyObject *dict_values(PyShDictObject *self, PyObject *) {
   PyObject *list = PyList_New(self->map->get()->size());
 
   Py_ssize_t i = 0;
   for (Value::Map_type::const_iterator iter = self->map->get()->begin();
        iter != self->map->get()->end(); ++iter)
-    PyList_SetItem(list, i++, ctx->shcore_value_to_pyobj(iter->second));
+    PyList_SetItem(list, i++, py::convert(iter->second));
 
   return list;
 }
 
-static PyObject *dict_has_key(PyShDictObject *self, PyObject *arg) {
+PyObject *dict_has_key(PyShDictObject *self, PyObject *arg) {
   if (!arg) {
     Python_context::set_python_error(PyExc_ValueError,
                                      "missing required argument");
     return NULL;
   }
 
-  const char *key_to_find = PyString_AsString(arg);
+  std::string key_to_find;
   bool found = false;
 
-  if (key_to_find) found = self->map->get()->has_key(key_to_find);
+  if (Python_context::pystring_to_string(arg, &key_to_find)) {
+    found = self->map->get()->has_key(key_to_find);
+  }
 
   return PyBool_FromLong(found);
 }
 
-static PyObject *dict_update(PyShDictObject *self, PyObject *arg) {
-  Python_context *ctx = Python_context::get_and_check();
-  if (!ctx) return NULL;
-
+PyObject *dict_update(PyShDictObject *self, PyObject *arg) {
   if (!arg) {
     Python_context::set_python_error(PyExc_ValueError,
                                      "dict argument required for update()");
@@ -142,7 +132,7 @@ static PyObject *dict_update(PyShDictObject *self, PyObject *arg) {
   Value value;
 
   try {
-    value = ctx->pyobj_to_shcore_value(arg);
+    value = py::convert(arg);
   } catch (const std::exception &exc) {
     Python_context::set_python_error(exc);
     return NULL;
@@ -161,10 +151,7 @@ static PyObject *dict_update(PyShDictObject *self, PyObject *arg) {
   Py_RETURN_NONE;
 }
 
-static PyObject *dict_get(PyShDictObject *self, PyObject *arg) {
-  Python_context *ctx = Python_context::get_and_check();
-  if (!ctx) return NULL;
-
+PyObject *dict_get(PyShDictObject *self, PyObject *arg) {
   PyObject *def = NULL;
   char *key;
 
@@ -178,7 +165,7 @@ static PyObject *dict_get(PyShDictObject *self, PyObject *arg) {
 
   if (key) {
     if (self->map->get()->has_key(key))
-      return ctx->shcore_value_to_pyobj((self->map->get()->find(key))->second);
+      return py::convert((self->map->get()->find(key))->second);
     else {
       if (def) {
         Py_INCREF(def);
@@ -193,10 +180,7 @@ static PyObject *dict_get(PyShDictObject *self, PyObject *arg) {
   Py_RETURN_NONE;
 }
 
-static PyObject *dict_setdefault(PyShDictObject *self, PyObject *arg) {
-  Python_context *ctx = Python_context::get_and_check();
-  if (!ctx) return NULL;
-
+PyObject *dict_setdefault(PyShDictObject *self, PyObject *arg) {
   PyObject *def = Py_None;
   char *key;
 
@@ -210,12 +194,12 @@ static PyObject *dict_setdefault(PyShDictObject *self, PyObject *arg) {
 
   if (key) {
     if (self->map->get()->has_key(key))
-      return ctx->shcore_value_to_pyobj((self->map->get()->find(key))->second);
+      return py::convert((self->map->get()->find(key))->second);
     else {
       if (def != Py_None) Py_INCREF(def);
       try {
         shcore::Value::Map_type *map = self->map->get();
-        (*map)[key] = ctx->pyobj_to_shcore_value(def);
+        (*map)[key] = py::convert(def);
         return def;
       } catch (const std::exception &exc) {
         Python_context::set_python_error(exc);
@@ -225,16 +209,15 @@ static PyObject *dict_setdefault(PyShDictObject *self, PyObject *arg) {
   Py_RETURN_NONE;
 }
 
-static PyObject *dict_repr(PyShDictObject *self) {
+PyObject *dict_repr(PyShDictObject *self) {
   return PyString_FromString(Value(*self->map).repr().c_str());
 }
 
-static PyObject *dict_str(PyShDictObject *self) {
+PyObject *dict_str(PyShDictObject *self) {
   return PyString_FromString(Value(*self->map).descr().c_str());
 }
 
-static int dict_init(PyShDictObject *self, PyObject *args,
-                     PyObject *UNUSED(kwds)) {
+int dict_init(PyShDictObject *self, PyObject *args, PyObject *UNUSED(kwds)) {
   PyObject *valueptr = NULL;
 
   if (!PyArg_ParseTuple(args, "")) return -1;
@@ -244,7 +227,7 @@ static int dict_init(PyShDictObject *self, PyObject *args,
   if (valueptr) {
     try {
       self->map->reset(
-          static_cast<Value::Map_type *>(PyCObject_AsVoidPtr(valueptr)));
+          static_cast<Value::Map_type *>(PyCapsule_GetPointer(valueptr, NULL)));
     } catch (const std::exception &exc) {
       Python_context::set_python_error(exc);
       return -1;
@@ -260,34 +243,29 @@ static int dict_init(PyShDictObject *self, PyObject *args,
   return 0;
 }
 
-static void dict_dealloc(PyShDictObject *self) {
+void dict_dealloc(PyShDictObject *self) {
   delete self->map;
 
-  self->ob_type->tp_free(self);
+  Py_TYPE(self)->tp_free(self);
 }
 
-static Py_ssize_t dict_length(PyShDictObject *self) {
+Py_ssize_t dict_length(PyShDictObject *self) {
   return self->map->get()->size();
 }
 
-static PyObject *dict_subscript(PyShDictObject *self, PyObject *key) {
-  AutoPyObject tmp;
-  if (PyUnicode_Check(key)) key = tmp = PyUnicode_AsUTF8String(key);
+PyObject *dict_subscript(PyShDictObject *self, PyObject *key) {
+  std::string k;
 
-  if (!PyString_Check(key)) {
+  if (!Python_context::pystring_to_string(key, &k)) {
     Python_context::set_python_error(PyExc_KeyError,
                                      "shell.Dict key must be a string");
     return NULL;
   }
-  const char *k = PyString_AsString(key);
-
-  Python_context *ctx = Python_context::get_and_check();
-  if (!ctx) return NULL;
 
   try {
     const auto &result = self->map->get()->find(k);
     if (result != self->map->get()->end()) {
-      return ctx->shcore_value_to_pyobj(result->second);
+      return py::convert(result->second);
     } else {
       Python_context::set_python_error(PyExc_KeyError, k);
     }
@@ -297,27 +275,21 @@ static PyObject *dict_subscript(PyShDictObject *self, PyObject *key) {
   return NULL;
 }
 
-static int dict_as_subscript(PyShDictObject *self, PyObject *key,
-                             PyObject *value) {
-  AutoPyObject tmp;
-  if (PyUnicode_Check(key)) key = tmp = PyUnicode_AsUTF8String(key);
+int dict_as_subscript(PyShDictObject *self, PyObject *key, PyObject *value) {
+  std::string k;
 
-  if (!PyString_Check(key)) {
+  if (!Python_context::pystring_to_string(key, &k)) {
     Python_context::set_python_error(PyExc_KeyError,
                                      "shell.Dict key must be a string");
     return -1;
   }
-  const char *k = PyString_AsString(key);
-
-  Python_context *ctx = Python_context::get_and_check();
-  if (!ctx) return -1;
 
   try {
     if (value == NULL)
       self->map->get()->erase(k);
     else {
       try {
-        Value v = ctx->pyobj_to_shcore_value(value);
+        Value v = py::convert(value);
         shcore::Value::Map_type *map = self->map->get();
         (*map)[k] = v;
       } catch (const std::exception &exc) {
@@ -331,24 +303,17 @@ static int dict_as_subscript(PyShDictObject *self, PyObject *key,
   return -1;
 }
 
-static PyObject *dict_getattro(PyShDictObject *self, PyObject *attr_name) {
-  AutoPyObject tmp;
-  if (PyUnicode_Check(attr_name))
-    attr_name = tmp = PyUnicode_AsUTF8String(attr_name);
-
-  if (PyString_Check(attr_name)) {
-    const char *attrname = PyString_AsString(attr_name);
+PyObject *dict_getattro(PyShDictObject *self, PyObject *attr_name) {
+  std::string attrname;
 
+  if (Python_context::pystring_to_string(attr_name, &attrname)) {
     PyObject *object;
     if ((object = PyObject_GenericGetAttr((PyObject *)self, attr_name)))
       return object;
     PyErr_Clear();
 
     if (self->map->get()->has_key(attrname)) {
-      Python_context *ctx = Python_context::get_and_check();
-      if (!ctx) return NULL;
-
-      return ctx->shcore_value_to_pyobj((**self->map)[attrname]);
+      return py::convert((**self->map)[attrname]);
     } else {
       std::string err = std::string("unknown attribute: ") + attrname;
       Python_context::set_python_error(PyExc_IndexError, err.c_str());
@@ -361,20 +326,22 @@ static PyObject *dict_getattro(PyShDictObject *self, PyObject *attr_name) {
 }
 
 struct Key_iterator {
-  PyObject_HEAD;
+  // clang-format off
+  PyObject_HEAD
   shcore::Value::Map_type_ref *map;
+  // clang-format on
   size_t initial_size;
   shcore::Value::Map_type::iterator next;
 };
 
-static void Key_iterator_dealloc(Key_iterator *self) {
+void Key_iterator_dealloc(Key_iterator *self) {
   if (self->map) {
     delete self->map;
   }
   PyObject_Del(self);
 }
 
-static PyObject *Key_iterator_next_key(Key_iterator *self) {
+PyObject *Key_iterator_next_key(Key_iterator *self) {
   const auto &map = self->map->get();
 
   if (!map) {
@@ -401,6 +368,16 @@ static PyObject *Key_iterator_next_key(Key_iterator *self) {
   }
 }
 
+#if PY_VERSION_HEX >= 0x03080000 && PY_VERSION_HEX < 0x03090000
+#ifdef __clang__
+// The tp_print is marked as deprecated, which makes clang unhappy, 'cause it's
+// initialized below. Skipping initialization also makes clang unhappy, so we're
+// disabling the deprecated declarations warning.
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+#endif  // __clang__
+#endif  // PY_VERSION_HEX
+
 static PyTypeObject Key_iterator_type = {
     PyVarObject_HEAD_INIT(&PyType_Type, 0)  // PyObject_VAR_HEAD
     "shell.DictIter",                       /* tp_name */
@@ -431,10 +408,31 @@ static PyTypeObject Key_iterator_type = {
     PyObject_SelfIter,                   /* tp_iter */
     (iternextfunc)Key_iterator_next_key, /* tp_iternext */
     0,                                   /* tp_methods */
-    0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+#if PY_VERSION_HEX >= 0x02060000
+    ,
+    0  // tp_version_tag
+#endif
+#if PY_VERSION_HEX >= 0x03040000
+    ,
+    0  // tp_finalize
+#endif
+#if PY_VERSION_HEX >= 0x03080000
+    ,
+    0,  // tp_vectorcall
+#if PY_VERSION_HEX < 0x03090000
+    0  // tp_print
+#endif
+#endif
 };
 
-static PyObject *dict_iter(PyShDictObject *self) {
+#if PY_VERSION_HEX >= 0x03080000 && PY_VERSION_HEX < 0x03090000
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif  // __clang__
+#endif  // PY_VERSION_HEX
+
+PyObject *dict_iter(PyShDictObject *self) {
   Key_iterator *iterator = PyObject_New(Key_iterator, &Key_iterator_type);
   iterator->map = new Value::Map_type_ref(*self->map);
   const auto &map = self->map->get();
@@ -452,13 +450,13 @@ static PyMethodDef PyShDictMethods[] = {
     //{"__getitem__", (PyCFunction)dict_subscript, METH_O|METH_COEXIST,
     // getitem_doc},
     {"__dir__", (PyCFunction)dict_dir, METH_NOARGS, nullptr},
-    {"keys", (PyCFunction)dict_keys, 0, NULL},
-    {"items", (PyCFunction)dict_items, 0, NULL},
-    {"values", (PyCFunction)dict_values, 0, NULL},
-    {"has_key", (PyCFunction)dict_has_key, 0, NULL},
-    {"update", (PyCFunction)dict_update, 0, NULL},
-    {"get", (PyCFunction)dict_get, METH_VARARGS, NULL},
-    {"setdefault", (PyCFunction)dict_setdefault, 0, NULL},
+    {"keys", (PyCFunction)dict_keys, METH_NOARGS, nullptr},
+    {"items", (PyCFunction)dict_items, METH_NOARGS, nullptr},
+    {"values", (PyCFunction)dict_values, METH_NOARGS, nullptr},
+    {"has_key", (PyCFunction)dict_has_key, METH_O, nullptr},
+    {"update", (PyCFunction)dict_update, METH_O, nullptr},
+    {"get", (PyCFunction)dict_get, METH_VARARGS, nullptr},
+    {"setdefault", (PyCFunction)dict_setdefault, METH_VARARGS, nullptr},
     {NULL, NULL, 0, NULL}};
 
 static PyMappingMethods PyShDictObject_as_mapping = {
@@ -467,9 +465,18 @@ static PyMappingMethods PyShDictObject_as_mapping = {
     (objobjargproc)dict_as_subscript  // objobjargproc mp_ass_subscript;
 };
 
+#if PY_VERSION_HEX >= 0x03080000 && PY_VERSION_HEX < 0x03090000
+#ifdef __clang__
+// The tp_print is marked as deprecated, which makes clang unhappy, 'cause it's
+// initialized below. Skipping initialization also makes clang unhappy, so we're
+// disabling the deprecated declarations warning.
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+#endif  // __clang__
+#endif  // PY_VERSION_HEX
+
 static PyTypeObject PyShDictObjectType = {
-    PyObject_HEAD_INIT(&PyType_Type)  // PyObject_VAR_HEAD
-    0,
+    PyVarObject_HEAD_INIT(&PyType_Type, 0)  // PyObject_VAR_HEAD
     "shell.Dict",  // char *tp_name; /* For printing, in format
                    // "<module>.<name>" */
     sizeof(PyShDictObject),
@@ -544,14 +551,33 @@ static PyTypeObject PyShDictObjectType = {
     0,  // PyObject *tp_cache;
     0,  // PyObject *tp_subclasses;
     0,  // PyObject *tp_weakdict;
-    0,  // tp_del
-#if (PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION > 5)
+    0   // tp_del
+#if PY_VERSION_HEX >= 0x02060000
+    ,
     0  // tp_version_tag
 #endif
+#if PY_VERSION_HEX >= 0x03040000
+    ,
+    0  // tp_finalize
+#endif
+#if PY_VERSION_HEX >= 0x03080000
+    ,
+    0,  // tp_vectorcall
+#if PY_VERSION_HEX < 0x03090000
+    0  // tp_print
+#endif
+#endif
 };
 
+#if PY_VERSION_HEX >= 0x03080000 && PY_VERSION_HEX < 0x03090000
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif  // __clang__
+#endif  // PY_VERSION_HEX
+
+}  // namespace
+
 void Python_context::init_shell_dict_type() {
-  PyShDictObjectType.tp_new = PyType_GenericNew;
   if (PyType_Ready(&PyShDictObjectType) < 0) {
     throw std::runtime_error("Could not initialize Shcore Map type in python");
   }
@@ -564,21 +590,22 @@ void Python_context::init_shell_dict_type() {
       PyModule_GetDict(get_shell_python_support_module()), "Dict");
 }
 
-PyObject *shcore::wrap(std::shared_ptr<Value::Map_type> map) {
+PyObject *wrap(const std::shared_ptr<Value::Map_type> &map) {
   PyShDictObject *map_wrapper =
       PyObject_New(PyShDictObject, &PyShDictObjectType);
   map_wrapper->map = new Value::Map_type_ref(map);
   return reinterpret_cast<PyObject *>(map_wrapper);
 }
 
-bool shcore::unwrap(PyObject *value,
-                    std::shared_ptr<Value::Map_type> &ret_object) {
-  Python_context *ctx = Python_context::get_and_check();
-  if (!ctx) return false;
-
-  if (PyObject_IsInstance(value, ctx->get_shell_dict_class())) {
-    ret_object = *((PyShDictObject *)value)->map;
-    return true;
+bool unwrap(PyObject *value, std::shared_ptr<Value::Map_type> *ret_object) {
+  if (const auto ctx = Python_context::get_and_check()) {
+    if (PyObject_IsInstance(value, ctx->get_shell_dict_class())) {
+      *ret_object = *((PyShDictObject *)value)->map;
+      return true;
+    }
   }
+
   return false;
 }
+
+}  // namespace shcore
