@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -25,6 +25,7 @@
 
 #include <cassert>
 
+#include "scripting/obj_date.h"
 #include "scripting/python_array_wrapper.h"
 #include "scripting/python_function_wrapper.h"
 #include "scripting/python_map_wrapper.h"
@@ -38,7 +39,7 @@ Value convert(PyObject *py, Python_context *context) {
   return convert(py, &context);
 }
 
-Value convert(PyObject *py, Python_context **context) {
+Value convert(PyObject *py, Python_context **context, bool is_binary) {
   assert(context);
 
   // Some numeric conversions yield errors, in that case the string
@@ -95,14 +96,17 @@ Value convert(PyObject *py, Python_context **context) {
       retval = Value(value);
     }
   } else if (PyByteArray_Check(py)) {
-    return Value(PyByteArray_AsString(py), PyByteArray_Size(py));
+    return Value(PyByteArray_AsString(py), PyByteArray_Size(py), true);
   } else if (PyBytes_Check(py)) {
-    return Value(PyBytes_AsString(py), PyBytes_Size(py));
+    return Value(PyBytes_AsString(py), PyBytes_Size(py), is_binary);
   } else if (PyUnicode_Check(py)) {
     // TODO: In case of error calls ourself using NULL, either handle here
     // before calling recursively or always allow NULL
     Release ref{PyUnicode_AsUTF8String(py)};
-    retval = convert(ref, context);
+
+    // After unicode check, the value will match the Bytes check but the content
+    // should be treated as string
+    retval = convert(ref, context, false);
   } else if (array_check(py)) {
     std::shared_ptr<Value::Array_type> array;
 
@@ -170,21 +174,63 @@ Value convert(PyObject *py, Python_context **context) {
     } else if (unwrap(py, function)) {
       return Value(function);
     } else {
-      PyObject *obj_repr = PyObject_Repr(py);
-      std::string s;
-      Value ret_val;
-
-      if (Python_context::pystring_to_string(obj_repr, &s)) {
-        ret_val = Value(s);
+      if (!*context) {
+        try {
+          *context = Python_context::get();
+        } catch (const std::exception &e) {
+          throw std::runtime_error(
+              std::string{"Could not get SHELL context: "} + e.what());
+        }
       }
 
-      Py_DECREF(obj_repr);
-
-      if (ret_val)
+      auto get_int64 = [](PyObject *obj, const char *name) {
+        auto att = PyObject_GetAttrString(obj, name);
+        int64_t ret_val = PyLong_AsLongLong(att);
+        Py_XDECREF(att);
         return ret_val;
-      else
-        throw std::invalid_argument(
-            "Cannot convert Python value to internal value");
+      };
+
+      if (PyObject_TypeCheck(py, (*context)->get_datetime_type())) {
+        auto year = get_int64(py, "year");
+        auto month = get_int64(py, "month");
+        auto day = get_int64(py, "day");
+        auto hour = get_int64(py, "hour");
+        auto min = get_int64(py, "minute");
+        auto sec = get_int64(py, "second");
+        auto usec = get_int64(py, "microsecond");
+
+        return shcore::Value(Object_bridge_ref(
+            new Date(year, month, day, hour, min, sec, usec)));
+      } else if (PyObject_TypeCheck(py, (*context)->get_date_type())) {
+        auto year = get_int64(py, "year");
+        auto month = get_int64(py, "month");
+        auto day = get_int64(py, "day");
+
+        return shcore::Value(Object_bridge_ref(new Date(year, month, day)));
+      } else if (PyObject_TypeCheck(py, (*context)->get_time_type())) {
+        auto hour = get_int64(py, "hour");
+        auto min = get_int64(py, "minute");
+        auto sec = get_int64(py, "second");
+        auto usec = get_int64(py, "microsecond");
+
+        return shcore::Value(Object_bridge_ref(new Date(hour, min, sec, usec)));
+      } else {
+        PyObject *obj_repr = PyObject_Repr(py);
+        std::string s;
+        Value ret_val;
+
+        if (Python_context::pystring_to_string(obj_repr, &s)) {
+          ret_val = Value(s);
+        }
+
+        Py_DECREF(obj_repr);
+
+        if (ret_val)
+          return ret_val;
+        else
+          throw std::invalid_argument(
+              "Cannot convert Python value to internal value");
+      }
     }
   }
   // } TODO: else if (Buffer/MemoryView || Tuple || DateTime || generic_object
@@ -192,16 +238,16 @@ Value convert(PyObject *py, Python_context **context) {
   return retval;
 }
 
-PyObject *convert(const Value &value) {
+PyObject *convert(const Value &value, Python_context * /*context*/) {
   PyObject *r = nullptr;
   switch (value.type) {
     case Undefined:
-      Py_INCREF(Py_None);
       r = Py_None;
+      Py_INCREF(r);
       break;
     case Null:
-      Py_INCREF(Py_None);
       r = Py_None;
+      Py_INCREF(r);
       break;
     case Bool:
       r = PyBool_FromLong(value.value.b);
@@ -218,9 +264,47 @@ PyObject *convert(const Value &value) {
     case Float:
       r = PyFloat_FromDouble(value.value.d);
       break;
-    case Object:
-      r = wrap(*value.value.o);
+    case Object: {
+      if (value.as_object()->class_name() == "Date") {
+        auto ctx = Python_context::get();
+        std::shared_ptr<Date> date = value.as_object<Date>();
+
+        // 0 dates are invalid in Python, but OK in MySQL, so we convert these
+        // to None, which is what they probably really mean
+        if (date->has_date() && date->get_year() == 0 &&
+            date->get_month() == 0 && date->get_day() == 0) {
+          r = Py_None;
+          Py_INCREF(r);
+        } else if (date->has_time()) {
+          if (!date->has_date()) {
+            r = ctx->create_time_object(date->get_hour(), date->get_min(),
+                                        date->get_sec(), date->get_usec());
+          } else {
+            r = ctx->create_datetime_object(date->get_year(), date->get_month(),
+                                            date->get_day(), date->get_hour(),
+                                            date->get_min(), date->get_sec(),
+                                            date->get_usec());
+          }
+        } else {
+          r = ctx->create_date_object(date->get_year(), date->get_month(),
+                                      date->get_day());
+        }
+
+        // The conversion failed, so we take the string representation of the
+        // object
+        if (r == nullptr) {
+          // Cleanup the error condition
+          ctx->fetch_and_clear_exception();
+
+          // Take the object string representation
+          r = PyString_FromString(value.descr().c_str());
+        }
+      } else {
+        r = wrap(*value.value.o);
+      }
       break;
+    }
+
     case Array:
       r = wrap(*value.value.array);
       break;
@@ -242,6 +326,10 @@ PyObject *convert(const Value &value) {
     case shcore::Function:
       r = wrap(*value.value.func);
       break;
+    case shcore::Binary:
+      r = PyBytes_FromStringAndSize(value.value.s->c_str(),
+                                    value.value.s->size());
+      break;
   }
   return r;
 }
