@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2014, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -24,6 +24,7 @@
 
 #include <tuple>
 
+#include "mysqlshdk/libs/db/mysql/session.h"
 #include "mysqlshdk/libs/utils/fault_injection.h"
 #include "mysqlshdk/libs/utils/logger.h"
 #include "mysqlshdk/libs/utils/threads.h"
@@ -81,7 +82,10 @@ Base_shell::Base_shell(const std::shared_ptr<Shell_options> &cmdline_options)
 
   _input_mode = shcore::Input_state::Ok;
 
-  _shell = std::make_shared<shcore::Shell_core>();
+  // Whether the Shell_core is interactive or not will depend from the
+  // interactive option which is set depending on how the shell is started
+  _shell = std::make_shared<shcore::Shell_core>(
+      m_shell_options.get()->get().interactive);
   _completer_object_registry =
       std::make_shared<shcore::completer::Object_registry>();
 }
@@ -245,9 +249,7 @@ std::string Base_shell::prompt() {
 }
 
 std::map<std::string, std::string> *Base_shell::prompt_variables() {
-  if (m_pending_update != Prompt_variables_update_type::NO_UPDATE) {
-    update_prompt_variables();
-  }
+  update_prompt_variables();
 
   return &_prompt_variables;
 }
@@ -274,7 +276,8 @@ void Base_shell::update_prompt_variables() {
     _prompt_variables["system_user"] = shcore::get_system_user();
 
     if (session && session->is_open()) {
-      mysqlshdk::db::Connection_options options(session->uri());
+      const mysqlshdk::db::Connection_options &options(
+          session->get_connection_options());
       std::string socket;
       std::string port;
 
@@ -285,6 +288,13 @@ void Base_shell::update_prompt_variables() {
       _prompt_variables["host"] =
           options.has_host() ? options.get_host() : "localhost";
 
+      const auto &ssh = options.get_ssh_options();
+      if (ssh.has_data()) {
+        _prompt_variables["ssh_host"] = ssh.get_host();
+      } else {
+        _prompt_variables["ssh_host"] = "";
+      }
+
       if (session->get_member("__connection_info").descr().find("TCP") !=
           std::string::npos) {
         port = options.has_port() ? std::to_string(options.get_port()) : "";
@@ -318,14 +328,33 @@ void Base_shell::update_prompt_variables() {
       _prompt_variables["connection_id"] = "";
     }
   }
+
+  _prompt_variables["trx"] = "";
+  _prompt_variables["autocommit"] = "";
+  _prompt_variables["slow_query"] = "";
   if (session && session->is_open()) {
-    try {
-      _prompt_variables["schema"] = session->get_current_schema();
-    } catch (...) {
-      _prompt_variables["schema"] = "";
+    if (m_pending_update != Prompt_variables_update_type::NO_UPDATE) {
+      try {
+        _prompt_variables["schema"] = session->get_current_schema();
+      } catch (...) {
+        _prompt_variables["schema"] = "";
+      }
+    }
+
+    if (auto s = std::dynamic_pointer_cast<mysqlshdk::db::mysql::Session>(
+            session->get_core_session())) {
+      auto status = s->get_server_status();
+      if (status & SERVER_STATUS_IN_TRANS) _prompt_variables["trx"] = "*";
+      if (status & SERVER_STATUS_IN_TRANS_READONLY)
+        _prompt_variables["trx"] = "^";
+      if ((status & SERVER_STATUS_AUTOCOMMIT) == 0)
+        _prompt_variables["autocommit"] = ".";
+      if (status & SERVER_QUERY_WAS_SLOW) _prompt_variables["slow_query"] = "&";
     }
   } else {
-    _prompt_variables["schema"] = "";
+    if (m_pending_update != Prompt_variables_update_type::NO_UPDATE) {
+      _prompt_variables["schema"] = "";
+    }
   }
   switch (_shell->interactive_mode()) {
     case shcore::IShell_core::Mode::None:
@@ -451,8 +480,6 @@ void Base_shell::clear_input() {
 }
 
 void Base_shell::process_line(const std::string &line) {
-  std::string to_history;
-
   if (_input_mode == shcore::Input_state::ContinuedBlock && line.empty()) {
     _input_mode = shcore::Input_state::Ok;
   }
@@ -467,29 +494,39 @@ void Base_shell::process_line(const std::string &line) {
 
   if (_input_mode != shcore::Input_state::ContinuedBlock &&
       !_input_buffer.empty()) {
-    try {
+    execute_buffered_code(false);
+  }
+}
+
+void Base_shell::execute_buffered_code(bool flush) {
+  std::string to_history;
+  try {
+    if (flush) {
+      shcore::Scoped_callback reset_state([&]() { clear_input(); });
+      _shell->flush_input(_input_buffer);
+    } else {
       _shell->handle_input(_input_buffer, _input_mode);
-
-      // Here we analyze the input mode as it was let after executing the code
-      if (_input_mode == shcore::Input_state::Ok) {
-        to_history = _shell->get_handled_input();
-      }
-    } catch (shcore::Exception &exc) {
-      print_value(shcore::Value(exc.error()), "error");
-      to_history = _input_buffer;
-    } catch (const std::exception &exc) {
-      std::string error(exc.what());
-      error += "\n";
-      print_diag(error);
-      to_history = _input_buffer;
     }
 
-    // TODO: Do we need this cleanup? i.e. in case of exceptions above??
-    // Clears the buffer if OK, if continued, buffer will contain
-    // the non executed code
+    // Here we analyze the input mode as it was let after executing the code
     if (_input_mode == shcore::Input_state::Ok) {
-      _input_buffer.clear();
+      to_history = _shell->get_handled_input();
     }
+  } catch (shcore::Exception &exc) {
+    print_value(shcore::Value(exc.error()), "error");
+    to_history = _input_buffer;
+  } catch (const std::exception &exc) {
+    std::string error(exc.what());
+    error += "\n";
+    print_diag(error);
+    to_history = _input_buffer;
+  }
+
+  // TODO: Do we need this cleanup? i.e. in case of exceptions above??
+  // Clears the buffer if OK, if continued, buffer will contain
+  // the non executed code
+  if (_input_mode == shcore::Input_state::Ok) {
+    _input_buffer.clear();
   }
 
   if (!to_history.empty()) {
@@ -497,6 +534,8 @@ void Base_shell::process_line(const std::string &line) {
   }
 }
 
+void Base_shell::flush_input() { execute_buffered_code(true); }
+
 void Base_shell::notify_executed_statement(const std::string &line) {
   shcore::Value::Map_type_ref data(new shcore::Value::Map_type());
   (*data)["statement"] = shcore::Value(line);
@@ -639,17 +678,17 @@ int Base_shell::process_stream(std::istream &stream, const std::string &source,
       if (options().full_interactive) print(prompt());
     }
 
-    // If stream ended, but doesn't have newline at the end, then last statement
-    // won't be executed.  We force process empty line to be able to escape from
-    // ContinuedBlock state and execute last statement.
-    if (_input_mode == shcore::Input_state::ContinuedBlock) {
-      process_line(std::string());
+    // If stream ended, and state is in continuation mode, the execution is
+    // forced
+    if (_input_mode == shcore::Input_state::ContinuedBlock ||
+        _input_mode == shcore::Input_state::ContinuedSingle) {
+      flush_input();
     }
 
     // Being interactive, we do not care about the return value
     return 0;
   } else {
-    return _shell->process_stream(stream, source, false);
+    return _shell->process_stream(stream, source);
   }
 }
 
