@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -26,6 +26,7 @@
 #include <cstdio>
 #include <memory>
 #include <string>
+#include <unordered_map>
 #include <utility>
 
 #ifdef __sun
@@ -45,20 +46,48 @@
 
 namespace mysqlsh {
 namespace {
-std::string json_obj(const char *key, const std::string &value) {
-  shcore::JSON_dumper dumper(
-      mysqlsh::current_shell_options()->get().wrap_json == "json");
+
+std::string json_obj(
+    const char *key, const std::string &value,
+    const std::vector<std::string> &description = {},
+    const std::unordered_map<std::string, std::string> &additional = {},
+    const std::function<void(shcore::JSON_dumper *)> &callback = {}) {
+  const auto &options = mysqlsh::current_shell_options()->get();
+  shcore::JSON_dumper dumper(options.wrap_json == "json", options.binary_limit);
   dumper.start_object();
   dumper.append_string(key);
   dumper.append_string(value);
+
+  // Inserts the additional entries into the object
+  for (const auto &entry : additional) {
+    dumper.append_string(entry.first);
+    dumper.append_string(entry.second);
+  }
+
+  // Inserts the description into the object
+  if (!description.empty()) {
+    dumper.append_string("description");
+    dumper.start_array();
+    for (const auto &desc : description) {
+      dumper.append_string(desc);
+    }
+
+    dumper.end_array();
+  }
+
+  // Allows defining yet additional entries into the object
+  if (callback) {
+    callback(&dumper);
+  }
+
   dumper.end_object();
 
   return dumper.str() + "\n";
 }
 
 std::string json_obj(const char *key, const shcore::Value &info) {
-  shcore::JSON_dumper dumper(
-      mysqlsh::current_shell_options()->get().wrap_json == "json");
+  const auto &options = mysqlsh::current_shell_options()->get();
+  shcore::JSON_dumper dumper(options.wrap_json == "json", options.binary_limit);
   dumper.start_object();
   dumper.append_value(key, info);
   dumper.end_object();
@@ -373,19 +402,13 @@ void Shell_console::print_para(const std::string &text) const {
 }
 
 shcore::Prompt_result Shell_console::call_prompt(
-    const std::string &prompt, std::string *out_val, Validator validator,
-    const char *tag, shcore::Interpreter_delegate::Prompt func) const {
-  std::string text;
-  if (use_json()) {
-    text = json_obj(tag, prompt);
-  } else {
-    text = mysqlshdk::textui::bold(prompt);
-  }
-
+    const std::string &text, const shcore::prompt::Prompt_options &options,
+    std::string *out_val, Validator validator,
+    shcore::Interpreter_delegate::Prompt func) const {
   shcore::Prompt_result result;
   bool valid = true;
   do {
-    result = (m_ideleg->*func)(text.c_str(), out_val);
+    result = (m_ideleg->*func)(text.c_str(), options, out_val);
 
     if (result == shcore::Prompt_result::Ok) {
       if (validator) {
@@ -401,202 +424,118 @@ shcore::Prompt_result Shell_console::call_prompt(
   return result;
 }
 
-shcore::Prompt_result Shell_console::prompt(const std::string &prompt,
-                                            std::string *ret_val,
-                                            Validator validator) const {
-  return call_prompt(prompt, ret_val, validator, "prompt",
+shcore::Prompt_result Shell_console::prompt(
+    const std::string &prompt, const shcore::prompt::Prompt_options &options,
+    std::string *out_val) const {
+  return call_prompt(prompt, options, out_val, {},
                      &shcore::Interpreter_delegate::prompt);
 }
 
-static char process_label(const std::string &s, std::string *out_display,
-                          std::string *out_clean_text) {
-  out_display->clear();
-  if (s.empty()) return 0;
+shcore::Prompt_result Shell_console::prompt(
+    const std::string &prompt, std::string *ret_val, Validator validator,
+    shcore::prompt::Prompt_type type, const std::string &title,
+    const std::vector<std::string> &description,
+    const std::string &default_value) const {
+  // Other prompt types mus use their specific Shell_console function
+  assert(type == shcore::prompt::Prompt_type::TEXT ||
+         type == shcore::prompt::Prompt_type::PASSWORD ||
+         type == shcore::prompt::Prompt_type::FILEOPEN ||
+         type == shcore::prompt::Prompt_type::FILESAVE ||
+         type == shcore::prompt::Prompt_type::DIRECTORY);
 
-  char letter = 0;
-  char prev = 0;
-  for (char c : s) {
-    if (prev == '&') letter = c;
-    if (c != '&') {
-      if (prev == '&') {
-        out_display->push_back('[');
-        out_display->push_back(c);
-        out_display->push_back(']');
-      } else {
-        out_display->push_back(c);
-      }
-      out_clean_text->push_back(c);
-    }
-    prev = c;
-  }
-  return letter;
+  shcore::prompt::Prompt_options options;
+  options.title = title;
+  options.type = type;
+  options.description = description;
+  options.default_value = shcore::Value(default_value);
+
+  options.done();
+
+  return call_prompt(prompt, options, ret_val, validator,
+                     &shcore::Interpreter_delegate::prompt);
 }
 
-Prompt_answer Shell_console::confirm(const std::string &prompt,
-                                     Prompt_answer def,
-                                     const std::string &yes_label,
-                                     const std::string &no_label,
-                                     const std::string &alt_label) const {
+Prompt_answer Shell_console::confirm(
+    const std::string &prompt, Prompt_answer def, const std::string &yes_label,
+    const std::string &no_label, const std::string &alt_label,
+    const std::string &title,
+    const std::vector<std::string> &description) const {
   assert(def != Prompt_answer::ALT || !alt_label.empty());
 
-  Prompt_answer final_ans = Prompt_answer::NONE;
+  shcore::prompt::Prompt_options options;
+  options.title = title;
+  options.type = shcore::prompt::Prompt_type::CONFIRM;
+  options.description = description;
+  options.yes_label = yes_label;
+  options.no_label = no_label;
+  options.alt_label = alt_label;
+
+  if (def == Prompt_answer::YES) {
+    options.default_value = shcore::Value(yes_label);
+  } else if (def == Prompt_answer::NO) {
+    options.default_value = shcore::Value(no_label);
+  } else if (def == Prompt_answer::ALT) {
+    options.default_value = shcore::Value(alt_label);
+  }
+
+  options.done();
+
   std::string ans;
-  char yes_letter = 0;
-  char no_letter = 0;
-  char alt_letter = 0;
-  std::string def_str;
-  std::string clean_yes_text, clean_no_text, clean_alt_text;
-  if (yes_label == "&Yes" && no_label == "&No" && alt_label.empty()) {
-    std::string display_text;
-    yes_letter = process_label(yes_label, &display_text, &clean_yes_text);
-    no_letter = process_label(no_label, &display_text, &clean_no_text);
 
-    if (def == Prompt_answer::YES)
-      def_str = "[Y/n]: ";
-    else if (def == Prompt_answer::NO)
-      def_str = "[y/N]: ";
-    else
-      def_str = "[y/n]: ";
-  } else {
-    std::string display_text;
-    yes_letter = process_label(yes_label, &display_text, &clean_yes_text);
-    if (!display_text.empty()) def_str.append(display_text).append("/");
+  auto res = call_prompt(prompt, options, &ans, {},
+                         &shcore::Interpreter_delegate::prompt);
 
-    no_letter = process_label(no_label, &display_text, &clean_no_text);
-    if (!display_text.empty()) def_str.append(display_text).append("/");
-
-    alt_letter = process_label(alt_label, &display_text, &clean_alt_text);
-    if (!display_text.empty()) def_str.append(display_text).append("/");
-
-    def_str.pop_back();  // erase trailing /
-
-    switch (def) {
-      case Prompt_answer::YES:
-        def_str.append(" (default ").append(clean_yes_text).append("): ");
-        break;
-
-      case Prompt_answer::NO:
-        def_str.append(" (default ").append(clean_no_text).append("): ");
-        break;
-
-      case Prompt_answer::ALT:
-        def_str.append(" (default ").append(clean_alt_text).append("): ");
-        break;
-
-      default:
-        break;
+  Prompt_answer final_ans = Prompt_answer::NONE;
+  // TODO(rennox): Does it make sense to validate this returned value? iiuc
+  // call_prompt will ALWAYS return OK for confirm prompts.
+  if (res == shcore::Prompt_result::Ok) {
+    if (ans == yes_label) {
+      final_ans = Prompt_answer::YES;
+    } else if (ans == no_label) {
+      final_ans = Prompt_answer::NO;
+    } else if (ans == alt_label) {
+      final_ans = Prompt_answer::ALT;
     }
   }
 
-  while (final_ans == Prompt_answer::NONE) {
-    auto res = this->prompt(prompt + " " + def_str, &ans);
-    if (res == shcore::Prompt_result::Cancel)
-      throw shcore::cancelled("Cancelled");
-    // Since fixing the prompt we need to preserve original behavior of the
-    // functionality, which was repeating prompt when user did press ctrl+d.
-    if (res == shcore::Prompt_result::CTRL_D) continue;
-    if (res == shcore::Prompt_result::Ok) {
-      if (ans.empty()) {
-        final_ans = def;
-      } else {
-        if (shcore::str_caseeq(ans, std::string{&yes_letter, 1}) ||
-            shcore::str_caseeq(ans, clean_yes_text)) {
-          final_ans = Prompt_answer::YES;
-        } else if (!clean_no_text.empty() &&
-                   (shcore::str_caseeq(ans, std::string{&no_letter, 1}) ||
-                    shcore::str_caseeq(ans, clean_no_text))) {
-          final_ans = Prompt_answer::NO;
-        } else if (!clean_alt_text.empty() &&
-                   (shcore::str_caseeq(ans, std::string{&alt_letter, 1}) ||
-                    shcore::str_caseeq(ans, clean_alt_text))) {
-          final_ans = Prompt_answer::ALT;
-        } else {
-          println("\nPlease pick an option out of " + def_str);
-        }
-      }
-    } else {
-      break;
-    }
-  }
   return final_ans;
-}  // namespace mysqlsh
+}
 
-bool Shell_console::select(const std::string &prompt_text, std::string *result,
+bool Shell_console::select(const std::string &prompt_text, std::string *out_val,
                            const std::vector<std::string> &options,
                            size_t default_option, bool allow_custom,
-                           Validator validator) const {
-  std::string answer;
-  std::string default_str;
-  std::string text(prompt_text);
-  result->clear();
+                           Validator validator, const std::string &title,
+                           const std::vector<std::string> &description) const {
+  assert(default_option <= options.size());
 
-  if (default_option != 0)
-    text += " [" + std::to_string(default_option) + "]: ";
-
-  int index = 1;
-  for (const auto &option : options)
-    println(shcore::str_format("  %d) %s", index++, option.c_str()));
-
-  println();
-
-  bool valid = false;
-
-  mysqlshdk::utils::nullable<std::string> good_answer;
-
-  while (!valid && good_answer.is_null()) {
-    auto res = prompt(text, &answer);
-    if (res == shcore::Prompt_result::Cancel)
-      throw shcore::cancelled("Cancelled");
-    // Since fixing the prompt we need to preserve original behavior of the
-    // functionality, which was repeating prompt when user did press ctrl+d.
-    if (res == shcore::Prompt_result::CTRL_D) continue;
-    if (res == shcore::Prompt_result::Ok) {
-      int option = static_cast<int>(default_option);
-
-      try {
-        if (!answer.empty())
-          option = std::stoi(answer);
-        else
-          valid = allow_custom;
-
-        // The selection is a number from the list
-        if (option > 0 && option <= static_cast<int>(options.size())) {
-          answer = options[option - 1];
-          valid = true;
-        }
-      } catch (const std::exception &err) {
-        // User typed something else and it is allowed
-        valid = allow_custom;
-      }
-
-      // If there's a validator, the answer should be validated
-      std::string warning;
-      if (valid && validator) {
-        warning = validator(answer);
-        valid = warning.empty();
-      } else if (!valid) {
-        warning = "Invalid option selected.";
-      }
-
-      if (valid)
-        good_answer = answer;
-      else
-        print_warning(warning);
-    } else {
-      break;
-    }
+  shcore::prompt::Prompt_options prompt_options;
+  prompt_options.title = title;
+  prompt_options.type = shcore::prompt::Prompt_type::SELECT;
+  prompt_options.description = description;
+  prompt_options.select_items = options;
+  prompt_options.allow_custom = allow_custom;
+  if (default_option != 0) {
+    prompt_options.default_value =
+        shcore::Value(static_cast<unsigned int>(default_option));
   }
 
-  if (!good_answer.is_null()) *result = *good_answer;
+  prompt_options.done();
 
-  return valid;
+  call_prompt(prompt_text, prompt_options, out_val, validator,
+              &shcore::Interpreter_delegate::prompt);
+
+  // TODO(rennox): Not needed to validate anything as SELECT prompts always
+  // return true
+  return true;
 }
 
 shcore::Prompt_result Shell_console::prompt_password(
-    const std::string &prompt, std::string *out_val,
-    Validator validator) const {
-  return call_prompt(prompt, out_val, validator, "password",
-                     &shcore::Interpreter_delegate::password);
+    const std::string &prompt, std::string *out_val, Validator validator,
+    const std::string &title,
+    const std::vector<std::string> &description) const {
+  return this->prompt(prompt, out_val, validator,
+                      shcore::prompt::Prompt_type::PASSWORD, title,
+                      description);
 }
 
 void Shell_console::print_value(const shcore::Value &value,
