@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -24,7 +24,7 @@
 #include "shellcore/shell_core.h"
 #include <fstream>
 #include <locale>
-#ifdef WIN32
+#ifdef _WIN32
 #include <windows.h>
 #else
 #include <unistd.h>
@@ -60,10 +60,8 @@ REGISTER_HELP(COMMANDS_EXAMPLE_DESC,
               "Displays information about the <b>\\connect</b> command.");
 namespace shcore {
 
-Shell_core::Shell_core(mysqlsh::IConsole *console)
-    : IShell_core(), m_console(console) {
-  assert(m_console);
-
+Shell_core::Shell_core(bool interactive)
+    : IShell_core(), m_global_return_code(0), m_interactive(interactive) {
   DEBUG_OBJ_ALLOC(Shell_core);
   _mode = Mode::None;
   _registry = new Object_registry();
@@ -86,11 +84,16 @@ Shell_core::~Shell_core() {
   }
   _globals.clear();
 
-  if (_langs[Mode::JavaScript]) delete _langs[Mode::JavaScript];
+  auto remove_language = [this](Mode m) {
+    auto it = _langs.find(m);
+    if (it != _langs.end()) {
+      delete it->second;
+    }
+  };
 
-  if (_langs[Mode::Python]) delete _langs[Mode::Python];
-
-  if (_langs[Mode::SQL]) delete _langs[Mode::SQL];
+  remove_language(Mode::JavaScript);
+  remove_language(Mode::Python);
+  remove_language(Mode::SQL);
 }
 
 std::string Shell_core::preprocess_input_line(const std::string &s) {
@@ -98,17 +101,23 @@ std::string Shell_core::preprocess_input_line(const std::string &s) {
 }
 
 void Shell_core::handle_input(std::string &code, Input_state &state) {
-  try {
-    _langs[_mode]->handle_input(code, state);
-  } catch (...) {
-    throw;
-  }
+  _langs[_mode]->handle_input(code, state);
 }
 
+void Shell_core::flush_input(const std::string &code) {
+  _langs[_mode]->flush_input(code);
+}
+
+Input_state Shell_core::input_state() { return _langs[_mode]->input_state(); }
+
 std::string Shell_core::get_handled_input() {
   return _langs[_mode]->get_handled_input();
 }
 
+void Shell_core::set_argv(const std::vector<std::string> &args) {
+  return _langs[_mode]->set_argv(args);
+}
+
 /*
  * process_stream will process the content of an opened stream until EOF is
  * found.
@@ -117,27 +126,25 @@ std::string Shell_core::get_handled_input() {
  * - 1 in case of any processing error is found.
  * - 0 if no processing errors were found.
  */
-int Shell_core::process_stream(std::istream &stream, const std::string &source,
-                               const std::vector<std::string> &argv) {
+int Shell_core::process_stream(std::istream &stream,
+                               const std::string &source) {
   // NOTE: global return code is unused at the moment
   //       return code should be determined at application level on
   //       process_result this global return code may be used again once the
   //       exit() function is in place
   Input_state state = Input_state::Ok;
-  _global_return_code = 0;
+  m_global_return_code = 0;
 
   _input_source = source;
-  _input_args = argv;
-
   if (_mode == Shell_core::Mode::SQL) {
-    if (!_langs[_mode]->handle_input_stream(&stream)) _global_return_code = 1;
+    if (!_langs[_mode]->handle_input_stream(&stream)) m_global_return_code = 1;
   } else {
     std::string data;
     if (&std::cin == &stream) {
+      std::string line;
       while (!stream.eof()) {
-        std::string line;
-
-        std::getline(stream, line);
+        line.clear();
+        shcore::getline(stream, line);
         data.append(line).append("\n");
       }
     } else {
@@ -157,10 +164,7 @@ int Shell_core::process_stream(std::istream &stream, const std::string &source,
     handle_input(data, state);
   }
 
-  _input_source.clear();
-  _input_args.clear();
-
-  return _global_return_code;
+  return m_global_return_code;
 }
 
 bool Shell_core::switch_mode(Mode mode) {
@@ -300,14 +304,12 @@ std::shared_ptr<mysqlsh::ShellBaseSession> Shell_core::get_dev_session() {
 std::string Shell_core::get_main_delimiter() const {
   auto it = _langs.find(Mode::SQL);
   if (it == _langs.end()) return ";";
-  return dynamic_cast<Shell_sql *>(it->second)->get_main_delimiter();
+  return static_cast<Shell_sql *>(it->second)->get_main_delimiter();
 }
 
-void Shell_core::execute_module(const std::string &file_name,
+void Shell_core::execute_module(const std::string &module_name,
                                 const std::vector<std::string> &argv) {
-  _input_args = argv;
-
-  _langs[_mode]->execute_module(file_name);
+  _langs[_mode]->execute_module(module_name, argv);
 }
 
 bool Shell_core::load_plugin(Mode mode, const Plugin_definition &plugin) {
@@ -525,16 +527,19 @@ void Shell_command_handler::add_command(const std::string &triggers,
                                                       case_sensitive_help);
 
     if (topics.empty()) {
-      Help_topic *topic;
-      topic = Help_registry::get()->add_help_topic(
-          tokens[0], shcore::Topic_type::COMMAND, help_tag, "Commands", mode);
+      std::vector<Help_topic *> new_topics =
+          Help_registry::get()->add_help_topic(tokens[0],
+                                               shcore::Topic_type::COMMAND,
+                                               help_tag, "Commands", mode);
 
       // If case insensitive, first trigger is already registered
       if (!case_sensitive_help) tokens.erase(tokens.begin());
 
       for (auto &token : tokens) {
-        Help_registry::get()->register_keyword(token, mode, topic,
-                                               case_sensitive_help);
+        for (auto &topic : new_topics) {
+          Help_registry::get()->register_keyword(token, mode, topic,
+                                                 case_sensitive_help);
+        }
       }
 
       // If case sensitive, we need now to remove the first trigger
