@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -58,6 +58,8 @@ TEST(Connection_options, default_initialization) {
   ASSERT_FALSE(options.has_transport_type());
   ASSERT_FALSE(options.get_ssl_options().has_data());
   ASSERT_FALSE(options.has_compression());
+  ASSERT_FALSE(options.has_compression_algorithms());
+  ASSERT_FALSE(options.has_compression_level());
 
   // has verifies the existence of the option
   EXPECT_TRUE(options.has(mysqlshdk::db::kScheme));
@@ -285,21 +287,27 @@ TEST(Connection_options, uri_constructor) {
 }
 
 void combine(
-    const std::string &input, const std::string &twisted, size_t index,
+    const std::string &input,
     std::function<void(const std::string &, const std::string)> callback) {
-  std::string my_string(twisted);
-  if (my_string.empty()) my_string = shcore::str_lower(input);
+  std::string my_string(input);
 
-  if (index >= my_string.size()) {
-    callback(input, my_string);
-    return;
+  callback(input, my_string);
+
+  // Turn first character of each word to uppercase
+  bool convert = true;
+  for (size_t index = 0; index < my_string.size(); index++) {
+    if (convert) {
+      my_string[index] = std::toupper(my_string[index]);
+      convert = false;
+      callback(input, my_string);
+    } else {
+      convert = my_string[index] == '-';
+    }
   }
 
-  my_string[index] = std::tolower(my_string[index]);
-  combine(input, twisted, index + 1, callback);
-
-  my_string[index] = std::toupper(my_string[index]);
-  combine(input, my_string, index + 1, callback);
+  // Now try with all characters uppercase
+  my_string = shcore::str_upper(my_string);
+  callback(input, my_string);
 }
 
 namespace case_insensitive {
@@ -326,7 +334,7 @@ TEST(Connection_options, case_insensitive_ssl_mode) {
   auto callback = std::bind(case_insensitive::callback, std::placeholders::_1,
                             std::placeholders::_2, "REQUIRED");
 
-  combine(mysqlshdk::db::kSslMode, "", 0, callback);
+  combine(mysqlshdk::db::kSslMode, callback);
 }
 
 TEST(Connection_options, case_insensitive_get_server_public_key) {
@@ -335,9 +343,7 @@ TEST(Connection_options, case_insensitive_get_server_public_key) {
   auto callback = std::bind(case_insensitive::callback, std::placeholders::_1,
                             std::placeholders::_2, "true");
 
-  // We start from 10th index because this function has O(2^n) complexity
-  // and tested string length is 21 ("get-server-public-key").
-  combine(mysqlshdk::db::kGetServerPublicKey, "", 10, callback);
+  combine(mysqlshdk::db::kGetServerPublicKey, callback);
 }
 
 TEST(Connection_options, case_insensitive_server_public_key_path) {
@@ -346,9 +352,7 @@ TEST(Connection_options, case_insensitive_server_public_key_path) {
   auto callback = std::bind(case_insensitive::callback, std::placeholders::_1,
                             std::placeholders::_2, "pubkey.pem");
 
-  // We start from 11th index because this function has O(2^n) complexity
-  // and tested string length is 22 ("server-public-key-path").
-  combine(mysqlshdk::db::kServerPublicKeyPath, "", 11, callback);
+  combine(mysqlshdk::db::kServerPublicKeyPath, callback);
 }
 
 TEST(Connection_options, case_insensitive_options) {
@@ -366,10 +370,11 @@ TEST(Connection_options, case_insensitive_options) {
   attributes.erase(mysqlshdk::db::kServerPublicKeyPath);
   attributes.erase(mysqlshdk::db::kConnectTimeout);
   attributes.erase(mysqlshdk::db::kCompression);
+  attributes.erase(mysqlshdk::db::kCompressionLevel);
   attributes.erase(mysqlshdk::db::kConnectionAttributes);
 
   for (auto property : attributes) {
-    combine(property, "", 0, callback);
+    combine(property, callback);
   }
 }
 
@@ -380,7 +385,7 @@ TEST(Connection_options, connect_timeout) {
   auto callback = std::bind(case_insensitive::callback, std::placeholders::_1,
                             std::placeholders::_2, "1000");
 
-  combine(mysqlshdk::db::kConnectTimeout, "", 0, callback);
+  combine(mysqlshdk::db::kConnectTimeout, callback);
 
   // Test rejection of invalid values
   auto invalid_values = {"-1", "-100", "10.0", "whatever"};
@@ -424,9 +429,9 @@ TEST(Connection_options, compression) {
   // This callback will get called with every combination of
   // uppercase/lowercase letters for connect-timeout
   auto callback = std::bind(case_insensitive::callback, std::placeholders::_1,
-                            std::placeholders::_2, "true");
+                            std::placeholders::_2, "REQUIRED");
 
-  combine(mysqlshdk::db::kCompression, "", 0, callback);
+  combine(mysqlshdk::db::kCompression, callback);
 
   // Test rejection of invalid values
   auto invalid_values = {"-1", "-100", "10.0", "whatever"};
@@ -437,7 +442,9 @@ TEST(Connection_options, compression) {
 
     std::string msg("Invalid value '");
     msg.append(value);
-    msg.append("' for 'compression'. Allowed values: true, false, 1, 0.");
+    msg.append(
+        "' for 'compression'. Allowed values: 'REQUIRED', 'PREFERRED', "
+        "'DISABLED', 'True', 'False', '1', and '0'.");
 
     std::string uri_msg("Invalid URI: ");
     uri_msg.append(msg);
@@ -450,7 +457,8 @@ TEST(Connection_options, compression) {
   }
 
   // Tests acceptance of valid values
-  std::vector<std::string> valid_values = {"1", "true", "0", "false"};
+  std::vector<std::string> valid_values = {
+      "1", "true", "0", "false", "required", "Disabled", "PREFERRED"};
 
   for (const auto &value : valid_values) {
     std::string uri("root@host?compression=");
@@ -458,10 +466,14 @@ TEST(Connection_options, compression) {
 
     EXPECT_NO_THROW({
       mysqlshdk::db::Connection_options data(uri);
-      if (shcore::lexical_cast<bool>(value))
-        EXPECT_TRUE(data.get_compression());
-      else
-        EXPECT_FALSE(data.get_compression());
+      try {
+        if (shcore::lexical_cast<bool>(value))
+          EXPECT_EQ("REQUIRED", data.get_compression());
+        else
+          EXPECT_EQ("DISABLED", data.get_compression());
+      } catch (...) {
+        EXPECT_EQ(shcore::str_upper(value), data.get_compression());
+      }
     });
 
     mysqlshdk::db::Connection_options sample;
@@ -469,6 +481,41 @@ TEST(Connection_options, compression) {
   }
 }
 
+TEST(Connection_options, compression_level) {
+  auto invalid_values = {"-1e", "-0.5", "10.0", "whatever"};
+
+  for (const auto &value : invalid_values) {
+    std::string uri("root@host?compression-level=");
+    uri.append(value);
+
+    std::string msg = "The value of 'compression-level' must be an integer.";
+    std::string uri_msg("Invalid URI: ");
+    uri_msg.append(msg);
+    MY_EXPECT_THROW(std::invalid_argument, uri_msg.c_str(),
+                    { mysqlshdk::db::Connection_options data(uri); });
+
+    mysqlshdk::db::Connection_options sample;
+    MY_EXPECT_THROW(std::invalid_argument, msg.c_str(),
+                    sample.set(mysqlshdk::db::kCompressionLevel, value));
+  }
+
+  // Tests acceptance of valid values
+  std::vector<std::string> valid_values = {"-10", "1", "7", "22", "200"};
+
+  for (const auto &value : valid_values) {
+    std::string uri("root@host?compression-level=");
+    uri.append(value);
+
+    EXPECT_NO_THROW({
+      mysqlshdk::db::Connection_options data(uri);
+      EXPECT_EQ(shcore::lexical_cast<int>(value), data.get_compression_level());
+    });
+
+    mysqlshdk::db::Connection_options sample;
+    EXPECT_NO_THROW(sample.set(mysqlshdk::db::kCompressionLevel, {value}));
+  }
+}
+
 TEST(Connection_options, set_host) {
   // localhost does not determine the session type
   {
@@ -626,7 +673,8 @@ TEST(Connection_options, set_socket) {
 TEST(Connection_options, case_insensitive_duplicated_attribute) {
   // This callback will get called with every combination of
   // uppercase/lowercase letters on ssl-mode
-  auto callback = [](const std::string &property, const std::string &twisted) {
+  auto callback = [](const std::string & /* property */,
+                     const std::string &twisted) {
     mysqlshdk::db::Connection_options data;
     data.set(mysqlshdk::db::kAuthMethod, "Value");
 
@@ -638,13 +686,14 @@ TEST(Connection_options, case_insensitive_duplicated_attribute) {
                     data.set(twisted, "Whatever"));
   };
 
-  combine(mysqlshdk::db::kAuthMethod, "", 0, callback);
+  combine(mysqlshdk::db::kAuthMethod, callback);
 }
 
 TEST(Connection_options, invalid_options_after_WL10912) {
   // This callback will get called with every combination of
   // uppercase/lowercase letters for each property
-  auto callback = [](const std::string &property, const std::string &twisted) {
+  auto callback = [](const std::string & /* property */,
+                     const std::string &twisted) {
     std::string uri = "mysql://root@host?" + twisted + "=whatever";
     std::string message =
         "Invalid URI: Invalid connection option '" + twisted + "'.";
@@ -657,7 +706,7 @@ TEST(Connection_options, invalid_options_after_WL10912) {
       "sslMode",   "sslCa",   "sslCaPath", "sslCrl",     "sslCrlPath",
       "sslCipher", "sslCert", "sslKey",    "authMethod", "sslTlsVersion"};
 
-  for (auto property : invalid_options) combine(property, "", 0, callback);
+  for (auto property : invalid_options) combine(property, callback);
 }
 
 }  // namespace testing
