@@ -2,48 +2,46 @@
 ||
 
 //@<OUT> X Table Format
-+-------+
-| data  |
-+-------+
-| ab cd |
-| ab	cd |
-| ab
-cd |
-+-------+
+table
++--------------+
+| data         |
++--------------+
+| 0x6162006364 |
+| 0x6162096364 |
+| 0x61620A6364 |
++--------------+
 
 
 //@<OUT> X Vertical Format
 *************************** 1. row ***************************
-data: ab cd
+data: 0x6162006364
 *************************** 2. row ***************************
-data: ab	cd
+data: 0x6162096364
 *************************** 3. row ***************************
-data: ab
-cd
-
+data: 0x61620A6364
 
 //@<OUT> X Tabbed Format
 data
-ab\0cd
-ab\tcd
-ab\ncd
+0x6162006364
+0x6162096364
+0x61620A6364
 
 //@<OUT> X Json Format
 {
-    "data": "ab\u0000cd"
+    "data": "YWIAY2Q="
 }
 {
-    "data": "ab\tcd"
+    "data": "YWIJY2Q="
 }
 {
-    "data": "ab\ncd"
+    "data": "YWIKY2Q="
 }
 3 rows in set ([[*]] sec)
 
 //@<OUT> X Raw Json Format
-{"data":"ab\u0000cd"}
-{"data":"ab\tcd"}
-{"data":"ab\ncd"}
+{"data":"YWIAY2Q="}
+{"data":"YWIJY2Q="}
+{"data":"YWIKY2Q="}
 3 rows in set ([[*]] sec)
 
 //@<OUT> X Json Wrapping
@@ -51,13 +49,13 @@ ab\ncd
     "hasData": true,
     "rows": [
         {
-            "data": "ab\u0000cd"
+            "data": "YWIAY2Q="
         },
         {
-            "data": "ab\tcd"
+            "data": "YWIJY2Q="
         },
         {
-            "data": "ab\ncd"
+            "data": "YWIKY2Q="
         }
     ],
     "executionTime": "[[*]] sec",
@@ -71,46 +69,44 @@ ab\ncd
 }
 
 //@<OUT> Classic Table Format
-+-------+
-| data  |
-+-------+
-| ab cd |
-| ab	cd |
-| ab
-cd |
-+-------+
++--------------+
+| data         |
++--------------+
+| 0x6162006364 |
+| 0x6162096364 |
+| 0x61620A6364 |
++--------------+
 
 //@<OUT> Classic Vertical Format
 *************************** 1. row ***************************
-data: ab cd
+data: 0x6162006364
 *************************** 2. row ***************************
-data: ab	cd
+data: 0x6162096364
 *************************** 3. row ***************************
-data: ab
-cd
+data: 0x61620A6364
 
 //@<OUT> Classic Tabbed Format
 data
-ab\0cd
-ab\tcd
-ab\ncd
+0x6162006364
+0x6162096364
+0x61620A6364
 
 //@<OUT> Classic Json Format
 {
-    "data": "ab\u0000cd"
+    "data": "YWIAY2Q="
 }
 {
-    "data": "ab\tcd"
+    "data": "YWIJY2Q="
 }
 {
-    "data": "ab\ncd"
+    "data": "YWIKY2Q="
 }
 3 rows in set ([[*]] sec)
 
 //@<OUT> Classic Raw Json Format
-{"data":"ab\u0000cd"}
-{"data":"ab\tcd"}
-{"data":"ab\ncd"}
+{"data":"YWIAY2Q="}
+{"data":"YWIJY2Q="}
+{"data":"YWIKY2Q="}
 3 rows in set ([[*]] sec)
 
 //@<OUT> Classic Json Wrapping
@@ -118,13 +114,13 @@ ab\ncd
     "hasData": true,
     "rows": [
         {
-            "data": "ab\u0000cd"
+            "data": "YWIAY2Q="
         },
         {
-            "data": "ab\tcd"
+            "data": "YWIJY2Q="
         },
         {
-            "data": "ab\ncd"
+            "data": "YWIKY2Q="
         }
     ],
     "executionTime": "[[*]] sec",
@@ -288,22 +284,22 @@ data: ®7⃣⏰☕♒♣ ⛽🌄🌠🎨🐍🐾
 +--------------------------------------------------------+-----+
 
 //@<OUT> pulling as table in table format {__os_type != "windows" && VER(>= 8.0.19)}
-+--------------------------------------------------------+-----+--------------------+
-| doc                                                    | _id | _json_schema       |
-+--------------------------------------------------------+-----+--------------------+
-| {"_id": "1", "name": "生活是美好的", "year": 1997}     | 1   | {"type": "object"} |
-| {"_id": "10", "name": "®7⃣⏰☕♒♣⛽🌄🌠🎨🐍🐾", "year": 2004}    | 10  | {"type": "object"} |
-| {"_id": "11", "name": "pure ascii text", "year": 2014} | 11  | {"type": "object"} |
-| {"_id": "12", "name": "látiñ text row", "year": 2016}  | 12  | {"type": "object"} |
-| {"_id": "2", "name": "辛德勒的名单", "year": 1993}     | 2   | {"type": "object"} |
-| {"_id": "3", "name": "指環王", "year": 2001}           | 3   | {"type": "object"} |
-| {"_id": "4", "name": "尋找尼莫", "year": 2003}         | 4   | {"type": "object"} |
-| {"_id": "5", "name": "الجنة الآن", "year": 2003}       | 5   | {"type": "object"} |
-| {"_id": "6", "name": "😁😍😠😭🙅🙉", "year": 2004}           | 6   | {"type": "object"} |
-| {"_id": "7", "name": "✅✨✋✈❄❔➗", "year": 2004}          | 7   | {"type": "object"} |
-| {"_id": "8", "name": "🚀🚑🚙🚬🚻🛀", "year": 2004}           | 8   | {"type": "object"} |
-| {"_id": "9", "name": "🇯🇵🈳🆕🆒", "year": 2004}            | 9   | {"type": "object"} |
-+--------------------------------------------------------+-----+--------------------+
++--------------------------------------------------------+--------+--------------------+
+| doc                                                    | _id    | _json_schema       |
++--------------------------------------------------------+--------+--------------------+
+| {"_id": "1", "name": "生活是美好的", "year": 1997}     | 0x31   | {"type": "object"} |
+| {"_id": "10", "name": "®7⃣⏰☕♒♣⛽🌄🌠🎨🐍🐾", "year": 2004}    | 0x3130 | {"type": "object"} |
+| {"_id": "11", "name": "pure ascii text", "year": 2014} | 0x3131 | {"type": "object"} |
+| {"_id": "12", "name": "látiñ text row", "year": 2016}  | 0x3132 | {"type": "object"} |
+| {"_id": "2", "name": "辛德勒的名单", "year": 1993}     | 0x32   | {"type": "object"} |
+| {"_id": "3", "name": "指環王", "year": 2001}           | 0x33   | {"type": "object"} |
+| {"_id": "4", "name": "尋找尼莫", "year": 2003}         | 0x34   | {"type": "object"} |
+| {"_id": "5", "name": "الجنة الآن", "year": 2003}       | 0x35   | {"type": "object"} |
+| {"_id": "6", "name": "😁😍😠😭🙅🙉", "year": 2004}           | 0x36   | {"type": "object"} |
+| {"_id": "7", "name": "✅✨✋✈❄❔➗", "year": 2004}          | 0x37   | {"type": "object"} |
+| {"_id": "8", "name": "🚀🚑🚙🚬🚻🛀", "year": 2004}           | 0x38   | {"type": "object"} |
+| {"_id": "9", "name": "🇯🇵🈳🆕🆒", "year": 2004}            | 0x39   | {"type": "object"} |
++--------------------------------------------------------+--------+--------------------+
 
 //@<OUT> pulling as table in table format {__os_type == "windows" && VER(< 8.0.19)}
 +----------------------------------------------------------+-----+
@@ -324,22 +320,22 @@ data: ®7⃣⏰☕♒♣ ⛽🌄🌠🎨🐍🐾
 +----------------------------------------------------------+-----+
 
 //@<OUT> pulling as table in table format {__os_type == "windows" && VER(>= 8.0.19)}
-+----------------------------------------------------------+-----+--------------------+
-| doc                                                      | _id | _json_schema       |
-+----------------------------------------------------------+-----+--------------------+
-| {"_id": "1", "name": "生活是美好的", "year": 1997}       | 1   | {"type": "object"} |
-| {"_id": "10", "name": "®7⃣⏰☕♒♣⛽🌄🌠🎨🐍🐾", "year": 2004} | 10  | {"type": "object"} |
-| {"_id": "11", "name": "pure ascii text", "year": 2014}   | 11  | {"type": "object"} |
-| {"_id": "12", "name": "látiñ text row", "year": 2016}    | 12  | {"type": "object"} |
-| {"_id": "2", "name": "辛德勒的名单", "year": 1993}       | 2   | {"type": "object"} |
-| {"_id": "3", "name": "指環王", "year": 2001}             | 3   | {"type": "object"} |
-| {"_id": "4", "name": "尋找尼莫", "year": 2003}           | 4   | {"type": "object"} |
-| {"_id": "5", "name": "الجنة الآن", "year": 2003}         | 5   | {"type": "object"} |
-| {"_id": "6", "name": "😁😍😠😭🙅🙉", "year": 2004}       | 6   | {"type": "object"} |
-| {"_id": "7", "name": "✅✨✋✈❄❔➗", "year": 2004}            | 7   | {"type": "object"} |
-| {"_id": "8", "name": "🚀🚑🚙🚬🚻🛀", "year": 2004}       | 8   | {"type": "object"} |
-| {"_id": "9", "name": "🇯🇵🈳🆕🆒", "year": 2004}         | 9   | {"type": "object"} |
-+----------------------------------------------------------+-----+--------------------+
++----------------------------------------------------------+--------+--------------------+
+| doc                                                      | _id    | _json_schema       |
++----------------------------------------------------------+--------+--------------------+
+| {"_id": "1", "name": "生活是美好的", "year": 1997}       | 0x31   | {"type": "object"} |
+| {"_id": "10", "name": "®7⃣⏰☕♒♣⛽🌄🌠🎨🐍🐾", "year": 2004} | 0x3130 | {"type": "object"} |
+| {"_id": "11", "name": "pure ascii text", "year": 2014}   | 0x3131 | {"type": "object"} |
+| {"_id": "12", "name": "látiñ text row", "year": 2016}    | 0x3132 | {"type": "object"} |
+| {"_id": "2", "name": "辛德勒的名单", "year": 1993}       | 0x32   | {"type": "object"} |
+| {"_id": "3", "name": "指環王", "year": 2001}             | 0x33   | {"type": "object"} |
+| {"_id": "4", "name": "尋找尼莫", "year": 2003}           | 0x34   | {"type": "object"} |
+| {"_id": "5", "name": "الجنة الآن", "year": 2003}         | 0x35   | {"type": "object"} |
+| {"_id": "6", "name": "😁😍😠😭🙅🙉", "year": 2004}       | 0x36   | {"type": "object"} |
+| {"_id": "7", "name": "✅✨✋✈❄❔➗", "year": 2004}            | 0x37   | {"type": "object"} |
+| {"_id": "8", "name": "🚀🚑🚙🚬🚻🛀", "year": 2004}       | 0x38   | {"type": "object"} |
+| {"_id": "9", "name": "🇯🇵🈳🆕🆒", "year": 2004}         | 0x39   | {"type": "object"} |
++----------------------------------------------------------+--------+--------------------+
 
 //@<OUT> pulling as table in tabbed format {VER(< 8.0.19)}
 doc	_id
@@ -358,18 +354,18 @@ doc	_id
 
 //@<OUT> pulling as table in tabbed format {VER(>= 8.0.19)}
 doc	_id	_json_schema
-{"_id": "1", "name": "生活是美好的", "year": 1997}	1	{"type": "object"}
-{"_id": "10", "name": "®7⃣⏰☕♒♣⛽🌄🌠🎨🐍🐾", "year": 2004}	10	{"type": "object"}
-{"_id": "11", "name": "pure ascii text", "year": 2014}	11	{"type": "object"}
-{"_id": "12", "name": "látiñ text row", "year": 2016}	12	{"type": "object"}
-{"_id": "2", "name": "辛德勒的名单", "year": 1993}	2	{"type": "object"}
-{"_id": "3", "name": "指環王", "year": 2001}	3	{"type": "object"}
-{"_id": "4", "name": "尋找尼莫", "year": 2003}	4	{"type": "object"}
-{"_id": "5", "name": "الجنة الآن", "year": 2003}	5	{"type": "object"}
-{"_id": "6", "name": "😁😍😠😭🙅🙉", "year": 2004}	6	{"type": "object"}
-{"_id": "7", "name": "✅✨✋✈❄❔➗", "year": 2004}	7	{"type": "object"}
-{"_id": "8", "name": "🚀🚑🚙🚬🚻🛀", "year": 2004}	8	{"type": "object"}
-{"_id": "9", "name": "🇯🇵🈳🆕🆒", "year": 2004}	9	{"type": "object"}
+{"_id": "1", "name": "生活是美好的", "year": 1997}	0x31	{"type": "object"}
+{"_id": "10", "name": "®7⃣⏰☕♒♣⛽🌄🌠🎨🐍🐾", "year": 2004}	0x3130	{"type": "object"}
+{"_id": "11", "name": "pure ascii text", "year": 2014}	0x3131	{"type": "object"}
+{"_id": "12", "name": "látiñ text row", "year": 2016}	0x3132	{"type": "object"}
+{"_id": "2", "name": "辛德勒的名单", "year": 1993}	0x32	{"type": "object"}
+{"_id": "3", "name": "指環王", "year": 2001}	0x33	{"type": "object"}
+{"_id": "4", "name": "尋找尼莫", "year": 2003}	0x34	{"type": "object"}
+{"_id": "5", "name": "الجنة الآن", "year": 2003}	0x35	{"type": "object"}
+{"_id": "6", "name": "😁😍😠😭🙅🙉", "year": 2004}	0x36	{"type": "object"}
+{"_id": "7", "name": "✅✨✋✈❄❔➗", "year": 2004}	0x37	{"type": "object"}
+{"_id": "8", "name": "🚀🚑🚙🚬🚻🛀", "year": 2004}	0x38	{"type": "object"}
+{"_id": "9", "name": "🇯🇵🈳🆕🆒", "year": 2004}	0x39	{"type": "object"}
 
 //@<OUT> pulling as table in vertical format {VER(< 8.0.19)}
 *************************** 1. row ***************************
@@ -412,51 +408,51 @@ _id: 9
 //@<OUT> pulling as table in vertical format {VER(>= 8.0.19)}
 *************************** 1. row ***************************
          doc: {"_id": "1", "name": "生活是美好的", "year": 1997}
-         _id: 1
+         _id: 0x31
 _json_schema: {"type": "object"}
 *************************** 2. row ***************************
          doc: {"_id": "10", "name": "®7⃣⏰☕♒♣⛽🌄🌠🎨🐍🐾", "year": 2004}
-         _id: 10
+         _id: 0x3130
 _json_schema: {"type": "object"}
 *************************** 3. row ***************************
          doc: {"_id": "11", "name": "pure ascii text", "year": 2014}
-         _id: 11
+         _id: 0x3131
 _json_schema: {"type": "object"}
 *************************** 4. row ***************************
          doc: {"_id": "12", "name": "látiñ text row", "year": 2016}
-         _id: 12
+         _id: 0x3132
 _json_schema: {"type": "object"}
 *************************** 5. row ***************************
          doc: {"_id": "2", "name": "辛德勒的名单", "year": 1993}
-         _id: 2
+         _id: 0x32
 _json_schema: {"type": "object"}
 *************************** 6. row ***************************
          doc: {"_id": "3", "name": "指環王", "year": 2001}
-         _id: 3
+         _id: 0x33
 _json_schema: {"type": "object"}
 *************************** 7. row ***************************
          doc: {"_id": "4", "name": "尋找尼莫", "year": 2003}
-         _id: 4
+         _id: 0x34
 _json_schema: {"type": "object"}
 *************************** 8. row ***************************
          doc: {"_id": "5", "name": "الجنة الآن", "year": 2003}
-         _id: 5
+         _id: 0x35
 _json_schema: {"type": "object"}
 *************************** 9. row ***************************
          doc: {"_id": "6", "name": "😁😍😠😭🙅🙉", "year": 2004}
-         _id: 6
+         _id: 0x36
 _json_schema: {"type": "object"}
 *************************** 10. row ***************************
          doc: {"_id": "7", "name": "✅✨✋✈❄❔➗", "year": 2004}
-         _id: 7
+         _id: 0x37
 _json_schema: {"type": "object"}
 *************************** 11. row ***************************
          doc: {"_id": "8", "name": "🚀🚑🚙🚬🚻🛀", "year": 2004}
-         _id: 8
+         _id: 0x38
 _json_schema: {"type": "object"}
 *************************** 12. row ***************************
          doc: {"_id": "9", "name": "🇯🇵🈳🆕🆒", "year": 2004}
-         _id: 9
+         _id: 0x39
 _json_schema: {"type": "object"}
 
 
