@@ -64,6 +64,7 @@ Checking instance configuration...
 Configuration file <<<__sandbox1_cnf_path>>> will also be checked.
 
 NOTE: Some configuration options need to be fixed:
+?{VER(<8.0.21)}
 +-----------------+---------------+----------------+------------------------------------------------+
 | Variable        | Current Value | Required Value | Note                                           |
 +-----------------+---------------+----------------+------------------------------------------------+
@@ -98,6 +99,36 @@ NOTE: Please use the dba.configure_instance() command to repair these issues.
     ],
     "status": "error"
 }
+?{}
+?{VER(>=8.0.21)}
++-----------+---------------+----------------+-----------------------------------------------+
+| Variable  | Current Value | Required Value | Note                                          |
++-----------+---------------+----------------+-----------------------------------------------+
+| gtid_mode | OFF           | ON             | Update the config file and restart the server |
+| server_id | 0             | <unique ID>    | Update the config file and restart the server |
++-----------+---------------+----------------+-----------------------------------------------+
+
+Some variables need to be changed, but cannot be done dynamically on the server.
+NOTE: Please use the dba.configure_instance() command to repair these issues.
+
+{
+    "config_errors": [
+        {
+            "action": "config_update+restart",
+            "current": "OFF",
+            "option": "gtid_mode",
+            "required": "ON"
+        },
+        {
+            "action": "config_update+restart",
+            "current": "0",
+            "option": "server_id",
+            "required": "<unique ID>"
+        }
+    ],
+    "status": "error"
+}
+?{}
 
 #@<OUT> check_instance_configuration() - instance valid for cluster usage 2 {VER(<8.0.11)}
 Validating local MySQL instance listening at port <<<__mysql_sandbox_port2>>> for use in an InnoDB cluster...
@@ -113,7 +144,7 @@ Checking instance configuration...
 Configuration file <<<__sandbox2_cnf_path>>> will also be checked.
 Instance configuration is compatible with InnoDB cluster
 
-The instance 'localhost:<<<__mysql_sandbox_port2>>>' is valid for InnoDB cluster usage.
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is valid to be used in an InnoDB cluster.
 
 {
     "status": "ok"
@@ -133,7 +164,7 @@ Checking instance configuration...
 Configuration file <<<__sandbox2_cnf_path>>> will also be checked.
 Instance configuration is compatible with InnoDB cluster
 
-The instance 'localhost:<<<__mysql_sandbox_port2>>>' is valid for InnoDB cluster usage.
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is valid to be used in an InnoDB cluster.
 
 {
     "status": "ok"
@@ -153,7 +184,7 @@ Checking instance configuration...
 Configuration file <<<__sandbox3_cnf_path>>> will also be checked.
 Instance configuration is compatible with InnoDB cluster
 
-The instance 'localhost:<<<__mysql_sandbox_port3>>>' is valid for InnoDB cluster usage.
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port3>>>' is valid to be used in an InnoDB cluster.
 
 {
     "status": "ok"
@@ -173,7 +204,7 @@ Checking instance configuration...
 Configuration file <<<__sandbox3_cnf_path>>> will also be checked.
 Instance configuration is compatible with InnoDB cluster
 
-The instance 'localhost:<<<__mysql_sandbox_port3>>>' is valid for InnoDB cluster usage.
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port3>>>' is valid to be used in an InnoDB cluster.
 
 {
     "status": "ok"
@@ -187,7 +218,12 @@ Please note that sandbox instances are only suitable for deploying test clusters
 This instance reports its own address as <<<hostname>>>:<<<__mysql_sandbox_port1>>>
 Assuming full account name 'myAdmin'@'%' for myAdmin
 
+?{VER(>=8.0.23)}
+applierWorkerThreads will be set to the default value of 4.
+
+?{}
 NOTE: Some configuration options need to be fixed:
+?{VER(<8.0.21)}
 +-----------------+---------------+----------------+------------------------------------------------+
 | Variable        | Current Value | Required Value | Note                                           |
 +-----------------+---------------+----------------+------------------------------------------------+
@@ -197,11 +233,27 @@ NOTE: Some configuration options need to be fixed:
 +-----------------+---------------+----------------+------------------------------------------------+
 
 Some variables need to be changed, but cannot be done dynamically on the server.
-Do you want to perform the required configuration changes? [y/n]:
+Do you want to perform the required configuration changes? [y/n]: Do you want to restart the instance after configuring it? [y/n]:
 Cluster admin user 'myAdmin'@'%' created.
 Configuring instance...
-The instance 'localhost:<<<__mysql_sandbox_port1>>>' was configured for InnoDB cluster usage.
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' was configured to be used in an InnoDB cluster.
 NOTE: MySQL server needs to be restarted for configuration changes to take effect.
+?{}
+?{VER(>=8.0.21)}
++-----------+---------------+----------------+-----------------------------------------------+
+| Variable  | Current Value | Required Value | Note                                          |
++-----------+---------------+----------------+-----------------------------------------------+
+| gtid_mode | OFF           | ON             | Update the config file and restart the server |
+| server_id | 0             | <unique ID>    | Update the config file and restart the server |
++-----------+---------------+----------------+-----------------------------------------------+
+
+Some variables need to be changed, but cannot be done dynamically on the server.
+Do you want to perform the required configuration changes? [y/n]: Do you want to restart the instance after configuring it? [y/n]:
+Cluster admin user 'myAdmin'@'%' created.
+Configuring instance...
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' was configured to be used in an InnoDB cluster.
+NOTE: MySQL server needs to be restarted for configuration changes to take effect.
+?{}
 
 #@<OUT> configure_instance() - instance not valid for cluster usage {VER(<8.0.11)}
 Configuring local MySQL instance listening at port <<<__mysql_sandbox_port1>>> for use in an InnoDB cluster...
@@ -224,7 +276,7 @@ Some variables need to be changed, but cannot be done dynamically on the server:
 Do you want to perform the required configuration changes? [y/n]:
 Cluster admin user 'myAdmin'@'%' created.
 Configuring instance...
-The instance 'localhost:<<<__mysql_sandbox_port1>>>' was configured for InnoDB cluster usage.
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' was configured to be used in an InnoDB cluster.
 NOTE: MySQL server needs to be restarted for configuration changes to take effect.
 
 #@<OUT> configure_instance() - create admin account 2
@@ -235,7 +287,11 @@ Please note that sandbox instances are only suitable for deploying test clusters
 This instance reports its own address as <<<hostname>>>:<<<__mysql_sandbox_port2>>>
 Assuming full account name 'myAdmin'@'%' for myAdmin
 
-The instance 'localhost:<<<__mysql_sandbox_port2>>>' is valid for InnoDB cluster usage.
+?{VER(>=8.0.23)}
+applierWorkerThreads will be set to the default value of 4.
+
+?{}
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is valid to be used in an InnoDB cluster.
 
 Cluster admin user 'myAdmin'@'%' created.
 
@@ -247,7 +303,11 @@ Please note that sandbox instances are only suitable for deploying test clusters
 This instance reports its own address as <<<hostname>>>:<<<__mysql_sandbox_port3>>>
 Assuming full account name 'myAdmin'@'%' for myAdmin
 
-The instance 'localhost:<<<__mysql_sandbox_port3>>>' is valid for InnoDB cluster usage.
+?{VER(>=8.0.23)}
+applierWorkerThreads will be set to the default value of 4.
+
+?{}
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port3>>>' is valid to be used in an InnoDB cluster.
 
 Cluster admin user 'myAdmin'@'%' created.
 
@@ -264,7 +324,7 @@ No incompatible tables detected
 Checking instance configuration...
 Instance configuration is compatible with InnoDB cluster
 
-The instance 'localhost:<<<__mysql_sandbox_port1>>>' is valid for InnoDB cluster usage.
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' is valid to be used in an InnoDB cluster.
 
 {
     "status": "ok"
@@ -283,7 +343,7 @@ No incompatible tables detected
 Checking instance configuration...
 Instance configuration is compatible with InnoDB cluster
 
-The instance 'localhost:<<<__mysql_sandbox_port1>>>' is valid for InnoDB cluster usage.
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' is valid to be used in an InnoDB cluster.
 
 {
     "status": "ok"
@@ -297,8 +357,12 @@ Please note that sandbox instances are only suitable for deploying test clusters
 
 This instance reports its own address as <<<hostname>>>:<<<__mysql_sandbox_port2>>>
 
-The instance 'localhost:<<<__mysql_sandbox_port2>>>' is valid for InnoDB cluster usage.
-The instance 'localhost:<<<__mysql_sandbox_port2>>>' is already ready for InnoDB cluster usage.
+?{VER(>=8.0.23)}
+applierWorkerThreads will be set to the default value of 4.
+
+?{}
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is valid to be used in an InnoDB cluster.
+The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is already ready to be used in an InnoDB cluster.
 
 #@ configure_instance() 2 - instance already valid for cluster usage
 ||
@@ -306,13 +370,15 @@ The instance 'localhost:<<<__mysql_sandbox_port2>>>' is already ready for InnoDB
 #@<OUT> create_cluster() {VER(>=8.0.11)}
 A new InnoDB cluster will be created on instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>'.
 
-Validating instance at <<<hostname>>>:<<<__mysql_sandbox_port1>>>...
+Validating instance configuration at <<<hostname>>>:<<<__mysql_sandbox_port1>>>...
 NOTE: Instance detected as a sandbox.
 Please note that sandbox instances are only suitable for deploying test clusters for use within the same host.
 
 This instance reports its own address as <<<hostname>>>:<<<__mysql_sandbox_port1>>>
 
 Instance configuration is suitable.
+NOTE: Group Replication will communicate with other members using '<<<hostname>>>:<<<__mysql_sandbox_gr_port1>>>'. Use the localAddress option to override.
+
 Creating InnoDB cluster 'testCluster' on '<<<hostname>>>:<<<__mysql_sandbox_port1>>>'...
 
 Adding Seed Instance...
@@ -323,14 +389,16 @@ one server failure.
 #@<OUT> create_cluster() {VER(<8.0.11)}
 A new InnoDB cluster will be created on instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>'.
 
-Validating instance at <<<hostname>>>:<<<__mysql_sandbox_port1>>>...
+Validating instance configuration at <<<hostname>>>:<<<__mysql_sandbox_port1>>>...
 NOTE: Instance detected as a sandbox.
 Please note that sandbox instances are only suitable for deploying test clusters for use within the same host.
 
 This instance reports its own address as <<<hostname>>>:<<<__mysql_sandbox_port1>>>
 
 Instance configuration is suitable.
-WARNING: Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' cannot persist Group Replication configuration since MySQL version <<<__version>>> does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the <Dba>.configure_local_instance() command locally to persist the changes.
+NOTE: Group Replication will communicate with other members using '<<<hostname>>>:<<<__mysql_sandbox_gr_port1>>>'. Use the localAddress option to override.
+
+WARNING: Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' cannot persist Group Replication configuration since MySQL version <<<__version>>> does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the dba.configure_local_instance() command locally to persist the changes.
 Creating InnoDB cluster 'testCluster' on '<<<hostname>>>:<<<__mysql_sandbox_port1>>>'...
 
 Adding Seed Instance...
@@ -338,59 +406,44 @@ Cluster successfully created. Use Cluster.add_instance() to add MySQL instances.
 At least 3 instances are needed for the cluster to be able to withstand up to
 one server failure.
 
-#@<OUT> add_instance() 1 clone {VER(>=8.0.17)}
-NOTE: The target instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' has not been pre-provisioned (GTID set is
-empty), but the cluster was configured to assume that incremental distributed
-state recovery can correctly provision it in this case.
-The safest and most convenient way to provision a new instance is through
-automatic clone provisioning, which will completely overwrite the state of
-'<<<hostname>>>:<<<__mysql_sandbox_port2>>>' with a physical snapshot from an existing cluster member. To use
-this method by default, set the 'recoveryMethod' option to 'clone'.
-
-The incremental distributed state recovery may be safely used if you are sure
-all updates ever executed in the cluster were done with GTIDs enabled, there
-are no purged transactions and the new instance contains the same GTID set as
-the cluster or a subset of it. To use this method by default, set the
-'recoveryMethod' option to 'incremental'.
-
-Incremental distributed state recovery was selected because it seems to be safely usable.
-Validating instance at <<<hostname>>>:<<<__mysql_sandbox_port2>>>...
-NOTE: Instance detected as a sandbox.
-Please note that sandbox instances are only suitable for deploying test clusters for use within the same host.
-
-This instance reports its own address as <<<hostname>>>:<<<__mysql_sandbox_port2>>>
-
-Instance configuration is suitable.
-A new instance will be added to the InnoDB cluster. Depending on the amount of
-data on the cluster this might take from a few seconds to several hours.
-
-Adding instance to the cluster...
+#@# add_instance() 1 clone {VER(>=8.0.17)}
+|NOTE: The target instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' has not been|
+|The safest and most convenient way to provision a new instance is through|
+|automatic clone provisioning, which will completely overwrite the state of|
+|The incremental state recovery may be safely used if you are sure|
+|Incremental state recovery was selected because it seems to be safely usable.|
+|Validating instance configuration at <<<hostname>>>:<<<__mysql_sandbox_port2>>>...|
+|Instance configuration is suitable.|
+||
+|Adding instance to the cluster...|
 
 #@<OUT> add_instance() 1 clone {VER(>=8.0.17)}
-{{State recovery already finished for 'localhost:<<<__mysql_sandbox_port2>>>'|Incremental distributed state recovery is now in progress.}}
+{{State recovery already finished for '<<<hostname>>>:<<<__mysql_sandbox_port2>>>'|Incremental state recovery is now in progress.}}
 
 #@<OUT> add_instance() 1 clone {VER(>=8.0.17)}
 The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was successfully added to the cluster.
 
 #@<OUT> add_instance() 1 {VER(<8.0.11)}
-Validating instance at <<<hostname>>>:<<<__mysql_sandbox_port2>>>...
+Validating instance configuration at <<<hostname>>>:<<<__mysql_sandbox_port2>>>...
 NOTE: Instance detected as a sandbox.
 Please note that sandbox instances are only suitable for deploying test clusters for use within the same host.
 
 This instance reports its own address as <<<hostname>>>:<<<__mysql_sandbox_port2>>>
 
 Instance configuration is suitable.
-WARNING: Instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' cannot persist Group Replication configuration since MySQL version <<<__version>>> does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the <Dba>.configure_local_instance() command locally to persist the changes.
+NOTE: Group Replication will communicate with other members using '<<<hostname>>>:<<<__mysql_sandbox_port2>>>1'. Use the localAddress option to override.
+
 A new instance will be added to the InnoDB cluster. Depending on the amount of
 data on the cluster this might take from a few seconds to several hours.
 
+WARNING: Instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' cannot persist Group Replication configuration since MySQL version <<<__version>>> does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the dba.configure_local_instance() command locally to persist the changes.
 Adding instance to the cluster...
 
 #@<OUT> add_instance() 1 {VER(<8.0.11)}
-{{State recovery already finished for 'localhost:<<<__mysql_sandbox_port2>>>'|Incremental distributed state recovery is now in progress.}}
+{{State recovery already finished for '<<<hostname>>>:<<<__mysql_sandbox_port2>>>'|Incremental state recovery is now in progress.}}
 
 #@<OUT> add_instance() 1 {VER(<8.0.11)}
-WARNING: Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' cannot persist configuration since MySQL version <<<__version>>> does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the <Dba>.configure_local_instance() command locally to persist the changes.
+WARNING: Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' cannot persist configuration since MySQL version <<<__version>>> does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the dba.configure_local_instance() command locally to persist the changes.
 The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was successfully added to the cluster.
 
 #@ add_instance() 2
@@ -405,17 +458,17 @@ The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was successfully added
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
-                "role": "HA"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
-                "role": "HA"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "role": "HA"
             },
             {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
                 "label": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
-                "role": "HA"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "role": "HA"
             }
         ],
         "topologyMode": "Single-Primary"
@@ -434,24 +487,30 @@ The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was successfully added
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/W",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port3>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             }
         },
         "topologyMode": "Single-Primary"
@@ -477,17 +536,21 @@ The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was successfully added
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/W",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             }
         },
         "topologyMode": "Single-Primary"
@@ -519,24 +582,30 @@ The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was successfully added
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/W",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port3>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             }
         },
         "topologyMode": "Single-Primary"
@@ -571,24 +640,30 @@ The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was successfully added
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/W",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port3>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             }
         },
         "topologyMode": "Single-Primary"
@@ -610,17 +685,20 @@ The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was successfully added
         "primary": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
         "ssl": "DISABLED",
         "status": "NO_QUORUM",
-        "statusText": "Cluster has no quorum as visible from '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' and cannot process write transactions. 2 members are not active",
+        "statusText": "Cluster has no quorum as visible from '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' and cannot process write transactions. 2 members are not active.",
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "n/a",
                 "readReplicas": {},
                 "role": "HA",
@@ -629,6 +707,8 @@ The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was successfully added
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port3>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
+                "memberRole": "SECONDARY",
+                "memberState": "(MISSING)",
                 "mode": "n/a",
                 "readReplicas": {},
                 "role": "HA",
@@ -642,13 +722,13 @@ The instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' was successfully added
 }
 
 #@<OUT> Restore the cluster quorum
-Restoring replicaset 'default' from loss of quorum, by using the partition composed of [<<<hostname>>>:<<<__mysql_sandbox_port1>>>]
+Restoring cluster 'testCluster' from loss of quorum, by using the partition composed of [<<<hostname>>>:<<<__mysql_sandbox_port1>>>]
 
 Restoring the InnoDB cluster ...
 
 The InnoDB cluster was successfully restored using the partition from the instance 'myAdmin@<<<hostname>>>:<<<__mysql_sandbox_port1>>>'.
 
-WARNING: To avoid a split-brain scenario, ensure that all other members of the replicaset are removed or joined back to the group that was restored.
+WARNING: To avoid a split-brain scenario, ensure that all other members of the cluster are removed or joined back to the group that was restored.
 
 #@<OUT> Check the cluster status after restoring the quorum
 {
@@ -658,17 +738,20 @@ WARNING: To avoid a split-brain scenario, ensure that all other members of the r
         "primary": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
         "ssl": "DISABLED",
         "status": "OK_NO_TOLERANCE",
-        "statusText": "Cluster is NOT tolerant to any failures. 2 members are not active",
+        "statusText": "Cluster is NOT tolerant to any failures. 2 members are not active.",
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/W",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "n/a",
                 "readReplicas": {},
                 "role": "HA",
@@ -677,6 +760,7 @@ WARNING: To avoid a split-brain scenario, ensure that all other members of the r
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port3>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
+                "memberRole": "SECONDARY",
                 "mode": "n/a",
                 "readReplicas": {},
                 "role": "HA",
@@ -707,24 +791,30 @@ WARNING: To avoid a split-brain scenario, ensure that all other members of the r
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/W",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port3>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             }
         },
         "topologyMode": "Single-Primary"
@@ -768,24 +858,30 @@ WARNING: To avoid a split-brain scenario, ensure that all other members of the r
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/W",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port3>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port3>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             }
         },
         "topologyMode": "Single-Primary"
@@ -817,17 +913,21 @@ WARNING: To avoid a split-brain scenario, ensure that all other members of the r
         "topology": {
             "<<<hostname>>>:<<<__mysql_sandbox_port1>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port1>>>",
+                "memberRole": "PRIMARY",
                 "mode": "R/W",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             },
             "<<<hostname>>>:<<<__mysql_sandbox_port2>>>": {
                 "address": "<<<hostname>>>:<<<__mysql_sandbox_port2>>>",
+                "memberRole": "SECONDARY",
                 "mode": "R/O",
-                "readReplicas": {},
+                "readReplicas": {},<<<"\n                \"replicationLag\": [[*]]," if (__version_num>=80011) else "">>>
                 "role": "HA",
-                "status": "ONLINE"<<<",\n[[*]]\"version\": \"" + __version + "\"" if (__version_num>=80011) else "">>>
+                "status": "ONLINE",
+                "version": "[[*]]"
             }
         },
         "topologyMode": "Single-Primary"
@@ -836,17 +936,17 @@ WARNING: To avoid a split-brain scenario, ensure that all other members of the r
 }
 
 #@<OUT> Dissolve cluster {VER(<8.0.11)}
-Instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is attempting to leave the cluster...
+* Instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is attempting to leave the cluster...
 WARNING: On instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' configuration cannot be persisted since MySQL version <<<__version>>> does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please set the 'group_replication_start_on_boot' variable to 'OFF' in the server configuration file, otherwise it might rejoin the cluster upon restart.
-Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' is attempting to leave the cluster...
+* Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' is attempting to leave the cluster...
 WARNING: On instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' configuration cannot be persisted since MySQL version <<<__version>>> does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please set the 'group_replication_start_on_boot' variable to 'OFF' in the server configuration file, otherwise it might rejoin the cluster upon restart.
 
 The cluster was successfully dissolved.
 Replication was disabled but user data was left intact.
 
 #@<OUT> Dissolve cluster {VER(>=8.0.11)}
-Instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is attempting to leave the cluster...
-Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' is attempting to leave the cluster...
+* Instance '<<<hostname>>>:<<<__mysql_sandbox_port2>>>' is attempting to leave the cluster...
+* Instance '<<<hostname>>>:<<<__mysql_sandbox_port1>>>' is attempting to leave the cluster...
 
 The cluster was successfully dissolved.
 Replication was disabled but user data was left intact.
