@@ -1,6 +1,5 @@
-// Assumptions: validateMember exists
-
-var mysql = require('mysql');
+// Assumptions: validateMembers exists
+var mysql=require('mysql');
 
 // The tests assume the next variables have been put in place
 // on the JS Context
@@ -9,14 +8,13 @@ var mysql = require('mysql');
 // __port: <port>
 // __user: <user>
 // __uripwd: <uri>:<pwd>@<host>
-// __pwd: <pwd>
 
-//@ mysql module: exports
-var exports = dir(mysql);
-print('Exported Items:', exports.length);
-validateMember(exports, 'getClassicSession');
-validateMember(exports, 'getSession');
-validateMember(exports, 'help');
+//@<> mysql module: exports
+validateMembers(mysql, [
+    'getClassicSession',
+    'ErrorCode',
+    'getSession',
+    'help'])
 
 //@# getClassicSession errors
 mysql.getClassicSession()
@@ -29,3 +27,6 @@ mysql.getSession()
 mysql.getSession(1, 2, 3)
 mysql.getSession(["bla"])
 mysql.getSession("some@uri", 25)
+
+//@<> ErrorCode
+EXPECT_EQ(1045, mysql.ErrorCode.ER_ACCESS_DENIED_ERROR)
