@@ -15,6 +15,7 @@ validate_members(mySession, [
   'get_default_schema',
   'get_schema',
   'get_schemas',
+  'get_ssh_uri',
   'get_uri',
   'help',
   'is_open',
@@ -29,6 +30,7 @@ validate_members(mySession, [
   'uri',
   'current_schema',
   'set_savepoint',
+  'ssh_uri',
   'release_savepoint',
   'rollback_to',
   '_enable_notices',
