@@ -1,23 +1,25 @@
-/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
-
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License, version 2.0,
- as published by the Free Software Foundation.
-
- This program is also distributed with certain software (including
- but not limited to OpenSSL) that is licensed under separate terms, as
- designated in a particular file or component or in included license
- documentation.  The authors of MySQL hereby grant you an additional
- permission to link the program and your derivative works with the
- separately licensed software that they have included with MySQL.
- This program is distributed in the hope that it will be useful,  but
- WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
- the GNU General Public License, version 2.0, for more details.
-
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software Foundation, Inc.,
- 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
+/*
+ * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, version 2.0,
+ * as published by the Free Software Foundation.
+ *
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have included with MySQL.
+ * This program is distributed in the hope that it will be useful,  but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+ * the GNU General Public License, version 2.0, for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <stdarg.h>
 #include <cstdio>
@@ -86,8 +88,18 @@ class Shell_cmdline_options : public tests::Shell_base_test {
       return options->host;
     else if (option == "user")
       return options->user;
-    else if (option == "password")
-      return options->password ? std::string(options->password) : "";
+    else if (option == "password" || option == "password1")
+      return options->mfa_passwords[0].has_value()
+                 ? std::string(*options->mfa_passwords[0])
+                 : "";
+    else if (option == "password2")
+      return options->mfa_passwords[1].has_value()
+                 ? std::string(*options->mfa_passwords[1])
+                 : "";
+    else if (option == "password3")
+      return options->mfa_passwords[2].has_value()
+                 ? std::string(*options->mfa_passwords[2])
+                 : "";
     else if (option == "port")
       return AS__STRING(options->port);
     else if (option == "schema")
@@ -147,7 +159,7 @@ class Shell_cmdline_options : public tests::Shell_base_test {
     else if (option == "run_file")
       return options->run_file;
     else if (option == "connect-timeout")
-      return options->m_connect_timeout;
+      return options->connect_timeout_cmdline;
     else if (option == "quiet-start")
       return AS__STRING(static_cast<int>(options->quiet_start));
     else if (option == "showColumnTypeInfo")
@@ -672,7 +684,9 @@ TEST_F(Shell_cmdline_options, default_values) {
   EXPECT_EQ(options.log_level, shcore::Logger::LOG_INFO);
   EXPECT_EQ("table", options.result_format);
   EXPECT_EQ("off", options.wrap_json);
-  EXPECT_EQ(nullptr, options.password);
+  EXPECT_FALSE(options.mfa_passwords[0].has_value());
+  EXPECT_FALSE(options.mfa_passwords[1].has_value());
+  EXPECT_FALSE(options.mfa_passwords[2].has_value());
   EXPECT_FALSE(options.passwords_from_stdin);
   EXPECT_EQ(options.port, 0);
   EXPECT_FALSE(options.prompt_password);
@@ -697,7 +711,7 @@ TEST_F(Shell_cmdline_options, default_values) {
   EXPECT_TRUE(options.execute_statement.empty());
   EXPECT_TRUE(options.wizards);
   EXPECT_TRUE(options.default_session_type);
-  EXPECT_TRUE(options.m_connect_timeout.empty());
+  EXPECT_TRUE(options.connect_timeout_cmdline.empty());
   EXPECT_EQ(Shell_options::Quiet_start::NOT_SET, options.quiet_start);
   EXPECT_FALSE(options.show_column_type_info);
   EXPECT_TRUE(options.compress.empty());
@@ -731,13 +745,23 @@ TEST_F(Shell_cmdline_options, app) {
   test_option_with_no_value("-C", "compress", "REQUIRED");
   test_option_with_no_value("--compress", "compress", "REQUIRED");
   test_option_with_no_value("-p", "prompt_password", "1");
+  test_option_with_no_value("--password", "prompt_password", "1");
+  test_option_with_no_value("--password1", "prompt_password", "1");
 
   test_option_equal_value("dbpassword", "mypwd", IS_CONNECTION_DATA,
                           "password");
+  test_option_equal_value("password1", "mypwd", IS_CONNECTION_DATA, "password");
+  test_option_equal_value("password2", "mypwd", IS_CONNECTION_DATA);
+  test_option_equal_value("password3", "mypwd", IS_CONNECTION_DATA);
+
   test_option_space_value("dbpassword", "mypwd", IS_CONNECTION_DATA, "password",
                           "");
   test_option_space_value("dbpassword", "mypwd", IS_CONNECTION_DATA,
                           "prompt_password", "1");
+  test_option_space_value("password1", "mypwd", IS_CONNECTION_DATA,
+                          "prompt_password", "1");
+  test_option_space_value("password2", "mypwd", IS_CONNECTION_DATA);
+  test_option_space_value("password3", "mypwd", IS_CONNECTION_DATA);
   test_short_option_value("dbpassword", "p", "mypwd", IS_CONNECTION_DATA,
                           "password");
   test_short_option_space_value("dbpassword", "p", "mypwd", IS_CONNECTION_DATA,
@@ -1271,11 +1295,10 @@ TEST_F(Shell_cmdline_options, conflicts_compression) {
   char *argv0[] = {const_cast<char *>("ut"), const_cast<char *>("--compress"),
                    const_cast<char *>("--compression-algorithms=uncompressed"),
                    uri, NULL};
-  Shell_options so(4, argv0);
-  EXPECT_EQ(0, so.get().exit_code);
-  EXPECT_THROW_LIKE(so.get().connection_options(), std::invalid_argument,
-                    "Conflicting connection options: compression=REQUIRED, "
-                    "compression-algorithms=uncompressed");
+  test_conflicting_options(
+      "compressoon conflicts", 3, argv0,
+      "Conflicting connection options: compression=REQUIRED, "
+      "compression-algorithms=uncompressed.\n");
 }
 
 TEST_F(Shell_cmdline_options, test_uri_with_password) {
@@ -1294,7 +1317,7 @@ TEST_F(Shell_cmdline_options, test_uri_with_password) {
       std::make_shared<Shell_options>(2, argv);
 
   EXPECT_EQ(0, options->get().exit_code);
-  EXPECT_STREQ(argv[1], "--uri=root:****@localhost:3301");
+  EXPECT_STREQ(argv[1], "--uri=root@localhost:3301");
   EXPECT_STREQ("", cerr.str().c_str());
 
   SCOPED_TRACE("TESTING: single letter user without password in uri");
@@ -1305,7 +1328,7 @@ TEST_F(Shell_cmdline_options, test_uri_with_password) {
   options = std::make_shared<Shell_options>(2, argv1);
 
   EXPECT_EQ(0, options->get().exit_code);
-  EXPECT_STREQ(argv1[1], "--uri=r:@localhost:3301");
+  EXPECT_STREQ(argv1[1], "--uri=r@localhost:3301");
   EXPECT_STREQ("", cerr.str().c_str());
 
   SCOPED_TRACE("TESTING: single letter user with password in uri");
@@ -1316,7 +1339,7 @@ TEST_F(Shell_cmdline_options, test_uri_with_password) {
   options = std::make_shared<Shell_options>(2, argv2);
 
   EXPECT_EQ(0, options->get().exit_code);
-  EXPECT_STREQ(argv2[1], "--uri=r:****@localhost:3301");
+  EXPECT_STREQ(argv2[1], "--uri=r@localhost:3301");
   EXPECT_STREQ("", cerr.str().c_str());
 
   SCOPED_TRACE("TESTING: single letter user with single letter password");
@@ -1327,7 +1350,7 @@ TEST_F(Shell_cmdline_options, test_uri_with_password) {
   options = std::make_shared<Shell_options>(2, argv3);
 
   EXPECT_EQ(0, options->get().exit_code);
-  EXPECT_STREQ(argv3[1], "--uri=r:*@localhost:3301");
+  EXPECT_STREQ(argv3[1], "--uri=r@localhost:3301");
   EXPECT_STREQ("", cerr.str().c_str());
 
   SCOPED_TRACE("TESTING: user with password with special sign encoded in uri");
@@ -1338,7 +1361,7 @@ TEST_F(Shell_cmdline_options, test_uri_with_password) {
   options = std::make_shared<Shell_options>(2, argv4);
 
   EXPECT_EQ(0, options->get().exit_code);
-  EXPECT_STREQ(argv4[1], "--uri=root:******@localhost:3301");
+  EXPECT_STREQ(argv4[1], "--uri=root@localhost:3301");
   EXPECT_STREQ("", cerr.str().c_str());
 
   SCOPED_TRACE(
@@ -1350,7 +1373,7 @@ TEST_F(Shell_cmdline_options, test_uri_with_password) {
   options = std::make_shared<Shell_options>(2, argv5);
 
   EXPECT_EQ(0, options->get().exit_code);
-  EXPECT_STREQ(argv5[1], "root:******@localhost:3301");
+  EXPECT_STREQ(argv5[1], "root@localhost:3301");
   EXPECT_STREQ("", cerr.str().c_str());
 
   // Restore old cerr.
@@ -1557,4 +1580,47 @@ TEST_F(Shell_cmdline_options, test_file_and_execute) {
   }
 }
 
+TEST_F(Shell_cmdline_options, mfa_tests) {
+  {
+    // Password in --password1 differs from password in URI
+    char uri[] = {"--uri=mysqlx://root:password@localhost"};
+    char wrong_pwd1[] = {"--password1=example"};
+    char *argv0[] = {const_cast<char *>("ut"), uri, wrong_pwd1, NULL};
+
+    test_conflicting_options(
+        "Different passwords", 3, argv0,
+        "Conflicting options: provided password differs from the "
+        "password in the URI.\n");
+  }
+  {
+    // Password 1 and X Protocol is OK
+    char pwd1[] = {"--password1=example"};
+    char *argv0[] = {const_cast<char *>("ut"),
+                     const_cast<char *>("mysqlx://root@localhost"), pwd1, NULL};
+    Shell_options so(3, argv0);
+    EXPECT_EQ(0, so.get().exit_code);
+    EXPECT_NO_THROW(so.get().connection_options());
+  }
+  {
+    char *passwords[] = {const_cast<char *>("--password2=whatever"),
+                         const_cast<char *>("--password3=whatever")};
+    char *x_proto_args[] = {const_cast<char *>("--mysqlx"),
+                            const_cast<char *>("--mx"),
+                            const_cast<char *>("--sqlx")};
+
+    // MFA and X Protocol are not OK
+    for (size_t index = 0; index < 2; index++) {
+      for (size_t pindex = 0; pindex < 2; pindex++) {
+        char *argv0[] = {const_cast<char *>("ut"), passwords[pindex],
+                         x_proto_args[index],
+                         const_cast<char *>("root@localhost"), NULL};
+
+        test_conflicting_options("Different passwords", 4, argv0,
+                                 "Multi-factor authentication is only "
+                                 "compatible with MySQL protocol\n");
+      }
+    }
+  }
+}
+
 }  // namespace shcore
