@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -46,9 +46,9 @@ namespace sql_shell_tests {
 class Environment {
  public:
   Environment() : m_console(&output_handler.deleg) {
-    shell_core.reset(new Shell_core(&m_console));
+    shell_core = std::make_shared<Shell_core>();
 
-    shell_sql.reset(new Shell_sql(shell_core.get()));
+    shell_sql = std::make_shared<Shell_sql>(shell_core.get());
   }
 
   ~Environment() {}
@@ -76,7 +76,7 @@ class Shell_sql_test : public Shell_core_test_wrapper {
     }
   }
 
-  void process_sql_result(std::shared_ptr<mysqlshdk::db::IResult>,
+  void process_sql_result(const std::shared_ptr<mysqlshdk::db::IResult> &,
                           const shcore::Sql_result_info &) {}
 
   void handle_input(std::string &query, Input_state &state) {
@@ -394,6 +394,21 @@ TEST_F(Shell_sql_test, multiple_single_double_quotes) {
   EXPECT_EQ("", env.shell_sql->get_continued_input_context());
 }
 
+TEST_F(Shell_sql_test, stmt_inline_comment) {
+  // A multiline comment in the middle of a statement is
+  // not trimmed off
+  Input_state state;
+  std::string query = "SELECT 1 /* This is an inline comment */AS _one;";
+
+  handle_input(query, state);
+  EXPECT_EQ(Input_state::Ok, state);
+  EXPECT_EQ("", query);
+
+  // SQL has not been handled yet
+  EXPECT_EQ("SELECT 1 /* This is an inline comment */AS _one;",
+            env.shell_sql->get_handled_input());
+}
+
 TEST_F(Shell_sql_test, continued_stmt_multiline_comment) {
   // A multiline comment in the middle of a statement is
   // not trimmed off
