@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -86,15 +86,19 @@ class Mock_session : public mysqlshdk::db::ISession,
  public:
   Mock_session();
   MOCK_METHOD1(
-      connect,
+      do_connect,
       void(const mysqlshdk::db::Connection_options &connection_options));
-
   // Execution
   std::shared_ptr<mysqlshdk::db::IResult> querys(const char *sql, size_t len,
                                                  bool buffered) override {
     return Mock_session_common::do_querys(sql, len, buffered);
   }
 
+  std::shared_ptr<mysqlshdk::db::IResult> query_udf(std::string_view sql,
+                                                    bool buffered) override {
+    return Mock_session_common::do_querys(sql.data(), sql.size(), buffered);
+  }
+
   MOCK_METHOD2(executes, void(const char *, size_t));
   MOCK_METHOD1(execute, void(const std::string &));
   MOCK_METHOD0(start_transaction, void());
@@ -112,13 +116,12 @@ class Mock_session : public mysqlshdk::db::ISession,
   // Error handling
   MOCK_CONST_METHOD0(get_last_error, mysqlshdk::db::Error *());
 
-  // Disconnection
-  MOCK_METHOD0(close, void());
-
   Mock_session &expect_query(const std::string &query) {
     Mock_session_common::do_expect_query(query);
     return *this;
   }
+  // Disconnection
+  MOCK_METHOD0(do_close, void());
 
   void expect_query(const Fake_result_data &data) {
     expect_query(data.sql).then_return({data});
