@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -29,8 +29,12 @@
 #include "unittest/gprod_clean.h"
 #include "unittest/gtest_clean.h"
 
-#include "scripting/types.h"
-#include "scripting/types_cpp.h"
+#include "mysqlshdk/include/scripting/obj_date.h"
+#include "mysqlshdk/include/scripting/type_info/custom.h"
+#include "mysqlshdk/include/scripting/type_info/generic.h"
+#include "mysqlshdk/include/scripting/types.h"
+#include "mysqlshdk/include/scripting/types_cpp.h"
+
 #include "unittest/test_utils.h"
 
 namespace shcore {
@@ -154,7 +158,7 @@ class Test_object : public Cpp_object_bridge {
     return "null";
   }
 
-  std::string f_s_o(std::shared_ptr<Test_object> obj) { return "ble"; }
+  std::string f_s_o(std::shared_ptr<Test_object>) { return "ble"; }
 
   std::shared_ptr<Test_object> f_o_o(std::shared_ptr<Test_object> obj) {
     return obj;
@@ -177,23 +181,23 @@ class Test_object : public Cpp_object_bridge {
 
   int f_overload() { return 10; }
 
-  int f_overload(int a) { return 11; }
+  int f_overload(int) { return 11; }
 
-  int f_overload_dup(int a) { return 12; }
+  int f_overload_dup(int) { return 12; }
 
-  int f_overload(int a, int b) { return 13; }
+  int f_overload(int, int) { return 13; }
 
-  int f_overload(const std::string &a) { return 14; }
+  int f_overload(const std::string &) { return 14; }
 
-  int f_overload(shcore::Dictionary_t dict) { return 15; }
+  int f_overload(shcore::Dictionary_t) { return 15; }
 
-  int f_overload(int a, const std::string &b) { return 121; }
+  int f_overload(int, const std::string &) { return 121; }
 
-  float f_overload_bad(int a) { return 16; }
+  float f_overload_bad(int) { return 16; }
 
-  std::string f_overload_bad2(int a, int b, int c) { return "17"; }
+  std::string f_overload_bad2(int, int, int) { return "17"; }
 
-  int f_overload2(int a) { return 18; }
+  int f_overload2(int) { return 18; }
 
   FRIEND_TEST(Types_cpp, arg_check_overload);
   FRIEND_TEST(Types_cpp, arg_check_overload_ambiguous);
@@ -449,4 +453,24 @@ TEST_F(Types_cpp, arg_check_overload_ambiguous) {
   EXPECT_EQ(obj.f_overload(11), obj.call("overload", make_args(11)).as_int());
   EXPECT_EQ(obj.f_overload(0), obj.call("overload", make_args()).as_int());
 }
+
+TEST_F(Types_cpp, time_usec_parsing) {
+  // check correct parsing of decimal seconds as usec values
+
+  EXPECT_EQ(Date::unrepr("00:00:00.0").get_usec(), 0);
+  EXPECT_EQ(Date::unrepr("00:00:00.1").get_usec(), 100000);
+  EXPECT_EQ(Date::unrepr("00:00:00.12").get_usec(), 120000);
+  EXPECT_EQ(Date::unrepr("00:00:00.123").get_usec(), 123000);
+  EXPECT_EQ(Date::unrepr("00:00:00.1234").get_usec(), 123400);
+  EXPECT_EQ(Date::unrepr("00:00:00.12345").get_usec(), 123450);
+  EXPECT_EQ(Date::unrepr("00:00:00.123456").get_usec(), 123456);
+  EXPECT_EQ(Date::unrepr("00:00:00.1234567").get_usec(), 123456);
+  EXPECT_EQ(Date::unrepr("00:00:00.1234561").get_usec(), 123456);
+  EXPECT_EQ(Date::unrepr("00:00:00.12345678").get_usec(), 123456);
+
+  EXPECT_EQ(Date::unrepr("00:00:00.100000").get_usec(), 100000);
+  EXPECT_EQ(Date::unrepr("00:00:00.000100").get_usec(), 100);
+  EXPECT_EQ(Date::unrepr("00:00:00.000001").get_usec(), 1);
+}
+
 }  // namespace shcore
